@@ -1,4 +1,4 @@
-/* $Id: bootp.c $ */
+/* $Id: bootp.c 37746 2011-07-04 06:07:37Z vboxsync $ */
 /** @file
  * NAT - BOOTP/DHCP server emulation.
  */
@@ -255,9 +255,9 @@ static int dhcp_do_ack_offer(PNATState pData, struct mbuf *m, BOOTPClient *bc, i
     Log(("NAT: DHCP: bp_file:%s\n", &rbp->bp_file));
     /* Address/port of the DHCP server. */
     rbp->bp_yiaddr = bc->addr; /* Client IP address */
-    Log(("NAT: DHCP: bp_yiaddr:%R[IP4]\n", &rbp->bp_yiaddr));
+    Log(("NAT: DHCP: bp_yiaddr:%RTnaipv4\n", rbp->bp_yiaddr));
     rbp->bp_siaddr = pData->tftp_server; /* Next Server IP address, i.e. TFTP */
-    Log(("NAT: DHCP: bp_siaddr:%R[IP4]\n", &rbp->bp_siaddr));
+    Log(("NAT: DHCP: bp_siaddr:%RTnaipv4\n", rbp->bp_siaddr));
     if (fDhcpRequest)
     {
         rbp->bp_ciaddr.s_addr = bc->addr.s_addr; /* Client IP address */
@@ -267,7 +267,7 @@ static int dhcp_do_ack_offer(PNATState pData, struct mbuf *m, BOOTPClient *bc, i
 #else
     saddr.s_addr = pData->special_addr.s_addr;
 #endif
-    Log(("NAT: DHCP: s_addr:%R[IP4]\n", &saddr));
+    Log(("NAT: DHCP: s_addr:%RTnaipv4\n", saddr));
 
 #define FILL_BOOTP_EXT(q, tag, len, pvalue)                     \
     do {                                                        \
@@ -454,7 +454,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, const uint8_
             {
                if ((bp->bp_ciaddr.s_addr & RT_H2N_U32(pData->netmask)) != pData->special_addr.s_addr)
                {
-                   LogRel(("NAT: Client %R[IP4] requested IP -- sending NAK\n", &bp->bp_ciaddr));
+                   LogRel(("NAT: Client %RTnaipv4 requested IP -- sending NAK\n", bp->bp_ciaddr));
                    offReply = dhcp_send_nack(pData, bp, bc, m);
                    return offReply;
                }
@@ -478,7 +478,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, const uint8_
             ui32 = *(uint32_t *)(req_ip + 2);
             if ((ui32 & RT_H2N_U32(pData->netmask)) != pData->special_addr.s_addr)
             {
-                LogRel(("NAT: address %R[IP4] has been requested -- sending NAK\n", &ui32));
+                LogRel(("NAT: address %RTnaipv4 has been requested -- sending NAK\n", ui32));
                 offReply = dhcp_send_nack(pData, bp, bc, m);
                 return offReply;
             }
@@ -502,7 +502,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, const uint8_
             break;
     }
 
-    LogRel(("NAT: DHCP offered IP address %R[IP4]\n", &bc->addr));
+    LogRel(("NAT: DHCP offered IP address %RTnaipv4\n", bc->addr));
     offReply = dhcp_send_ack(pData, bp, bc, m, /* fDhcpRequest=*/ 1);
     return offReply;
 }
@@ -529,7 +529,7 @@ static int dhcp_decode_discover(PNATState pData, struct bootp_t *bp, const uint8
         }
 
         bc->xid = bp->bp_xid;
-        LogRel(("NAT: DHCP offered IP address %R[IP4]\n", &bc->addr));
+        LogRel(("NAT: DHCP offered IP address %RTnaipv4\n", bc->addr));
         offReply = dhcp_send_offer(pData, bp, bc, m);
         return offReply;
     }
@@ -542,7 +542,7 @@ static int dhcp_decode_discover(PNATState pData, struct bootp_t *bp, const uint8
             return -1;
         }
 
-        LogRel(("NAT: DHCP offered IP address %R[IP4]\n", &bc->addr));
+        LogRel(("NAT: DHCP offered IP address %RTnaipv4\n", bc->addr));
         offReply = dhcp_send_ack(pData, bp, bc, m, /* fDhcpRequest=*/ 0);
         return offReply;
     }
@@ -553,7 +553,7 @@ static int dhcp_decode_discover(PNATState pData, struct bootp_t *bp, const uint8
 static int dhcp_decode_release(PNATState pData, struct bootp_t *bp, const uint8_t *buf, int size)
 {
     int rc = release_addr(pData, &bp->bp_ciaddr);
-    LogRel(("NAT: %s %R[IP4]\n",
+    LogRel(("NAT: %s %RTnaipv4\n",
             RT_SUCCESS(rc) ? "DHCP released IP address" : "Ignored DHCP release for IP address",
             &bp->bp_ciaddr));
     return 0;
@@ -709,7 +709,7 @@ static void dhcp_decode(PNATState pData, struct bootp_t *bp, const uint8_t *buf,
                 Assert(bc);
                 bc->addr.s_addr = req_ip.s_addr;
                 slirp_arp_who_has(pData, bc->addr.s_addr);
-                LogRel(("NAT: %R[IP4] has been already registered\n", &req_ip));
+                LogRel(("NAT: %RTnaipv4 has been already registered\n", req_ip));
             }
             /* no response required */
             break;
