@@ -1,4 +1,4 @@
-/* $Id: ip_icmp.c $ */
+/* $Id: ip_icmp.c 37936 2011-07-14 03:54:41Z vboxsync $ */
 /** @file
  * NAT - IP/ICMP handling.
  */
@@ -185,7 +185,7 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
     fport = ~0;
 
 
-    Log(("%s: processing (proto:%d)\n", __FUNCTION__, ip->ip_p));
+    LogFlowFunc(("ENTER: ip->ip_p:%d\n", ip->ip_p));
     switch (ip->ip_p)
     {
         case IPPROTO_ICMP:
@@ -262,7 +262,7 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
             for (so = head_socket->so_prev; so != head_socket; so = so->so_prev)
             {
                 /* Should be reaplaced by hash here */
-                Log(("trying:%R[natsock] against %R[IP4]:%d lport=%d hlport=%d\n", so, &faddr, fport, lport, so->so_hlport));
+                Log(("trying:%R[natsock] against %RTnaipv4:%d lport=%d hlport=%d\n", so, &faddr, fport, lport, so->so_hlport));
                 if (   so->so_faddr.s_addr == faddr.s_addr
                     && so->so_fport == fport
                     && so->so_hlport == lport)
@@ -294,11 +294,16 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
          * better add flag if it should removed from lis
          */
         LIST_INSERT_HEAD(&pData->icmp_msg_head, icm, im_list);
+        LogFlowFunc(("LEAVE: icm:%p\n", icm));
         return (icm);
     }
     if (found == 1)
+    {
+        LogFlowFunc(("LEAVE: icm:%p\n", icm));
         return icm;
+    }
 
+    LogFlowFunc(("LEAVE: NULL\n"));
     return NULL;
 }
 
@@ -334,7 +339,7 @@ icmp_input(PNATState pData, struct mbuf *m, int hlen)
 
     /* int code; */
 
-    LogFlow(("icmp_input: m = %lx, m_len = %d\n", (long)m, m ? m->m_len : 0));
+    LogFlowFunc(("ENTER: m = %lx, m_len = %d\n", (long)m, m ? m->m_len : 0));
 
     icmpstat.icps_received++;
 
@@ -536,7 +541,7 @@ done:
  * ICMP fragmentation is illegal.  All machines must accept 576 bytes in one
  * packet.  The maximum payload is 576-20(ip hdr)-8(icmp hdr)=548
  *
- * @note This function will NOT free msrc!
+ * @note This function will free msrc!
  */
 
 #define ICMP_MAXDATALEN (IP_MSS-28)
@@ -590,25 +595,15 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
 
     new_m_size = sizeof(struct ip) + ICMP_MINLEN + msrc->m_len + ICMP_MAXDATALEN;
     if (new_m_size < MSIZE)
-    {
         size = MCLBYTES;
-    }
     else if (new_m_size < MCLBYTES)
-    {
         size = MCLBYTES;
-    }
     else if(new_m_size < MJUM9BYTES)
-    {
         size = MJUM9BYTES;
-    }
     else if (new_m_size < MJUM16BYTES)
-    {
         size = MJUM16BYTES;
-    }
     else
-    {
         AssertMsgFailed(("Unsupported size"));
-    }
     m = m_getjcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR, size);
     if (!m)
         goto end_error;
@@ -677,16 +672,29 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     ip->ip_dst = ip->ip_src;    /* ip adresses */
     ip->ip_src = alias_addr;
 
+    /* returns pointer back. */
+    m->m_data -= hlen;
+    m->m_len  += hlen;
     (void) ip_output0(pData, (struct socket *)NULL, m, 1);
 
     icmpstat.icps_reflect++;
 
+    /* clear source datagramm in positive branch */
+    m_freem(pData, msrc);
+    LogFlowFuncLeave();
     return;
 
 end_error_free_m:
     m_freem(pData, m);
 
 end_error:
+
+    /*
+     * clear source datagramm in case if some of requirement haven't been met.
+     */
+    if (!msrc)
+        m_freem(pData, msrc);
+
     {
         static bool fIcmpErrorReported;
         if (!fIcmpErrorReported)
@@ -695,6 +703,7 @@ end_error:
             fIcmpErrorReported = true;
         }
     }
+    LogFlowFuncLeave();
 }
 #undef ICMP_MAXDATALEN
 
@@ -709,6 +718,7 @@ icmp_reflect(PNATState pData, struct mbuf *m)
     int hlen = ip->ip_hl << 2;
     int optlen = hlen - sizeof(struct ip);
     register struct icmp *icp;
+    LogFlowFunc(("ENTER: m:%p\n", m));
 
     /*
      * Send an icmp packet back to the ip level,
@@ -727,4 +737,5 @@ icmp_reflect(PNATState pData, struct mbuf *m)
     (void) ip_output(pData, (struct socket *)NULL, m);
 
     icmpstat.icps_reflect++;
+    LogFlowFuncLeave();
 }
