@@ -1,4 +1,4 @@
-/* $Id: tcp_input.c $ */
+/* $Id: tcp_input.c 37936 2011-07-14 03:54:41Z vboxsync $ */
 /** @file
  * NAT - TCP input.
  */
@@ -98,6 +98,7 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
     struct socket *so = tp->t_socket;
     int flags;
     STAM_PROFILE_START(&pData->StatTCP_reassamble, tcp_reassamble);
+    LogFlowFunc(("ENTER: pData:%p, tp:%R[tcpcb793], th:%p, tlenp:%p, m:%p\n", pData, tp, th, tlenp, m));
 
     /*
      * XXX: tcp_reass() is rather inefficient with its data structures
@@ -110,7 +111,10 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
      * force pre-ESTABLISHED data up to user socket.
      */
     if (th == NULL)
+    {
+        LogFlowFunc(("%d -> present\n", __LINE__));
         goto present;
+    }
 
     /*
      * Limit the number of segments in the reassembly queue to prevent
@@ -128,6 +132,7 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
         m_freem(pData, m);
         *tlenp = 0;
         STAM_PROFILE_STOP(&pData->StatTCP_reassamble, tcp_reassamble);
+        LogFlowFuncLeave();
         return (0);
     }
 
@@ -142,6 +147,7 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
         m_freem(pData, m);
         *tlenp = 0;
         STAM_PROFILE_STOP(&pData->StatTCP_reassamble, tcp_reassamble);
+        LogFlowFuncLeave();
         return (0);
     }
     tp->t_segqlen++;
@@ -183,6 +189,7 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
                  * This is needed after the 3-WHS
                  * completes.
                  */
+                LogFlowFunc(("%d -> present\n", __LINE__));
                 goto present;   /* ??? */
             }
             m_adj(m, i);
@@ -325,12 +332,14 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
             /* mbuf should be cleared in sofree called from tcp_close */
             tcp_close(pData, tp);
             STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
+            LogFlowFuncLeave();
             return;
         }
 
         tiwin = ti->ti_win;
         tiflags = ti->ti_flags;
 
+        LogFlowFunc(("%d -> cont_conn\n", __LINE__));
         goto cont_conn;
     }
 
@@ -374,6 +383,7 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
     if (cksum(m, len))
     {
         tcpstat.tcps_rcvbadsum++;
+        LogFlowFunc(("%d -> drop\n", __LINE__));
         goto drop;
     }
 
@@ -386,6 +396,7 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
         || off > tlen)
     {
         tcpstat.tcps_rcvbadoff++;
+        LogFlowFunc(("%d -> drop\n", __LINE__));
         goto drop;
     }
     tlen -= off;
@@ -436,6 +447,7 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
      * Locate pcb for segment.
      */
 findso:
+    LogFlowFunc(("(enter) findso: %R[natsock]\n", so));
     if (so != NULL && so != &tcb)
         SOCKET_UNLOCK(so);
     QSOCKET_LOCK(tcb);
@@ -482,6 +494,7 @@ findso:
         SOCKET_LOCK(so);
         QSOCKET_UNLOCK(tcb);
     }
+    LogFlowFunc(("(leave) findso: %R[natsock]\n", so));
 
     /*
      * If the state is CLOSED (i.e., TCB does not exist) then
@@ -499,13 +512,20 @@ findso:
     if (so == 0)
     {
         if ((tiflags & (TH_SYN|TH_FIN|TH_RST|TH_URG|TH_ACK)) != TH_SYN)
+        {
+            LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
             goto dropwithreset;
+        }
 
         if ((so = socreate()) == NULL)
+        {
+            LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
             goto dropwithreset;
+        }
         if (tcp_attach(pData, so) < 0)
         {
             RTMemFree(so); /* Not sofree (if it failed, it's not insqued) */
+            LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
             goto dropwithreset;
         }
         SOCKET_LOCK(so);
@@ -528,7 +548,7 @@ findso:
         so->so_iptos = ((struct ip *)ti)->ip_tos;
 
         tp = sototcpcb(so);
-        tp->t_state = TCPS_LISTEN;
+        TCP_STATE_SWITCH_TO(tp, TCPS_LISTEN);
     }
 
     /*
@@ -538,6 +558,7 @@ findso:
      */
     if (so->so_state & SS_ISFCONNECTING)
     {
+        LogFlowFunc(("%d -> drop\n", __LINE__));
         goto drop;
     }
 
@@ -545,9 +566,13 @@ findso:
 
     /* XXX Should never fail */
     if (tp == 0)
+    {
+        LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
         goto dropwithreset;
+    }
     if (tp->t_state == TCPS_CLOSED)
     {
+        LogFlowFunc(("%d -> drop\n", __LINE__));
         goto drop;
     }
 
@@ -755,11 +780,20 @@ findso:
         case TCPS_LISTEN:
         {
             if (tiflags & TH_RST)
+            {
+                LogFlowFunc(("%d -> drop\n", __LINE__));
                 goto drop;
+            }
             if (tiflags & TH_ACK)
+            {
+                LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
                 goto dropwithreset;
+            }
             if ((tiflags & TH_SYN) == 0)
+            {
+                LogFlowFunc(("%d -> drop\n", __LINE__));
                 goto drop;
+            }
 
             /*
              * This has way too many gotos...
@@ -789,7 +823,6 @@ findso:
                     m->m_len  += sizeof(struct tcpiphdr)+off-sizeof(struct tcphdr);
                     *ip = save_ip;
                     icmp_error(pData, m, ICMP_UNREACH, code, 0, strerror(errno));
-                    m_freem(pData, m);
                     tp->t_socket->so_m = NULL;
                 }
                 tp = tcp_close(pData, tp);
@@ -805,22 +838,26 @@ findso:
                 so->so_m = m;
                 so->so_ti = ti;
                 tp->t_timer[TCPT_KEEP] = TCPTV_KEEP_INIT;
-                tp->t_state = TCPS_SYN_RECEIVED;
+                TCP_STATE_SWITCH_TO(tp, TCPS_SYN_RECEIVED);
             }
             SOCKET_UNLOCK(so);
             STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
+            LogFlowFuncLeave();
             return;
 
 cont_conn:
             /* m==NULL
              * Check if the connect succeeded
              */
+            LogFlowFunc(("cont_conn:\n"));
             if (so->so_state & SS_NOFDREF)
             {
                 tp = tcp_close(pData, tp);
+                LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
                 goto dropwithreset;
             }
 cont_input:
+            LogFlowFunc(("cont_input:\n"));
             tcp_template(tp);
 
             if (optp)
@@ -835,9 +872,10 @@ cont_input:
             tcp_sendseqinit(tp);
             tcp_rcvseqinit(tp);
             tp->t_flags |= TF_ACKNOW;
-            tp->t_state = TCPS_SYN_RECEIVED;
+            TCP_STATE_SWITCH_TO(tp, TCPS_SYN_RECEIVED);
             tp->t_timer[TCPT_KEEP] = TCPTV_KEEP_INIT;
             tcpstat.tcps_accepts++;
+            LogFlowFunc(("%d -> trimthenstep6\n", __LINE__));
             goto trimthenstep6;
         } /* case TCPS_LISTEN */
 
@@ -857,17 +895,22 @@ cont_input:
             if (   (tiflags & TH_ACK)
                 && (   SEQ_LEQ(ti->ti_ack, tp->iss)
                     || SEQ_GT(ti->ti_ack, tp->snd_max)))
+            {
+                LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
                 goto dropwithreset;
+            }
 
             if (tiflags & TH_RST)
             {
                 if (tiflags & TH_ACK)
                     tp = tcp_drop(pData, tp, 0); /* XXX Check t_softerror! */
+                LogFlowFunc(("%d -> drop\n", __LINE__));
                 goto drop;
             }
 
             if ((tiflags & TH_SYN) == 0)
             {
+                LogFlowFunc(("%d -> drop\n", __LINE__));
                 goto drop;
             }
             if (tiflags & TH_ACK)
@@ -885,7 +928,7 @@ cont_input:
             {
                 tcpstat.tcps_connects++;
                 soisfconnected(so);
-                tp->t_state = TCPS_ESTABLISHED;
+                TCP_STATE_SWITCH_TO(tp, TCPS_ESTABLISHED);
 
                 /* Do window scaling on this connection? */
 #if 0
@@ -905,9 +948,10 @@ cont_input:
                     tcp_xmit_timer(pData, tp, tp->t_rtt);
             }
             else
-                tp->t_state = TCPS_SYN_RECEIVED;
+                TCP_STATE_SWITCH_TO(tp, TCPS_SYN_RECEIVED);
 
 trimthenstep6:
+            LogFlowFunc(("trimthenstep6:\n"));
             /*
              * Advance ti->ti_seq to correspond to first data byte.
              * If data, trim to stay within window,
@@ -925,7 +969,7 @@ trimthenstep6:
             }
             tp->snd_wl1 = ti->ti_seq - 1;
             tp->rcv_up = ti->ti_seq;
-            Log2(("hit6\n"));
+            LogFlowFunc(("%d -> step6\n", __LINE__));
             goto step6;
     } /* switch tp->t_state */
     /*
@@ -1031,6 +1075,7 @@ trimthenstep6:
     {
         tp = tcp_close(pData, tp);
         tcpstat.tcps_rcvafterclose++;
+        LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
         goto dropwithreset;
     }
 
@@ -1058,6 +1103,7 @@ trimthenstep6:
                 iss = tp->rcv_nxt + TCP_ISSINCR;
                 tp = tcp_close(pData, tp);
                 SOCKET_UNLOCK(tp->t_socket);
+                LogFlowFunc(("%d -> findso\n", __LINE__));
                 goto findso;
             }
             /*
@@ -1073,7 +1119,10 @@ trimthenstep6:
                 tcpstat.tcps_rcvwinprobe++;
             }
             else
+            {
+                LogFlowFunc(("%d -> dropafterack\n", __LINE__));
                 goto dropafterack;
+            }
         }
         else
             tcpstat.tcps_rcvbyteafterwin += todrop;
@@ -1111,6 +1160,7 @@ trimthenstep6:
         {
             case TCPS_SYN_RECEIVED:
 /*              so->so_error = ECONNREFUSED; */
+                LogFlowFunc(("%d -> close\n", __LINE__));
                 goto close;
 
             case TCPS_ESTABLISHED:
@@ -1119,16 +1169,19 @@ trimthenstep6:
             case TCPS_CLOSE_WAIT:
 /*              so->so_error = ECONNRESET; */
 close:
-                tp->t_state = TCPS_CLOSED;
-                tcpstat.tcps_drops++;
-                tp = tcp_close(pData, tp);
-                goto drop;
+            LogFlowFunc(("close:\n"));
+            TCP_STATE_SWITCH_TO(tp, TCPS_CLOSED);
+            tcpstat.tcps_drops++;
+            tp = tcp_close(pData, tp);
+            LogFlowFunc(("%d -> drop\n", __LINE__));
+            goto drop;
 
             case TCPS_CLOSING:
             case TCPS_LAST_ACK:
             case TCPS_TIME_WAIT:
-                tp = tcp_close(pData, tp);
-                goto drop;
+            tp = tcp_close(pData, tp);
+            LogFlowFunc(("%d -> drop\n", __LINE__));
+            goto drop;
         }
 
     /*
@@ -1138,6 +1191,7 @@ close:
     if (tiflags & TH_SYN)
     {
         tp = tcp_drop(pData, tp, 0);
+        LogFlowFunc(("%d -> dropwithreset\n", __LINE__));
         goto dropwithreset;
     }
 
@@ -1146,6 +1200,7 @@ close:
      */
     if ((tiflags & TH_ACK) == 0)
     {
+        LogFlowFunc(("%d -> drop\n", __LINE__));
         goto drop;
     }
 
@@ -1164,7 +1219,7 @@ close:
                 || SEQ_GT(ti->ti_ack, tp->snd_max))
                 goto dropwithreset;
             tcpstat.tcps_connects++;
-            tp->t_state = TCPS_ESTABLISHED;
+            TCP_STATE_SWITCH_TO(tp, TCPS_ESTABLISHED);
             /*
              * The sent SYN is ack'ed with our sequence number +1
              * The first data byte already in the buffer will get
@@ -1187,6 +1242,7 @@ close:
             (void) tcp_reass(pData, tp, (struct tcphdr *)0, (int *)0, (struct mbuf *)0);
             tp->snd_wl1 = ti->ti_seq - 1;
             /* Avoid ack processing; snd_una==ti_ack  =>  dup ack */
+            LogFlowFunc(("%d -> synrx_to_est\n", __LINE__));
             goto synrx_to_est;
             /* fall into ... */
 
@@ -1254,12 +1310,14 @@ close:
                             tp->t_maxseg * tp->t_dupacks;
                         if (SEQ_GT(onxt, tp->snd_nxt))
                             tp->snd_nxt = onxt;
+                        LogFlowFunc(("%d -> drop\n", __LINE__));
                         goto drop;
                     }
                     else if (tp->t_dupacks > tcprexmtthresh)
                     {
                         tp->snd_cwnd += tp->t_maxseg;
                         (void) tcp_output(pData, tp);
+                        LogFlowFunc(("%d -> drop\n", __LINE__));
                         goto drop;
                     }
                 }
@@ -1268,6 +1326,7 @@ close:
                 break;
             }
 synrx_to_est:
+            LogFlowFunc(("synrx_to_est:\n"));
             /*
              * If the congestion window was inflated to account
              * for the other side's cached packets, retract it.
@@ -1279,6 +1338,7 @@ synrx_to_est:
             if (SEQ_GT(ti->ti_ack, tp->snd_max))
             {
                 tcpstat.tcps_rcvacktoomuch++;
+                LogFlowFunc(("%d -> dropafterack\n", __LINE__));
                 goto dropafterack;
             }
             acked = ti->ti_ack - tp->snd_una;
@@ -1384,7 +1444,7 @@ synrx_to_est:
                             soisfdisconnected(so);
                             tp->t_timer[TCPT_2MSL] = tcp_maxidle;
                         }
-                        tp->t_state = TCPS_FIN_WAIT_2;
+                        TCP_STATE_SWITCH_TO(tp, TCPS_FIN_WAIT_2);
                     }
                     break;
 
@@ -1397,7 +1457,7 @@ synrx_to_est:
                 case TCPS_CLOSING:
                     if (ourfinisacked)
                     {
-                        tp->t_state = TCPS_TIME_WAIT;
+                        TCP_STATE_SWITCH_TO(tp, TCPS_TIME_WAIT);
                         tcp_canceltimers(tp);
                         tp->t_timer[TCPT_2MSL] = 2 * TCPTV_MSL;
                         soisfdisconnected(so);
@@ -1414,6 +1474,7 @@ synrx_to_est:
                     if (ourfinisacked)
                     {
                         tp = tcp_close(pData, tp);
+                        LogFlowFunc(("%d -> drop\n", __LINE__));
                         goto drop;
                     }
                     break;
@@ -1425,11 +1486,13 @@ synrx_to_est:
                  */
                 case TCPS_TIME_WAIT:
                     tp->t_timer[TCPT_2MSL] = 2 * TCPTV_MSL;
+                    LogFlowFunc(("%d -> dropafterack\n", __LINE__));
                     goto dropafterack;
             }
     } /* switch(tp->t_state) */
 
 step6:
+    LogFlowFunc(("step6:\n"));
     /*
      * Update window information.
      * Don't look at window if no ACK: TAC's send garbage on first SYN.
@@ -1472,6 +1535,7 @@ step6:
         {
             ti->ti_urp = 0;
             tiflags &= ~TH_URG;
+            LogFlowFunc(("%d -> dodata\n", __LINE__));
             goto dodata;
         }
 #endif
@@ -1506,6 +1570,7 @@ step6:
         if (SEQ_GT(tp->rcv_nxt, tp->rcv_up))
             tp->rcv_up = tp->rcv_nxt;
 dodata:
+    LogFlowFunc(("dodata:\n"));
 
     /*
      * If this is a small packet, then ACK now - with Nagel
@@ -1595,7 +1660,7 @@ dodata:
              */
             case TCPS_SYN_RECEIVED:
             case TCPS_ESTABLISHED:
-                tp->t_state = TCPS_CLOSE_WAIT;
+                TCP_STATE_SWITCH_TO(tp, TCPS_CLOSE_WAIT);
                 break;
 
             /*
@@ -1603,7 +1668,7 @@ dodata:
              * enter the CLOSING state.
              */
             case TCPS_FIN_WAIT_1:
-                tp->t_state = TCPS_CLOSING;
+                TCP_STATE_SWITCH_TO(tp, TCPS_CLOSING);
                 break;
 
             /*
@@ -1612,7 +1677,7 @@ dodata:
              * standard timers.
              */
             case TCPS_FIN_WAIT_2:
-                tp->t_state = TCPS_TIME_WAIT;
+                TCP_STATE_SWITCH_TO(tp, TCPS_TIME_WAIT);
                 tcp_canceltimers(tp);
                 tp->t_timer[TCPT_2MSL] = 2 * TCPTV_MSL;
                 soisfdisconnected(so);
@@ -1635,24 +1700,30 @@ dodata:
 
     SOCKET_UNLOCK(so);
     STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
+    LogFlowFuncLeave();
     return;
 
 dropafterack:
-    Log2(("drop after ack\n"));
+    LogFlowFunc(("dropafterack:\n"));
     /*
      * Generate an ACK dropping incoming segment if it occupies
      * sequence space, where the ACK reflects our state.
      */
     if (tiflags & TH_RST)
+    {
+        LogFlowFunc(("%d -> drop\n", __LINE__));
         goto drop;
+    }
     m_freem(pData, m);
     tp->t_flags |= TF_ACKNOW;
     (void) tcp_output(pData, tp);
     SOCKET_UNLOCK(so);
     STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
+    LogFlowFuncLeave();
     return;
 
 dropwithreset:
+    LogFlowFunc(("dropwithreset:\n"));
     /* reuses m if m!=NULL, m_free() unnecessary */
     if (tiflags & TH_ACK)
         tcp_respond(pData, tp, ti, m, (tcp_seq)0, ti->ti_ack, TH_RST);
@@ -1667,9 +1738,11 @@ dropwithreset:
     if (so != &tcb)
         SOCKET_UNLOCK(so);
     STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
+    LogFlowFuncLeave();
     return;
 
 drop:
+    LogFlowFunc(("drop:\n"));
     /*
      * Drop space held by incoming segment and return.
      */
@@ -1683,6 +1756,7 @@ drop:
 #endif
 
     STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
+    LogFlowFuncLeave();
     return;
 }
 
@@ -1692,7 +1766,7 @@ tcp_dooptions(PNATState pData, struct tcpcb *tp, u_char *cp, int cnt, struct tcp
     u_int16_t mss;
     int opt, optlen;
 
-    LogFlow(("tcp_dooptions: tp = %lx, cnt=%i\n", (long)tp, cnt));
+    LogFlowFunc(("tcp_dooptions: tp = %R[tcpcb793], cnt=%i\n", tp, cnt));
 
     for (; cnt > 0; cnt -= optlen, cp += optlen)
     {
@@ -1803,7 +1877,7 @@ tcp_xmit_timer(PNATState pData, register struct tcpcb *tp, int rtt)
 {
     register short delta;
 
-    LogFlow(("tcp_xmit_timer: tp = %lx rtt = %d\n", (long)tp, rtt));
+    LogFlowFunc(("ENTER: tcp_xmit_timer: tp = %R[tcpcb793] rtt = %d\n", tp, rtt));
 
     tcpstat.tcps_rttupdated++;
     if (tp->t_srtt != 0)
@@ -1893,7 +1967,7 @@ tcp_mss(PNATState pData, register struct tcpcb *tp, u_int offer)
     struct socket *so = tp->t_socket;
     int mss;
 
-    LogFlow(("tcp_mss: tp = %lx, offet = %d\n", (long)tp, offer));
+    LogFlowFunc(("ENTER: tcp_mss: tp = %R[tcpcb793], offer = %d\n", tp, offer));
 
     mss = min(if_mtu, if_mru) - sizeof(struct tcpiphdr);
     if (offer)
