@@ -1,4 +1,4 @@
-/* $Id: DrvVD.cpp $ */
+/* $Id: DrvVD.cpp 37641 2011-06-26 17:16:35Z vboxsync $ */
 /** @file
  * DrvVD - Generic VBox disk media driver.
  */
@@ -190,6 +190,8 @@ typedef struct VBOXDISK
     uint8_t            *pbData;
     /** Bandwidth group the disk is assigned to. */
     char               *pszBwGroup;
+    /** Flag whether async I/O using the host cache is enabled. */
+    bool                fAsyncIoWithHostCache;
 
     /** I/O interface for a cache image. */
     VDINTERFACE         VDIIOCache;
@@ -318,7 +320,7 @@ static DECLCALLBACK(void) drvvdAsyncTaskCompleted(PPDMDRVINS pDrvIns, void *pvTe
     PVBOXDISK pThis = PDMINS_2_DATA(pDrvIns, PVBOXDISK);
     PDRVVDSTORAGEBACKEND pStorageBackend = (PDRVVDSTORAGEBACKEND)pvTemplateUser;
 
-    LogFlowFunc(("pDrvIns=%#p pvTemplateUser=%#p pvUser=%#p rcReq\n",
+    LogFlowFunc(("pDrvIns=%#p pvTemplateUser=%#p pvUser=%#p rcReq=%d\n",
                  pDrvIns, pvTemplateUser, pvUser, rcReq));
 
     if (pStorageBackend->fSyncIoPending)
@@ -371,6 +373,8 @@ static DECLCALLBACK(int) drvvdAsyncIOOpen(void *pvUser, const char *pszLocation,
 
                     fFlags |= PDMACEP_FILE_FLAGS_DONT_LOCK;
                 }
+                if (pThis->fAsyncIoWithHostCache)
+                    fFlags |= PDMACEP_FILE_FLAGS_HOST_CACHE_ENABLED;
 
                 rc = PDMR3AsyncCompletionEpCreateForFile(&pStorageBackend->pEndpoint,
                                                          pszLocation, fFlags,
@@ -2379,6 +2383,16 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
 #endif /* VBOX_WITH_INIP */
         }
 
+        /*
+         * The image has a bandwidth group but the host cache is enabled.
+         * Use the async I/O framework but tell it to enable the host cache.
+         */
+        if (!fUseNewIo && pThis->pszBwGroup)
+        {
+            pThis->fAsyncIoWithHostCache = true;
+            fUseNewIo = true;
+        }
+
         /** @todo quick hack to work around problems in the async I/O
          * implementation (rw semaphore thread ownership problem)
          * while a merge is running. Remove once this is fixed. */
