@@ -1,4 +1,4 @@
-/* $Id: VBoxManage.cpp $ */
+/* $Id: VBoxManage.cpp 38417 2011-08-11 13:58:34Z vboxsync $ */
 /** @file
  * VBoxManage - VirtualBox's command-line interface.
  */
@@ -229,13 +229,13 @@ HRESULT showProgress(ComPtr<IProgress> progress)
     return hrc;
 }
 
-#endif /* !VBOX_ONLY_DOCS */
-
-#ifdef RT_OS_WINDOWS 
+#ifdef RT_OS_WINDOWS
 // Required for ATL
 static CComModule _Module;
 #endif
 
+#endif /* !VBOX_ONLY_DOCS */
+
 
 int main(int argc, char *argv[])
 {
@@ -405,6 +405,7 @@ int main(int argc, char *argv[])
             { "showvminfo",       USAGE_SHOWVMINFO,        handleShowVMInfo },
             { "registervm",       USAGE_REGISTERVM,        handleRegisterVM },
             { "unregistervm",     USAGE_UNREGISTERVM,      handleUnregisterVM },
+            { "clonevm",          USAGE_CLONEVM,           handleCloneVM },
             { "createhd",         USAGE_CREATEHD,          handleCreateHardDisk },
             { "createvdi",        USAGE_CREATEHD,          handleCreateHardDisk }, /* backward compatibility */
             { "modifyhd",         USAGE_MODIFYHD,          handleModifyHardDisk },
@@ -469,7 +470,19 @@ int main(int argc, char *argv[])
             }
         }
         if (!s_commandHandlers[commandIndex].command)
-            rcExit = errorSyntax(USAGE_ALL, "Invalid command '%s'", Utf8Str(argv[iCmd]).c_str());
+        {
+            /* Help topics. */
+            if (fShowHelp && !strcmp(argv[iCmd], "commands"))
+            {
+                RTPrintf("commands:\n");
+                for (unsigned i = 0; i < RT_ELEMENTS(s_commandHandlers) - 1; i++)
+                    if (   i ==  0  /* skip backwards compatibility entries */
+                        || s_commandHandlers[i].help != s_commandHandlers[i - 1].help)
+                        RTPrintf("    %s\n", s_commandHandlers[i].command);
+            }
+            else
+                rcExit = errorSyntax(USAGE_ALL, "Invalid command '%s'", Utf8Str(argv[iCmd]).c_str());
+        }
 
         /* Although all handlers should always close the session if they open it,
          * we do it here just in case if some of the handlers contains a bug --
