@@ -1,10 +1,10 @@
-/* $Id: VBoxManageModifyVM.cpp $ */
+/* $Id: VBoxManageModifyVM.cpp 37817 2011-07-07 13:02:40Z vboxsync $ */
 /** @file
  * VBoxManage - Implementation of modifyvm command.
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -100,18 +100,19 @@ enum
     MODIFYVM_FLOPPY,             // deprecated
     MODIFYVM_NICTRACEFILE,
     MODIFYVM_NICTRACE,
+    MODIFYVM_NICPROPERTY,
     MODIFYVM_NICTYPE,
     MODIFYVM_NICSPEED,
     MODIFYVM_NICBOOTPRIO,
+    MODIFYVM_NICPROMISC,
+    MODIFYVM_NICBWGROUP,
     MODIFYVM_NIC,
     MODIFYVM_CABLECONNECTED,
     MODIFYVM_BRIDGEADAPTER,
     MODIFYVM_HOSTONLYADAPTER,
     MODIFYVM_INTNET,
     MODIFYVM_NATNET,
-#ifdef VBOX_WITH_VDE
-    MODIFYVM_VDENET,
-#endif
+    MODIFYVM_GENERICDRV,
     MODIFYVM_NATBINDIP,
     MODIFYVM_NATSETTINGS,
     MODIFYVM_NATPF,
@@ -168,6 +169,10 @@ enum
     MODIFYVM_FAULT_TOLERANCE_PASSWORD,
     MODIFYVM_FAULT_TOLERANCE_SYNC_INTERVAL,
     MODIFYVM_CPU_EXECTUION_CAP,
+#ifdef VBOX_WITH_PCI_PASSTHROUGH
+    MODIFYVM_ATTACH_PCI,
+    MODIFYVM_DETACH_PCI,
+#endif
     MODIFYVM_CHIPSET
 };
 
@@ -226,18 +231,19 @@ static const RTGETOPTDEF g_aModifyVMOptions[] =
     { "--floppy",                   MODIFYVM_FLOPPY,                    RTGETOPT_REQ_STRING },
     { "--nictracefile",             MODIFYVM_NICTRACEFILE,              RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--nictrace",                 MODIFYVM_NICTRACE,                  RTGETOPT_REQ_BOOL_ONOFF | RTGETOPT_FLAG_INDEX },
+    { "--nicproperty",              MODIFYVM_NICPROPERTY,               RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--nictype",                  MODIFYVM_NICTYPE,                   RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--nicspeed",                 MODIFYVM_NICSPEED,                  RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_INDEX },
     { "--nicbootprio",              MODIFYVM_NICBOOTPRIO,               RTGETOPT_REQ_UINT32 | RTGETOPT_FLAG_INDEX },
+    { "--nicpromisc",               MODIFYVM_NICPROMISC,                RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
+    { "--nicbandwidthgroup",        MODIFYVM_NICBWGROUP,                RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--nic",                      MODIFYVM_NIC,                       RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--cableconnected",           MODIFYVM_CABLECONNECTED,            RTGETOPT_REQ_BOOL_ONOFF | RTGETOPT_FLAG_INDEX },
     { "--bridgeadapter",            MODIFYVM_BRIDGEADAPTER,             RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--hostonlyadapter",          MODIFYVM_HOSTONLYADAPTER,           RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--intnet",                   MODIFYVM_INTNET,                    RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--natnet",                   MODIFYVM_NATNET,                    RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
-#ifdef VBOX_WITH_VDE
-    { "--vdenet",                   MODIFYVM_VDENET,                    RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
-#endif
+    { "--nicgenericdrv",            MODIFYVM_GENERICDRV,                RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--natbindip",                MODIFYVM_NATBINDIP,                 RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--natsettings",              MODIFYVM_NATSETTINGS,               RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
     { "--natpf",                    MODIFYVM_NATPF,                     RTGETOPT_REQ_STRING | RTGETOPT_FLAG_INDEX },
@@ -294,6 +300,10 @@ static const RTGETOPTDEF g_aModifyVMOptions[] =
     { "--faulttolerancepassword",   MODIFYVM_FAULT_TOLERANCE_PASSWORD,  RTGETOPT_REQ_STRING },
     { "--faulttolerancesyncinterval", MODIFYVM_FAULT_TOLERANCE_SYNC_INTERVAL, RTGETOPT_REQ_UINT32 },
     { "--chipset",                  MODIFYVM_CHIPSET,                   RTGETOPT_REQ_STRING },
+#ifdef VBOX_WITH_PCI_PASSTHROUGH
+    { "--pciattach",                MODIFYVM_ATTACH_PCI,                RTGETOPT_REQ_STRING },
+    { "--pcidetach",                MODIFYVM_DETACH_PCI,                RTGETOPT_REQ_STRING },
+#endif
 };
 
 static void vrdeWarningDeprecatedOption(const char *pszOption)
@@ -301,6 +311,27 @@ static void vrdeWarningDeprecatedOption(const char *pszOption)
     RTStrmPrintf(g_pStdErr, "Warning: '--vrdp%s' is deprecated. Use '--vrde%s'.\n", pszOption, pszOption);
 }
 
+/** Parse PCI address in format 01:02.03 and convert it to the numeric representation. */
+static int32_t parsePci(const char* szPciAddr)
+{
+    char* pszNext = (char*)szPciAddr;
+    int rc;
+    uint8_t aVals[3] = {0, 0, 0};
+
+    rc = RTStrToUInt8Ex(pszNext, &pszNext, 16, &aVals[0]);
+    if (RT_FAILURE(rc) || pszNext == NULL || *pszNext != ':')
+        return -1;
+
+    rc = RTStrToUInt8Ex(pszNext+1, &pszNext, 16, &aVals[1]);
+    if (RT_FAILURE(rc) || pszNext == NULL || *pszNext != '.')
+        return -1;
+
+    rc = RTStrToUInt8Ex(pszNext+1, &pszNext, 16, &aVals[2]);
+    if (RT_FAILURE(rc) || pszNext == NULL)
+        return -1;
+
+    return (aVals[0] << 8) | (aVals[1] << 3) | (aVals[2] << 0);
+}
 
 int handleModifyVM(HandlerArg *a)
 {
@@ -317,13 +348,6 @@ int handleModifyVM(HandlerArg *a)
     if (a->argc < 2)
         return errorSyntax(USAGE_MODIFYVM, "Not enough parameters");
 
-    /* Get the number of network adapters */
-    ULONG NetworkAdapterCount = 0;
-    {
-        ComPtr <ISystemProperties> info;
-        CHECK_ERROR_RET(a->virtualBox, COMGETTER(SystemProperties)(info.asOutParam()), 1);
-        CHECK_ERROR_RET(info, COMGETTER(NetworkAdapterCount)(&NetworkAdapterCount), 1);
-    }
     ULONG SerialPortCount = 0;
     {
         ComPtr <ISystemProperties> info;
@@ -335,6 +359,10 @@ int handleModifyVM(HandlerArg *a)
     CHECK_ERROR_RET(a->virtualBox, FindMachine(Bstr(a->argv[0]).raw(),
                                                machine.asOutParam()), 1);
 
+
+    /* Get the number of network adapters */
+    ULONG NetworkAdapterCount = getMaxNics(a->virtualBox, machine);
+
     /* open a session for the VM */
     CHECK_ERROR_RET(machine, LockMachine(a->session, LockType_Write), 1);
 
@@ -683,7 +711,8 @@ int handleModifyVM(HandlerArg *a)
                 {
                     ComPtr<IMedium> hardDisk;
                     rc = findOrOpenMedium(a, ValueUnion.psz, DeviceType_HardDisk,
-                                          hardDisk, NULL);
+                                          hardDisk, false /* fForceNewUuidOnOpen */,
+                                          NULL);
                     if (FAILED(rc))
                         break;
                     if (hardDisk)
@@ -778,7 +807,8 @@ int handleModifyVM(HandlerArg *a)
                 {
                     ComPtr<IMedium> hardDisk;
                     rc = findOrOpenMedium(a, ValueUnion.psz, DeviceType_HardDisk,
-                                          hardDisk, NULL);
+                                          hardDisk, false /* fForceNewUuidOnOpen */,
+                                          NULL);
                     if (FAILED(rc))
                         break;
                     if (hardDisk)
@@ -905,7 +935,8 @@ int handleModifyVM(HandlerArg *a)
                 else
                 {
                     rc = findOrOpenMedium(a, ValueUnion.psz, DeviceType_DVD,
-                                          dvdMedium, NULL);
+                                          dvdMedium, false /* fForceNewUuidOnOpen */,
+                                          NULL);
                     if (FAILED(rc))
                         break;
                     if (!dvdMedium)
@@ -968,7 +999,8 @@ int handleModifyVM(HandlerArg *a)
                     else
                     {
                         rc = findOrOpenMedium(a, ValueUnion.psz, DeviceType_Floppy,
-                                              floppyMedium, NULL);
+                                              floppyMedium, false /* fForceNewUuidOnOpen */,
+                                              NULL);
                         if (FAILED(rc))
                             break;
                         if (!floppyMedium)
@@ -1007,6 +1039,43 @@ int handleModifyVM(HandlerArg *a)
                 break;
             }
 
+            case MODIFYVM_NICPROPERTY:
+            {
+                ComPtr<INetworkAdapter> nic;
+
+                CHECK_ERROR_BREAK(machine, GetNetworkAdapter(GetOptState.uIndex - 1, nic.asOutParam()));
+                ASSERT(nic);
+
+                if (nic)
+                {
+                    /* Parse 'name=value' */
+                    char *pszProperty = RTStrDup(ValueUnion.psz);
+                    if (pszProperty)
+                    {
+                        char *pDelimiter = strchr(pszProperty, '=');
+                        if (pDelimiter)
+                        {
+                            *pDelimiter = '\0';
+
+                            Bstr bstrName = pszProperty;
+                            Bstr bstrValue = &pDelimiter[1];
+                            CHECK_ERROR(nic, SetProperty(bstrName.raw(), bstrValue.raw()));
+                        }
+                        else
+                        {
+                            errorArgument("Invalid --nicproperty%d argument '%s'", GetOptState.uIndex, ValueUnion.psz);
+                            rc = E_FAIL;
+                        }
+                        RTStrFree(pszProperty);
+                    }
+                    else
+                    {
+                        RTStrmPrintf(g_pStdErr, "Error: Failed to allocate memory for --nicproperty%d '%s'\n", GetOptState.uIndex, ValueUnion.psz);
+                        rc = E_FAIL;
+                    }
+                }
+                break;
+            }
             case MODIFYVM_NICTYPE:
             {
                 ComPtr<INetworkAdapter> nic;
@@ -1084,6 +1153,61 @@ int handleModifyVM(HandlerArg *a)
                 break;
             }
 
+            case MODIFYVM_NICPROMISC:
+            {
+                NetworkAdapterPromiscModePolicy_T enmPromiscModePolicy;
+                if (!strcmp(ValueUnion.psz, "deny"))
+                    enmPromiscModePolicy = NetworkAdapterPromiscModePolicy_Deny;
+                else if (   !strcmp(ValueUnion.psz, "allow-vms")
+                         || !strcmp(ValueUnion.psz, "allow-network"))
+                    enmPromiscModePolicy = NetworkAdapterPromiscModePolicy_AllowNetwork;
+                else if (!strcmp(ValueUnion.psz, "allow-all"))
+                    enmPromiscModePolicy = NetworkAdapterPromiscModePolicy_AllowAll;
+                else
+                {
+                    errorArgument("Unknown promiscuous mode policy '%s'", ValueUnion.psz);
+                    rc = E_INVALIDARG;
+                    break;
+                }
+
+                ComPtr<INetworkAdapter> nic;
+                CHECK_ERROR_BREAK(machine, GetNetworkAdapter(GetOptState.uIndex - 1, nic.asOutParam()));
+                ASSERT(nic);
+
+                CHECK_ERROR(nic, COMSETTER(PromiscModePolicy)(enmPromiscModePolicy));
+                break;
+            }
+
+            case MODIFYVM_NICBWGROUP:
+            {
+                ComPtr<INetworkAdapter> nic;
+                CHECK_ERROR_BREAK(machine, GetNetworkAdapter(GetOptState.uIndex - 1, nic.asOutParam()));
+                ASSERT(nic);
+
+                if (!RTStrICmp(ValueUnion.psz, "none"))
+                {
+                    /* Just remove the bandwidth group. */
+                    CHECK_ERROR(nic, COMSETTER(BandwidthGroup)(NULL));
+                }
+                else
+                {
+                    ComPtr<IBandwidthControl> bwCtrl;
+                    ComPtr<IBandwidthGroup> bwGroup;
+
+                    CHECK_ERROR(machine, COMGETTER(BandwidthControl)(bwCtrl.asOutParam()));
+
+                    if (SUCCEEDED(rc))
+                    {
+                        CHECK_ERROR(bwCtrl, GetBandwidthGroup(Bstr(ValueUnion.psz).raw(), bwGroup.asOutParam()));
+                        if (SUCCEEDED(rc))
+                        {
+                            CHECK_ERROR(nic, COMSETTER(BandwidthGroup)(bwGroup));
+                        }
+                    }
+                }
+                break;
+            }
+
             case MODIFYVM_NIC:
             {
                 ComPtr<INetworkAdapter> nic;
@@ -1098,40 +1222,36 @@ int handleModifyVM(HandlerArg *a)
                 else if (!strcmp(ValueUnion.psz, "null"))
                 {
                     CHECK_ERROR(nic, COMSETTER(Enabled)(TRUE));
-                    CHECK_ERROR(nic, Detach());
+                    CHECK_ERROR(nic, COMSETTER(AttachmentType)(NetworkAttachmentType_Null));
                 }
                 else if (!strcmp(ValueUnion.psz, "nat"))
                 {
                     CHECK_ERROR(nic, COMSETTER(Enabled)(TRUE));
-                    CHECK_ERROR(nic, AttachToNAT());
+                    CHECK_ERROR(nic, COMSETTER(AttachmentType)(NetworkAttachmentType_NAT));
                 }
                 else if (  !strcmp(ValueUnion.psz, "bridged")
                         || !strcmp(ValueUnion.psz, "hostif")) /* backward compatibility */
                 {
                     CHECK_ERROR(nic, COMSETTER(Enabled)(TRUE));
-                    CHECK_ERROR(nic, AttachToBridgedInterface());
+                    CHECK_ERROR(nic, COMSETTER(AttachmentType)(NetworkAttachmentType_Bridged));
                 }
                 else if (!strcmp(ValueUnion.psz, "intnet"))
                 {
                     CHECK_ERROR(nic, COMSETTER(Enabled)(TRUE));
-                    CHECK_ERROR(nic, AttachToInternalNetwork());
+                    CHECK_ERROR(nic, COMSETTER(AttachmentType)(NetworkAttachmentType_Internal));
                 }
-#if defined(VBOX_WITH_NETFLT)
                 else if (!strcmp(ValueUnion.psz, "hostonly"))
                 {
 
                     CHECK_ERROR(nic, COMSETTER(Enabled)(TRUE));
-                    CHECK_ERROR(nic, AttachToHostOnlyInterface());
+                    CHECK_ERROR(nic, COMSETTER(AttachmentType)(NetworkAttachmentType_HostOnly));
                 }
-#endif
-#ifdef VBOX_WITH_VDE
-                else if (!strcmp(ValueUnion.psz, "vde"))
+                else if (!strcmp(ValueUnion.psz, "generic"))
                 {
 
                     CHECK_ERROR(nic, COMSETTER(Enabled)(TRUE));
-                    CHECK_ERROR(nic, AttachToVDE());
+                    CHECK_ERROR(nic, COMSETTER(AttachmentType)(NetworkAttachmentType_Generic));
                 }
-#endif
                 else
                 {
                     errorArgument("Invalid type '%s' specfied for NIC %u", ValueUnion.psz, GetOptState.uIndex);
@@ -1152,6 +1272,24 @@ int handleModifyVM(HandlerArg *a)
             }
 
             case MODIFYVM_BRIDGEADAPTER:
+            {
+                ComPtr<INetworkAdapter> nic;
+
+                CHECK_ERROR_BREAK(machine, GetNetworkAdapter(GetOptState.uIndex - 1, nic.asOutParam()));
+                ASSERT(nic);
+
+                /* remove it? */
+                if (!strcmp(ValueUnion.psz, "none"))
+                {
+                    CHECK_ERROR(nic, COMSETTER(BridgedInterface)(Bstr().raw()));
+                }
+                else
+                {
+                    CHECK_ERROR(nic, COMSETTER(BridgedInterface)(Bstr(ValueUnion.psz).raw()));
+                }
+                break;
+            }
+
             case MODIFYVM_HOSTONLYADAPTER:
             {
                 ComPtr<INetworkAdapter> nic;
@@ -1159,15 +1297,14 @@ int handleModifyVM(HandlerArg *a)
                 CHECK_ERROR_BREAK(machine, GetNetworkAdapter(GetOptState.uIndex - 1, nic.asOutParam()));
                 ASSERT(nic);
 
-                /** @todo NULL string deprecated */
                 /* remove it? */
                 if (!strcmp(ValueUnion.psz, "none"))
                 {
-                    CHECK_ERROR(nic, COMSETTER(HostInterface)(NULL));
+                    CHECK_ERROR(nic, COMSETTER(HostOnlyInterface)(Bstr().raw()));
                 }
                 else
                 {
-                    CHECK_ERROR(nic, COMSETTER(HostInterface)(Bstr(ValueUnion.psz).raw()));
+                    CHECK_ERROR(nic, COMSETTER(HostOnlyInterface)(Bstr(ValueUnion.psz).raw()));
                 }
                 break;
             }
@@ -1179,11 +1316,10 @@ int handleModifyVM(HandlerArg *a)
                 CHECK_ERROR_BREAK(machine, GetNetworkAdapter(GetOptState.uIndex - 1, nic.asOutParam()));
                 ASSERT(nic);
 
-                /** @todo NULL string deprecated */
                 /* remove it? */
                 if (!strcmp(ValueUnion.psz, "none"))
                 {
-                    CHECK_ERROR(nic, COMSETTER(InternalNetwork)(NULL));
+                    CHECK_ERROR(nic, COMSETTER(InternalNetwork)(Bstr().raw()));
                 }
                 else
                 {
@@ -1192,27 +1328,17 @@ int handleModifyVM(HandlerArg *a)
                 break;
             }
 
-#ifdef VBOX_WITH_VDE
-            case MODIFYVM_VDENET:
+            case MODIFYVM_GENERICDRV:
             {
                 ComPtr<INetworkAdapter> nic;
 
                 CHECK_ERROR_BREAK(machine, GetNetworkAdapter(GetOptState.uIndex - 1, nic.asOutParam()));
                 ASSERT(nic);
 
-                /** @todo NULL string deprecated */
-                /* remove it? */
-                if (!strcmp(ValueUnion.psz, "default"))
-                {
-                    CHECK_ERROR(nic, COMSETTER(VDENetwork)(NULL));
-                }
-                else
-                {
-                    CHECK_ERROR(nic, COMSETTER(VDENetwork)(Bstr(ValueUnion.psz).raw()));
-                }
+                CHECK_ERROR(nic, COMSETTER(GenericDriver)(Bstr(ValueUnion.psz).raw()));
                 break;
             }
-#endif
+
             case MODIFYVM_NATNET:
             {
                 ComPtr<INetworkAdapter> nic;
@@ -1472,7 +1598,7 @@ int handleModifyVM(HandlerArg *a)
                 /* generate one? */
                 if (!strcmp(ValueUnion.psz, "auto"))
                 {
-                    CHECK_ERROR(nic, COMSETTER(MACAddress)(NULL));
+                    CHECK_ERROR(nic, COMSETTER(MACAddress)(Bstr().raw()));
                 }
                 else
                 {
@@ -1796,7 +1922,7 @@ int handleModifyVM(HandlerArg *a)
                         CHECK_ERROR(vrdeServer, COMSETTER(VRDEExtPack)(bstr.raw()));
                     }
                     else
-                        CHECK_ERROR(vrdeServer, COMSETTER(VRDEExtPack)(NULL));
+                        CHECK_ERROR(vrdeServer, COMSETTER(VRDEExtPack)(Bstr().raw()));
                 }
                 break;
             }
@@ -1912,7 +2038,7 @@ int handleModifyVM(HandlerArg *a)
                         CHECK_ERROR(vrdeServer, COMSETTER(AuthLibrary)(bstr.raw()));
                     }
                     else
-                        CHECK_ERROR(vrdeServer, COMSETTER(AuthLibrary)(NULL));
+                        CHECK_ERROR(vrdeServer, COMSETTER(AuthLibrary)(Bstr().raw()));
                 }
                 break;
             }
@@ -2000,7 +2126,7 @@ int handleModifyVM(HandlerArg *a)
             case MODIFYVM_SNAPSHOTFOLDER:
             {
                 if (!strcmp(ValueUnion.psz, "default"))
-                    CHECK_ERROR(machine, COMSETTER(SnapshotFolder)(NULL));
+                    CHECK_ERROR(machine, COMSETTER(SnapshotFolder)(Bstr().raw()));
                 else
                     CHECK_ERROR(machine, COMSETTER(SnapshotFolder)(Bstr(ValueUnion.psz).raw()));
                 break;
@@ -2121,7 +2247,45 @@ int handleModifyVM(HandlerArg *a)
                 }
                 break;
             }
+#ifdef VBOX_WITH_PCI_PASSTHROUGH
+            case MODIFYVM_ATTACH_PCI:
+            {
+                const char* pAt = strchr(ValueUnion.psz, '@');
+                int32_t iHostAddr, iGuestAddr;
 
+                iHostAddr = parsePci(ValueUnion.psz);
+                iGuestAddr = pAt != NULL ? parsePci(pAt + 1) : iHostAddr;
+
+                if (iHostAddr == -1 || iGuestAddr == -1)
+                {
+                    errorArgument("Invalid --pciattach argument '%s' (valid: 'HB:HD.HF@GB:GD.GF' or just 'HB:HD.HF')", ValueUnion.psz);
+                    rc = E_FAIL;
+                }
+                else
+                {
+                    CHECK_ERROR(machine, AttachHostPciDevice(iHostAddr, iGuestAddr, TRUE));
+                }
+
+                break;
+            }
+            case MODIFYVM_DETACH_PCI:
+            {
+                int32_t iHostAddr;
+
+                iHostAddr = parsePci(ValueUnion.psz);
+                if (iHostAddr == -1)
+                {
+                    errorArgument("Invalid --pcidetach argument '%s' (valid: 'HB:HD.HF')", ValueUnion.psz);
+                    rc = E_FAIL;
+                }
+                else
+                {
+                    CHECK_ERROR(machine, DetachHostPciDevice(iHostAddr));
+                }
+
+                break;
+            }
+#endif
             default:
             {
                 errorGetOpt(USAGE_MODIFYVM, c, &ValueUnion);
