@@ -1,4 +1,4 @@
-/* $Id: VBoxMediaManagerDlg.cpp $ */
+/* $Id: VBoxMediaManagerDlg.cpp 38311 2011-08-04 13:08:39Z vboxsync $ */
 /** @file
  *
  * VBox frontends: Qt4 GUI ("VirtualBox"):
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -36,13 +36,14 @@
 /* Local includes */
 #include "VBoxGlobal.h"
 #include "VBoxMediaManagerDlg.h"
-#include "UINewHDWzd.h"
-#include "VBoxProblemReporter.h"
+#include "UINewHDWizard.h"
+#include "UIMessageCenter.h"
 #include "UIToolBar.h"
 #include "QIFileDialog.h"
 #include "QILabel.h"
 #include "UIIconPool.h"
 #include "UIVirtualBoxEventHandler.h"
+#include "UIMediumTypeChangeDialog.h"
 #endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
 #ifdef Q_WS_MAC
@@ -108,6 +109,8 @@ public:
     QString hardDiskFormat() const { return mMedium.hardDiskFormat (!mManager->showDiffs()); }
     QString hardDiskType() const { return mMedium.hardDiskType (!mManager->showDiffs()); }
 
+    QString details() const { return mMedium.storageDetails(); }
+
     QString usage() const { return mMedium.usage (!mManager->showDiffs()); }
 
     QString toolTip() const { return mMedium.toolTip (!mManager->showDiffs(), mManager->inAttachMode()); }
@@ -286,12 +289,16 @@ VBoxMediaManagerDlg::VBoxMediaManagerDlg (QWidget *aParent /* = 0 */, Qt::Window
 
     mNewAction     = new QAction (this);
     mAddAction     = new QAction (this);
+    mCopyAction    = new QAction (this);
+    mModifyAction  = new QAction (this);
     mRemoveAction  = new QAction (this);
     mReleaseAction = new QAction (this);
     mRefreshAction = new QAction (this);
 
     connect (mNewAction, SIGNAL (triggered()), this, SLOT (doNewMedium()));
     connect (mAddAction, SIGNAL (triggered()), this, SLOT (doAddMedium()));
+    connect (mCopyAction, SIGNAL (triggered()), this, SLOT (doCopyMedium()));
+    connect (mModifyAction, SIGNAL (triggered()), this, SLOT (doModifyMedium()));
     connect (mRemoveAction, SIGNAL (triggered()), this, SLOT (doRemoveMedium()));
     connect (mReleaseAction, SIGNAL (triggered()), this, SLOT (doReleaseMedium()));
     connect (mRefreshAction, SIGNAL (triggered()), this, SLOT (refreshAll()));
@@ -304,6 +311,14 @@ VBoxMediaManagerDlg::VBoxMediaManagerDlg (QWidget *aParent /* = 0 */, Qt::Window
         QSize (22, 22), QSize (16, 16),
         ":/hd_add_22px.png", ":/hd_add_16px.png",
         ":/hd_add_disabled_22px.png", ":/hd_add_disabled_16px.png"));
+    mCopyAction->setIcon(UIIconPool::iconSetFull (
+        QSize (22, 22), QSize (16, 16),
+        ":/hd_add_22px.png", ":/hd_add_16px.png",
+        ":/hd_add_disabled_22px.png", ":/hd_add_disabled_16px.png"));
+    mModifyAction->setIcon(UIIconPool::iconSetFull (
+        QSize (22, 22), QSize (16, 16),
+        ":/hd_new_22px.png", ":/hd_new_16px.png",
+        ":/hd_new_disabled_22px.png", ":/hd_new_disabled_16px.png"));
     mRemoveAction->setIcon(UIIconPool::iconSetFull (
         QSize (22, 22), QSize (16, 16),
         ":/hd_remove_22px.png", ":/hd_remove_16px.png",
@@ -317,6 +332,8 @@ VBoxMediaManagerDlg::VBoxMediaManagerDlg (QWidget *aParent /* = 0 */, Qt::Window
         ":/refresh_22px.png", ":/refresh_16px.png",
         ":/refresh_disabled_22px.png", ":/refresh_disabled_16px.png"));
 
+    mActionsContextMenu->addAction (mCopyAction);
+    mActionsContextMenu->addAction (mModifyAction);
     mActionsContextMenu->addAction (mRemoveAction);
     mActionsContextMenu->addAction (mReleaseAction);
 
@@ -345,6 +362,8 @@ VBoxMediaManagerDlg::VBoxMediaManagerDlg (QWidget *aParent /* = 0 */, Qt::Window
 
 //    mToolBar->addAction (mNewAction);
 //    mToolBar->addAction (mAddAction);
+    mToolBar->addAction (mCopyAction);
+    mToolBar->addAction (mModifyAction);
 //    mToolBar->addSeparator();
     mToolBar->addAction (mRemoveAction);
     mToolBar->addAction (mReleaseAction);
@@ -355,6 +374,8 @@ VBoxMediaManagerDlg::VBoxMediaManagerDlg (QWidget *aParent /* = 0 */, Qt::Window
     mActionsMenu = menuBar()->addMenu (QString::null);
 //    mActionsMenu->addAction (mNewAction);
 //    mActionsMenu->addAction (mAddAction);
+    mActionsMenu->addAction (mCopyAction);
+    mActionsMenu->addAction (mModifyAction);
 //    mActionsMenu->addSeparator();
     mActionsMenu->addAction (mRemoveAction);
     mActionsMenu->addAction (mReleaseAction);
@@ -379,7 +400,7 @@ VBoxMediaManagerDlg::VBoxMediaManagerDlg (QWidget *aParent /* = 0 */, Qt::Window
     /* Connects for the button box */
     connect (mButtonBox, SIGNAL (accepted()), this, SLOT (accept()));
     connect (mButtonBox, SIGNAL (rejected()), this, SLOT (reject()));
-    connect (mButtonBox, SIGNAL (helpRequested()), &vboxProblem(), SLOT (showHelpHelpDialog()));
+    connect (mButtonBox, SIGNAL (helpRequested()), &msgCenter(), SLOT (sltShowHelpHelpDialog()));
 }
 
 VBoxMediaManagerDlg::~VBoxMediaManagerDlg()
@@ -588,18 +609,24 @@ void VBoxMediaManagerDlg::retranslateUi()
 
     mNewAction->setText (tr ("&New..."));
     mAddAction->setText (tr ("&Add..."));
+    mCopyAction->setText (tr ("&Copy..."));
+    mModifyAction->setText (tr ("&Modify..."));
     mRemoveAction->setText (tr ("R&emove"));
     mReleaseAction->setText (tr ("Re&lease"));
     mRefreshAction->setText (tr ("Re&fresh"));
 
     mNewAction->setShortcut (QKeySequence (QKeySequence::New));
     mAddAction->setShortcut (QKeySequence ("Ins"));
+    mCopyAction->setShortcut (QKeySequence ("Ctrl+C"));
+    mModifyAction->setShortcut (QKeySequence ("Ctrl+Space"));
     mRemoveAction->setShortcut (QKeySequence (QKeySequence::Delete));
     mReleaseAction->setShortcut (QKeySequence ("Ctrl+L"));
     mRefreshAction->setShortcut (QKeySequence (QKeySequence::Refresh));
 
     mNewAction->setStatusTip (tr ("Create a new virtual hard disk"));
     mAddAction->setStatusTip (tr ("Add an existing medium"));
+    mCopyAction->setStatusTip (tr ("Copy an existing medium"));
+    mModifyAction->setStatusTip (tr ("Modify the attributes of the selected medium"));
     mRemoveAction->setStatusTip (tr ("Remove the selected medium"));
     mReleaseAction->setStatusTip (tr ("Release the selected medium by detaching it from the machines"));
     mRefreshAction->setStatusTip (tr ("Refresh the media list"));
@@ -608,6 +635,10 @@ void VBoxMediaManagerDlg::retranslateUi()
         QString (" (%1)").arg (mNewAction->shortcut().toString()));
     mAddAction->setToolTip (mAddAction->text().remove ('&') +
         QString (" (%1)").arg (mAddAction->shortcut().toString()));
+    mCopyAction->setToolTip (mCopyAction->text().remove ('&') +
+        QString (" (%1)").arg (mCopyAction->shortcut().toString()));
+    mModifyAction->setToolTip (mModifyAction->text().remove ('&') +
+        QString (" (%1)").arg (mModifyAction->shortcut().toString()));
     mRemoveAction->setToolTip (mRemoveAction->text().remove ('&') +
         QString (" (%1)").arg (mRemoveAction->shortcut().toString()));
     mReleaseAction->setToolTip (mReleaseAction->text().remove ('&') +
@@ -615,16 +646,6 @@ void VBoxMediaManagerDlg::retranslateUi()
     mRefreshAction->setToolTip (mRefreshAction->text().remove ('&') +
         QString (" (%1)").arg (mRefreshAction->shortcut().toString()));
 
-    mLbHD1->setText (QString ("<nobr>%1:</nobr>").arg (tr ("Location")));
-    mLbHD2->setText (QString ("<nobr>%1:</nobr>").arg (tr ("Type (Format)")));
-    mLbHD3->setText (QString ("<nobr>%1:</nobr>").arg (tr ("Attached to", "VMM: Virtual Disk")));
-
-    mLbCD1->setText (QString ("<nobr>%1:</nobr>").arg (tr ("Location")));
-    mLbCD2->setText (QString ("<nobr>%1:</nobr>").arg (tr ("Attached to", "VMM: CD/DVD Image")));
-
-    mLbFD1->setText (QString ("<nobr>%1:</nobr>").arg (tr ("Location")));
-    mLbFD2->setText (QString ("<nobr>%1:</nobr>").arg (tr ("Attached to", "VMM: Floppy Image")));
-
     mProgressBar->setText (tr ("Checking accessibility"));
 #ifdef Q_WS_MAC
     /* Make sure that the widgets aren't jumping around while the progress bar get visible. */
@@ -640,8 +661,7 @@ void VBoxMediaManagerDlg::retranslateUi()
     mToolBar->updateLayout();
 #endif /* QT_MAC_USE_COCOA */
 
-    if (mDoSelect)
-        mButtonBox->button (QDialogButtonBox::Ok)->setText (tr ("&Select"));
+    mButtonBox->button(QDialogButtonBox::Ok)->setText(mDoSelect ? tr("&Select") : tr("C&lose"));
 
     if (mTwHD->model()->rowCount() || mTwCD->model()->rowCount() || mTwFD->model()->rowCount())
         refreshAll();
@@ -951,7 +971,7 @@ void VBoxMediaManagerDlg::doNewMedium()
 {
     AssertReturnVoid (currentTreeWidgetType() == VBoxDefs::MediumType_HardDisk);
 
-    UINewHDWzd dlg (this);
+    UINewHDWizard dlg (this);
 
     if (dlg.exec() == QDialog::Accepted)
     {
@@ -1036,6 +1056,32 @@ void VBoxMediaManagerDlg::doAddMedium()
     }
 }
 
+void VBoxMediaManagerDlg::doCopyMedium()
+{
+    /* Get current tree: */
+    QTreeWidget *pTree = currentTreeWidget();
+    /* Get current item of current tree: */
+    MediaItem *pItem = toMediaItem(pTree->currentItem());
+
+    UINewHDWizard wizard(this /* parent dialog */,
+                         UINewHDWizard::tr("%1_copy", "copied virtual disk name").arg(QFileInfo(pItem->text(0)).baseName()) /* default name */,
+                         QFileInfo(pItem->location()).absolutePath() /* default path */,
+                         0 /* default size, not important for copying */,
+                         pItem->medium().medium() /* base medium for copying */);
+    wizard.exec();
+}
+
+void VBoxMediaManagerDlg::doModifyMedium()
+{
+    MediaItem *pMediumItem = toMediaItem(currentTreeWidget()->currentItem());
+    UIMediumTypeChangeDialog dlg(this, pMediumItem->id());
+    if (dlg.exec() == QDialog::Accepted)
+    {
+        pMediumItem->refreshAll();
+        m_pTypePane->setText(pMediumItem->hardDiskType());
+    }
+}
+
 void VBoxMediaManagerDlg::doRemoveMedium()
 {
     QTreeWidget *tree = currentTreeWidget();
@@ -1047,7 +1093,7 @@ void VBoxMediaManagerDlg::doRemoveMedium()
     AssertReturnVoid (!id.isNull());
     VBoxDefs::MediumType type = item->type();
 
-    if (!vboxProblem().confirmRemoveMedium (this, item->medium()))
+    if (!msgCenter().confirmRemoveMedium (this, item->medium()))
         return;
 
     COMResult result;
@@ -1060,13 +1106,13 @@ void VBoxMediaManagerDlg::doRemoveMedium()
 
             /* We don't want to try to delete inaccessible storage as it will
              * most likely fail. Note that
-             * VBoxProblemReporter::confirmRemoveMedium() is aware of that and
+             * UIMessageCenter::confirmRemoveMedium() is aware of that and
              * will give a corresponding hint. Therefore, once the code is
              * changed below, the hint should be re-checked for validity. */
             if (item->state() != KMediumState_Inaccessible &&
                 item->medium().medium().GetMediumFormat().GetCapabilities() & MediumFormatCapabilities_File)
             {
-                int rc = vboxProblem().
+                int rc = msgCenter().
                     confirmDeleteHardDiskStorage (this, item->location());
                 if (rc == QIMessageBox::Cancel)
                     return;
@@ -1080,10 +1126,10 @@ void VBoxMediaManagerDlg::doRemoveMedium()
                 CProgress progress = hardDisk.DeleteStorage();
                 if (hardDisk.isOk())
                 {
-                    vboxProblem().showModalProgressDialog(progress, windowTitle(), ":/progress_media_delete_90px.png", this, true);
+                    msgCenter().showModalProgressDialog(progress, windowTitle(), ":/progress_media_delete_90px.png", this, true);
                     if (!(progress.isOk() && progress.GetResultCode() == S_OK))
                     {
-                        vboxProblem().cannotDeleteHardDiskStorage(this, hardDisk, progress);
+                        msgCenter().cannotDeleteHardDiskStorage(this, hardDisk, progress);
                         return;
                     }
                 }
@@ -1114,7 +1160,7 @@ void VBoxMediaManagerDlg::doRemoveMedium()
     if (result.isOk())
         vboxGlobal().removeMedium (type, id);
     else
-        vboxProblem().cannotCloseMedium (this, item->medium(), result);
+        msgCenter().cannotCloseMedium (this, item->medium(), result);
 }
 
 void VBoxMediaManagerDlg::doReleaseMedium()
@@ -1155,7 +1201,7 @@ void VBoxMediaManagerDlg::doReleaseMedium()
 
     AssertReturnVoid (machines.size() > 0);
 
-    if (!vboxProblem().confirmReleaseMedium (this, item->medium(), usage))
+    if (!msgCenter().confirmReleaseMedium (this, item->medium(), usage))
         return;
 
     for (QList <QString>::const_iterator it = machineIds.begin(); it != machineIds.end(); ++ it)
@@ -1208,7 +1254,7 @@ bool VBoxMediaManagerDlg::releaseMediumFrom (const VBoxMedium &aMedium, const QS
                     if (!machine.isOk())
                     {
                         CStorageController controller = machine.GetStorageControllerByName (attachment.GetController());
-                        vboxProblem().cannotDetachDevice (this, machine, VBoxDefs::MediumType_HardDisk, aMedium.location(),
+                        msgCenter().cannotDetachDevice (this, machine, VBoxDefs::MediumType_HardDisk, aMedium.location(),
                                                           StorageSlot(controller.GetBus(), attachment.GetPort(), attachment.GetDevice()));
                         success = false;
                         break;
@@ -1230,7 +1276,7 @@ bool VBoxMediaManagerDlg::releaseMediumFrom (const VBoxMedium &aMedium, const QS
                     machine.MountMedium (attachment.GetController(), attachment.GetPort(), attachment.GetDevice(), CMedium(), false /* force */);
                     if (!machine.isOk())
                     {
-                        vboxProblem().cannotRemountMedium (this, machine, aMedium, false /* mount? */, false /* retry? */);
+                        msgCenter().cannotRemountMedium (this, machine, aMedium, false /* mount? */, false /* retry? */);
                         success = false;
                         break;
                     }
@@ -1251,7 +1297,7 @@ bool VBoxMediaManagerDlg::releaseMediumFrom (const VBoxMedium &aMedium, const QS
                     machine.MountMedium (attachment.GetController(), attachment.GetPort(), attachment.GetDevice(), CMedium(), false /* force */);
                     if (!machine.isOk())
                     {
-                        vboxProblem().cannotRemountMedium (this, machine, aMedium, false /* mount? */, false /* retry? */);
+                        msgCenter().cannotRemountMedium (this, machine, aMedium, false /* mount? */, false /* retry? */);
                         success = false;
                         break;
                     }
@@ -1268,7 +1314,7 @@ bool VBoxMediaManagerDlg::releaseMediumFrom (const VBoxMedium &aMedium, const QS
         machine.SaveSettings();
         if (!machine.isOk())
         {
-            vboxProblem().cannotSaveMachineSettings (machine);
+            msgCenter().cannotSaveMachineSettings (machine);
             success = false;
         }
     }
@@ -1390,11 +1436,17 @@ void VBoxMediaManagerDlg::processCurrentChanged (QTreeWidgetItem *aItem,
     /* New and Add are now enabled even when enumerating since it should be safe */
     bool newEnabled     = currentTreeWidgetType() == VBoxDefs::MediumType_HardDisk;
     bool addEnabled     = true;
+    bool copyEnabled    = currentTreeWidgetType() == VBoxDefs::MediumType_HardDisk &&
+                          notInEnum && item && checkMediumFor (item, Action_Copy);
+    bool modifyEnabled  = currentTreeWidgetType() == VBoxDefs::MediumType_HardDisk &&
+                          notInEnum && item && checkMediumFor (item, Action_Modify);
     bool removeEnabled  = notInEnum && item && checkMediumFor (item, Action_Remove);
     bool releaseEnabled = item && checkMediumFor (item, Action_Release);
 
     mNewAction->setEnabled (newEnabled);
     mAddAction->setEnabled (addEnabled);
+    mCopyAction->setEnabled (copyEnabled);
+    mModifyAction->setEnabled (modifyEnabled);
     mRemoveAction->setEnabled (removeEnabled);
     mReleaseAction->setEnabled (releaseEnabled);
 
@@ -1406,16 +1458,18 @@ void VBoxMediaManagerDlg::processCurrentChanged (QTreeWidgetItem *aItem,
 
     if (item)
     {
+        QString details = item->details();
         QString usage = item->usage().isNull() ?
                         formatPaneText (tr ("<i>Not&nbsp;Attached</i>"), false) :
                         formatPaneText (item->usage());
 
         if (item->treeWidget() == mTwHD)
         {
-            mIpHD1->setText (formatPaneText (item->location(), true, "end"));
-            mIpHD2->setText (formatPaneText (QString ("%1 (%2)").arg (item->hardDiskType())
-                                                                .arg (item->hardDiskFormat()), false));
-            mIpHD3->setText (usage);
+            m_pTypePane->setText(item->hardDiskType());
+            m_pLocationPane->setText(formatPaneText(item->location(), true, "end"));
+            m_pFormatPane->setText(item->hardDiskFormat());
+            m_pDetailsPane->setText(details);
+            m_pUsagePane->setText(usage);
         }
         else if (item->treeWidget() == mTwCD)
         {
@@ -1440,8 +1494,16 @@ void VBoxMediaManagerDlg::processDoubleClick (QTreeWidgetItem * /* aItem */, int
 {
     QTreeWidget *tree = currentTreeWidget();
 
-    if (mDoSelect && selectedItem (tree) && mButtonBox->button (QDialogButtonBox::Ok)->isEnabled())
-        accept();
+    if (mDoSelect)
+    {
+        if (selectedItem (tree) && mButtonBox->button (QDialogButtonBox::Ok)->isEnabled())
+            accept();
+    }
+    else
+    {
+        if (currentTreeWidgetType() == VBoxDefs::MediumType_HardDisk)
+            doModifyMedium();
+    }
 }
 
 void VBoxMediaManagerDlg::showContextMenu (const QPoint &aPos)
@@ -1532,12 +1594,12 @@ void VBoxMediaManagerDlg::addMediumToList(const QString &aLocation, VBoxDefs::Me
             AssertMsgFailedReturnVoid (("Invalid aType %d\n", aType));
     }
 
-    CMedium med = mVBox.OpenMedium(aLocation, devType, KAccessMode_ReadWrite);
+    CMedium med = mVBox.OpenMedium(aLocation, devType, KAccessMode_ReadWrite, false /* fForceNewUuid */);
     if (mVBox.isOk())
         medium = VBoxMedium(CMedium(med), aType, KMediumState_Created);
 
     if (!mVBox.isOk())
-        vboxProblem().cannotOpenMedium(this, mVBox, aType, aLocation);
+        msgCenter().cannotOpenMedium(this, mVBox, aType, aLocation);
     else
         vboxGlobal().addMedium(medium);
 }
@@ -1712,6 +1774,16 @@ bool VBoxMediaManagerDlg::checkMediumFor (MediaItem *aItem, Action aAction)
             }
             return true;
         }
+        case Action_Copy:
+        {
+            /* False for children: */
+            return !aItem->medium().parent();
+        }
+        case Action_Modify:
+        {
+            /* False for children: */
+            return !aItem->medium().parent();
+        }
         case Action_Remove:
         {
             /* Removable if not attached to anything */
@@ -1783,7 +1855,7 @@ void VBoxMediaManagerDlg::addDndUrls (const QList <QUrl> &aUrls)
 
 void VBoxMediaManagerDlg::clearInfoPanes()
 {
-    mIpHD1->clear(); mIpHD2->clear(); mIpHD3->clear();
+    m_pTypePane->clear(); m_pLocationPane->clear(); m_pFormatPane->clear(); m_pDetailsPane->clear(); m_pUsagePane->clear();
     mIpCD1->clear(); mIpCD2->clear();
     mIpFD1->clear(); mIpFD2->clear();
 }
