@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -177,6 +177,11 @@ public:
 
     const QRect availableGeometry(int iScreen = 0) const;
 
+    bool isPatmDisabled() const { return mDisablePatm; }
+    bool isCsamDisabled() const { return mDisableCsam; }
+    bool isSupervisorCodeExecedRecompiled() const { return mRecompileSupervisor; }
+    bool isUserCodeExecedRecompiled()       const { return mRecompileUser; }
+
 #ifdef VBOX_WITH_DEBUGGER_GUI
     bool isDebuggerEnabled(CMachine &aMachine);
     bool isDebuggerAutoShowEnabled(CMachine &aMachine);
@@ -314,6 +319,9 @@ public:
         AssertMsg (!mDiskTypes.value (t).isNull(), ("No text for %d", t));
         return mDiskTypes.value (t);
     }
+    QString differencingMediumTypeName() const { return mDiskTypes_Differencing; }
+
+    QString toString(KMediumVariant mediumVariant) const;
 
     /**
      * Similar to toString (KMediumType), but returns 'Differencing' for
@@ -481,6 +489,21 @@ public:
         return KNetworkAttachmentType (it.key());
     }
 
+    QString toString (KNetworkAdapterPromiscModePolicy t) const
+    {
+        AssertMsg (!mNetworkAdapterPromiscModePolicyTypes.value (t).isNull(), ("No text for %d", t));
+        return mNetworkAdapterPromiscModePolicyTypes.value (t);
+    }
+
+    KNetworkAdapterPromiscModePolicy toNetworkAdapterPromiscModePolicyType (const QString &s) const
+    {
+        QULongStringHash::const_iterator it =
+            qFind (mNetworkAdapterPromiscModePolicyTypes.begin(), mNetworkAdapterPromiscModePolicyTypes.end(), s);
+        AssertMsg (it != mNetworkAdapterPromiscModePolicyTypes.end(), ("No value for {%s}",
+                                                                       s.toLatin1().constData()));
+        return KNetworkAdapterPromiscModePolicy (it.key());
+    }
+
     QString toString (KNATProtocol t) const
     {
         AssertMsg (!mNATProtocolTypes.value (t).isNull(), ("No text for %d", t));
@@ -565,6 +588,8 @@ public:
 
     void startEnumeratingMedia();
 
+    void reloadProxySettings();
+
     /**
      * Returns a list of all currently registered media. This list is used to
      * globally track the accessibility state of all media on a dedicated thread.
@@ -620,7 +645,6 @@ public:
 
     /* public static stuff */
 
-    static bool shouldWarnAboutToLowVRAM(const CMachine *pMachine = 0);
     static bool isDOSType (const QString &aOSTypeId);
 
     static QString languageId();
@@ -647,7 +671,7 @@ public:
     static QString formatSize (quint64 aSize, uint aDecimal = 2,
                                VBoxDefs::FormatSize aMode = VBoxDefs::FormatSize_Round);
 
-    static quint64 requiredVideoMemory (CMachine *aMachine = 0, int cMonitors = 1);
+    static quint64 requiredVideoMemory(const QString &strGuestOSTypeId, int cMonitors = 1);
 
     static QString locationForHTML (const QString &aFileName);
 
@@ -699,8 +723,9 @@ public:
 #endif
 
 #ifdef VBOX_WITH_CRHGSMI
-    static quint64 required3DWddmOffscreenVideoMemory(CMachine *aMachine = 0, int cMonitors = 1);
-#endif
+    static bool isWddmCompatibleOsType(const QString &strGuestOSTypeId);
+    static quint64 required3DWddmOffscreenVideoMemory(const QString &strGuestOSTypeId, int cMonitors = 1);
+#endif /* VBOX_WITH_CRHGSMI */
 
 #ifdef Q_WS_MAC
     bool isSheetWindowsAllowed(QWidget *pParent) const;
@@ -748,6 +773,7 @@ public slots:
     void showUpdateDialog (bool aForce = true);
     void perDayNewVersionNotifier();
     void sltGUILanguageChange(QString strLang);
+    void sltProcessGlobalSettingChange();
 
 protected:
 
@@ -799,6 +825,15 @@ private:
     const char * vm_render_mode_str;
     bool mIsKWinManaged;
 
+    /** The --disable-patm option. */
+    bool mDisablePatm;
+    /** The --disable-csam option. */
+    bool mDisableCsam;
+    /** The --recompile-supervisor option. */
+    bool mRecompileSupervisor;
+    /** The --recompile-user option. */
+    bool mRecompileUser;
+
 #ifdef VBOX_WITH_DEBUGGER_GUI
     /** Whether the debugger should be accessible or not.
      * Use --dbg, the env.var. VBOX_GUI_DBG_ENABLED, --debug or the env.var.
@@ -853,6 +888,7 @@ private:
     QULongStringHash mAudioControllerTypes;
     QULongStringHash mNetworkAdapterTypes;
     QULongStringHash mNetworkAttachmentTypes;
+    QULongStringHash mNetworkAdapterPromiscModePolicyTypes;
     QULongStringHash mNATProtocolTypes;
     QULongStringHash mClipboardTypes;
     QULongStringHash mStorageControllerTypes;
