@@ -1,4 +1,4 @@
-/* $Id: UIMouseHandler.cpp $ */
+/* $Id: UIMouseHandler.cpp 38311 2011-08-04 13:08:39Z vboxsync $ */
 /** @file
  *
  * VBox frontends: Qt GUI ("VirtualBox"):
@@ -23,7 +23,7 @@
 
 /* Local includes */
 #include "VBoxGlobal.h"
-#include "VBoxProblemReporter.h"
+#include "UIMessageCenter.h"
 #include "UIKeyboardHandler.h"
 #include "UIMouseHandler.h"
 #include "UISession.h"
@@ -32,6 +32,10 @@
 #include "UIMachineView.h"
 #include "UIFrameBuffer.h"
 
+#ifdef Q_WS_WIN
+# include "VBoxUtils-win.h"
+#endif /* Q_WS_WIN */
+
 #ifdef Q_WS_X11
 # include <X11/XKBlib.h>
 # ifdef KeyPress
@@ -345,7 +349,7 @@ void UIMouseHandler::sltMouseCapabilityChanged()
 
     /* Notify user about mouse supports or not absolute pointing if that method was called by signal: */
     if (sender())
-        vboxProblem().remindAboutMouseIntegration(uisession()->isMouseSupportsAbsolute());
+        msgCenter().remindAboutMouseIntegration(uisession()->isMouseSupportsAbsolute());
 
     /* Notify all listeners: */
     emit mouseStateChanged(mouseState());
@@ -700,14 +704,11 @@ bool UIMouseHandler::mouseEvent(int iEventType, ulong uScreenId,
 #ifdef Q_WS_WIN
         /* Bringing mouse to the opposite side to simulate the endless moving: */
 
-        /* Acquiring visible viewport rectangle in local coordinates: */
-        QRect viewportRectangle = m_viewports[uScreenId]->visibleRegion().boundingRect();
+        /* Get visible-viewport-rectangle in global coordinates: */
+        QRect viewportRectangle = m_mouseCursorClippingRect;
+        /* Get top-left point of full-viewport-rectangle in global coordinates: */
         QPoint viewportRectangleGlobalPos = m_views[uScreenId]->mapToGlobal(m_viewports[uScreenId]->pos());
-        /* Shift viewport rectangle to global position to bound by available geometry: */
-        viewportRectangle.translate(viewportRectangleGlobalPos);
-        /* Acquiring viewport rectangle cropped by available geometry: */
-        viewportRectangle = viewportRectangle.intersected(QApplication::desktop()->availableGeometry());
-        /* Shift remaining viewport rectangle to local position as relative position is in local coordinates: */
+        /* Shift visible-viewport-rectangle to local position because relative position is in local coordinates: */
         viewportRectangle.translate(-viewportRectangleGlobalPos);
 
         /* Get boundaries: */
@@ -839,7 +840,7 @@ bool UIMouseHandler::mouseEvent(int iEventType, ulong uScreenId,
             {
                 if (uisession()->isPaused())
                 {
-                    vboxProblem().remindAboutPausedVMInput();
+                    msgCenter().remindAboutPausedVMInput();
                 }
                 else if (uisession()->isRunning())
                 {
@@ -847,7 +848,7 @@ bool UIMouseHandler::mouseEvent(int iEventType, ulong uScreenId,
                      * the capture state is to be defined by the dialog result itself: */
                     uisession()->setAutoCaptureDisabled(true);
                     bool autoConfirmed = false;
-                    bool ok = vboxProblem().confirmInputCapture(&autoConfirmed);
+                    bool ok = msgCenter().confirmInputCapture(&autoConfirmed);
                     if (autoConfirmed)
                         uisession()->setAutoCaptureDisabled(false);
                     /* Otherwise, the disable flag will be reset in the next console view's focus in event (since
@@ -881,13 +882,35 @@ void UIMouseHandler::updateMouseCursorClipping()
 
     if (uisession()->isMouseCaptured())
     {
-        /* Acquiring visible viewport rectangle: */
+        /* Get full-viewport-rectangle in local coordinates: */
         QRect viewportRectangle = m_viewports[m_iMouseCaptureViewIndex]->visibleRegion().boundingRect();
+        /* Get top-left point of full-viewport-rectangle in global coordinates: */
         QPoint viewportRectangleGlobalPos = m_views[m_iMouseCaptureViewIndex]->mapToGlobal(m_viewports[m_iMouseCaptureViewIndex]->pos());
+        /* Get full-viewport-rectangle in global coordinates: */
         viewportRectangle.translate(viewportRectangleGlobalPos);
+        /* Trim full-viewport-rectangle by available geometry: */
         viewportRectangle = viewportRectangle.intersected(QApplication::desktop()->availableGeometry());
+        /* Trim partial-viewport-rectangle by top-most windows: */
+        QRegion viewportRegion(viewportRectangle);
+        QRegion topMostRegion(NativeWindowSubsystem::areaCoveredByTopMostWindows());
+        viewportRegion -= topMostRegion;
+        /* Check if partial-viewport-region consists of 1 rectangle: */
+        if (viewportRegion.rectCount() > 1)
+        {
+            /* Choose the largest rectangle: */
+            QVector<QRect> rects = viewportRegion.rects();
+            QRect largestRect;
+            for (int i = 0; i < rects.size(); ++i)
+                largestRect = largestRect.width() * largestRect.height() < rects[i].width() * rects[i].height() ? rects[i] : largestRect;
+            /* Assign the partial-viewport-region to the largest rect: */
+            viewportRegion = largestRect;
+        }
+        /* Assign the partial-viewport-rectangle to the partial-viewport-region: */
+        viewportRectangle = viewportRegion.boundingRect();
+        /* Assign the visible-viewport-rectangle to the partial-viewport-rectangle: */
+        m_mouseCursorClippingRect = viewportRectangle;
         /* Prepare clipping area: */
-        RECT rect = { viewportRectangle.left() + 1, viewportRectangle.top() + 1, viewportRectangle.right(), viewportRectangle.bottom() };
+        RECT rect = { m_mouseCursorClippingRect.left() + 1, m_mouseCursorClippingRect.top() + 1, m_mouseCursorClippingRect.right(), m_mouseCursorClippingRect.bottom() };
         ::ClipCursor(&rect);
     }
     else
