@@ -1,4 +1,4 @@
-/* $Id: UIVMListView.cpp $ */
+/* $Id: UIVMListView.cpp 38311 2011-08-04 13:08:39Z vboxsync $ */
 /** @file
  *
  * VBox frontends: Qt GUI ("VirtualBox"):
@@ -23,7 +23,7 @@
 
 /* Local includes */
 #include "UIVMListView.h"
-#include "VBoxProblemReporter.h"
+#include "UIMessageCenter.h"
 #include "VBoxSelectorWnd.h"
 
 /* Global includes */
@@ -598,7 +598,7 @@ QPixmap UIVMListView::dragPixmap(const QModelIndex &index) const
     p.setPen(Qt::white);
     p.setFont(font());
     p.drawText(QRect(margin, margin + osTypeSize.height() + space,  s.width() - 2 * margin, nameSize.height()), Qt::AlignCenter, name);
-	/* Transparent icons are not supported on all platforms. */
+    /* Transparent icons are not supported on all platforms. */
 #if defined(Q_WS_MAC) || defined(Q_WS_WIN)
     p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
     p.fillRect(image.rect(), QColor(0, 0, 0, 177));
