@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,13 +25,14 @@
 #include "UIExportApplianceWzd.h"
 #include "UIIconPool.h"
 #include "UIImportApplianceWzd.h"
+#include "UICloneVMWizard.h"
 #include "UINewVMWzd.h"
 #include "UIVMDesktop.h"
 #include "UIVMListView.h"
 #include "UIVirtualBoxEventHandler.h"
 #include "VBoxGlobal.h"
 #include "VBoxMediaManagerDlg.h"
-#include "VBoxProblemReporter.h"
+#include "UIMessageCenter.h"
 #include "VBoxSelectorWnd.h"
 #include "UISettingsDialogSpecific.h"
 #include "UIToolBar.h"
@@ -40,6 +41,7 @@
 #include "UISelectorShortcuts.h"
 #include "UIDesktopServices.h"
 #include "UIGlobalSettingsExtension.h" /* extension pack installation */
+#include "UIActionPoolSelector.h"
 
 #ifdef VBOX_GUI_WITH_SYSTRAY
 # include "VBoxTrayIcon.h"
@@ -91,6 +93,9 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
     : QIWithRetranslateUI2<QMainWindow>(aParent, aFlags)
     , mDoneInaccessibleWarningOnce(false)
 {
+    /* Create offline action pool: */
+    UIActionPoolSelector::create();
+
     VBoxGlobalSettings settings = vboxGlobal().settings();
 
     if (aSelf)
@@ -140,6 +145,9 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
         QSize(32, 32), QSize(16, 16),
         ":/vm_settings_32px.png", ":/settings_16px.png",
         ":/vm_settings_disabled_32px.png", ":/settings_dis_16px.png"));
+    mVmCloneAction = new QAction(this);
+    mVmCloneAction->setIcon(UIIconPool::iconSet(
+        ":/vm_clone_16px.png", ":/vm_clone_disabled_16px.png"));
     mVmDeleteAction = new QAction(this);
     mVmDeleteAction->setIcon(UIIconPool::iconSetFull(
         QSize(32, 32), QSize(16, 16),
@@ -178,8 +186,6 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
     mVmCreateShortcut->setIcon(UIIconPool::iconSet(
         ":/vm_create_shortcut_16px.png", ":/vm_create_shortcut_disabled_16px.png"));
 
-    mHelpActions.setup(this);
-
     /* VM list toolbar */
     mVMToolBar = new UIToolBar(this);
     mVMToolBar->setContextMenuPolicy(Qt::CustomContextMenu);
@@ -193,7 +199,7 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
 #endif /* Q_WS_MAC */
 
     /* VM list view */
-    mVMModel = new UIVMItemModel();
+    mVMModel = new UIVMItemModel(this);
     mVMListView = new UIVMListView(mVMModel);
     mVMListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     connect(mVMListView, SIGNAL(sigUrlsDropped(QList<QUrl>)),
@@ -287,6 +293,7 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
     mVMMenu->addAction(mVmNewAction);
     mVMMenu->addAction(mVmAddAction);
     mVMMenu->addAction(mVmConfigAction);
+    mVMMenu->addAction(mVmCloneAction);
     mVMMenu->addAction(mVmDeleteAction);
     mVMMenu->addSeparator();
     mVMMenu->addAction(mVmStartAction);
@@ -305,6 +312,7 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
 
     mVMCtxtMenu = new QMenu(this);
     mVMCtxtMenu->addAction(mVmConfigAction);
+    mVMCtxtMenu->addAction(mVmCloneAction);
     mVMCtxtMenu->addAction(mVmDeleteAction);
     mVMCtxtMenu->addSeparator();
     mVMCtxtMenu->addAction(mVmStartAction);
@@ -322,8 +330,10 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
     connect(mVMCtxtMenu, SIGNAL(aboutToHide()),
             statusBar(), SLOT(clearMessage()));
 
-    mHelpMenu = menuBar()->addMenu(QString::null);
-    mHelpActions.addTo(mHelpMenu);
+    /* Prepare help menu: */
+    QMenu *pHelpMenu = gActionPool->action(UIActionIndex_Menu_Help)->menu();
+    prepareMenuHelp(pHelpMenu);
+    menuBar()->addMenu(pHelpMenu);
 
 #ifdef VBOX_GUI_WITH_SYSTRAY
     mTrayIcon = new VBoxTrayIcon(this, mVMModel);
@@ -434,6 +444,7 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
     connect(mVmAddAction, SIGNAL(triggered()), this, SLOT(vmAdd()));
 
     connect(mVmConfigAction, SIGNAL(triggered()), this, SLOT(vmSettings()));
+    connect(mVmCloneAction, SIGNAL(triggered()), this, SLOT(vmClone()));
     connect(mVmDeleteAction, SIGNAL(triggered()), this, SLOT(vmDelete()));
     connect(mVmStartAction, SIGNAL(triggered()), this, SLOT(vmStart()));
     connect(mVmDiscardAction, SIGNAL(triggered()), this, SLOT(vmDiscard()));
@@ -482,7 +493,7 @@ VBoxSelectorWnd(VBoxSelectorWnd **aSelf, QWidget* aParent,
 #endif
 
     /* Listen to potential downloaders signals: */
-    connect(&vboxProblem(), SIGNAL(sigDownloaderUserManualCreated()), this, SLOT(sltDownloaderUserManualEmbed()));
+    connect(&msgCenter(), SIGNAL(sigDownloaderUserManualCreated()), this, SLOT(sltDownloaderUserManualEmbed()));
 
     /* bring the VM list to the focus */
     mVMListView->setFocus();
@@ -551,6 +562,9 @@ VBoxSelectorWnd::~VBoxSelectorWnd()
 
     /* Delete the items from our model */
     mVMModel->clear();
+
+    /* Delete offline action pool: */
+    UIActionPoolSelector::destroy();
 }
 
 //
@@ -591,16 +605,18 @@ void VBoxSelectorWnd::fileExportAppliance()
 
 void VBoxSelectorWnd::fileSettings()
 {
-    VBoxGlobalSettings settings = vboxGlobal().settings();
-    CSystemProperties props = vboxGlobal().virtualBox().GetSystemProperties();
+    /* Check that we do NOT handling that already: */
+    if (mFileSettingsAction->data().toBool())
+        return;
+    /* Remember that we handling that already: */
+    mFileSettingsAction->setData(true);
 
-    UISettingsDialog *dlg = new UIGLSettingsDlg(this);
-    dlg->getFrom();
+    /* Create and execute global settings dialog: */
+    UISettingsDialogGlobal dlg(this);
+    dlg.execute();
 
-    if (dlg->exec() == QDialog::Accepted)
-        dlg->putBackTo();
-
-    delete dlg;
+    /* Remember that we do NOT handling that already: */
+    mFileSettingsAction->setData(false);
 }
 
 void VBoxSelectorWnd::fileExit()
@@ -686,32 +702,40 @@ void VBoxSelectorWnd::vmAdd(const QString &strFile /* = "" */)
                 mVMListView->setCurrentIndex(index);
             }
             else
-                vboxProblem().cannotReregisterMachine(this, strTmpFile, oldMachine.GetName());
+                msgCenter().cannotReregisterMachine(this, strTmpFile, oldMachine.GetName());
         }
         else
-            vboxProblem().cannotOpenMachine(this, strTmpFile, vbox);
+            msgCenter().cannotOpenMachine(this, strTmpFile, vbox);
     }
 }
 
 /**
  *  Opens the VM settings dialog.
  */
-void VBoxSelectorWnd::vmSettings(const QString &aCategory /* = QString::null */,
-                                 const QString &aControl /* = QString::null */,
-                                 const QString &aUuid /* = QString::null */)
+void VBoxSelectorWnd::vmSettings(const QString &strCategoryRef /* = QString::null */,
+                                 const QString &strControlRef /* = QString::null */,
+                                 const QString &strMachineId /* = QString::null */)
 {
-    if (!aCategory.isEmpty() && aCategory [0] != '#')
+    /* Check that we do NOT handling that already: */
+    if (mVmConfigAction->data().toBool())
+        return;
+    /* Remember that we handling that already: */
+    mVmConfigAction->setData(true);
+
+    /* Process href from VM details / description: */
+    if (!strCategoryRef.isEmpty() && strCategoryRef[0] != '#')
     {
-        /* Assume it's a href from the Details HTML */
-        vboxGlobal().openURL(aCategory);
+        vboxGlobal().openURL(strCategoryRef);
         return;
     }
-    QString strCategory = aCategory;
-    QString strControl = aControl;
-    /* Maybe the control is coded into the URL by %% */
-    if (aControl == QString::null)
+
+    /* Get category and control: */
+    QString strCategory = strCategoryRef;
+    QString strControl = strControlRef;
+    /* Check if control is coded into the URL by %%: */
+    if (strControl.isEmpty())
     {
-        QStringList parts = aCategory.split("%%");
+        QStringList parts = strCategory.split("%%");
         if (parts.size() == 2)
         {
             strCategory = parts.at(0);
@@ -719,44 +743,31 @@ void VBoxSelectorWnd::vmSettings(const QString &aCategory /* = QString::null */,
         }
     }
 
-    UIVMItem *item = aUuid.isNull() ? mVMListView->selectedItem() :
-                       mVMModel->itemById(aUuid);
+    /* Don't show the inaccessible warning if the user tries to open VM settings: */
+    mDoneInaccessibleWarningOnce = true;
+
+    /* Get corresponding VM item: */
+    UIVMItem *pItem = strMachineId.isNull() ? mVMListView->selectedItem() : mVMModel->itemById(strMachineId);
+    AssertMsgReturnVoid(pItem, ("Item must be always selected here!\n"));
+
+    /* Create and execute corresponding VM settings dialog: */
+    UISettingsDialogMachine dlg(this, pItem->id(), strCategory, strControl);
+    dlg.execute();
+
+    /* Remember that we do NOT handling that already: */
+    mVmConfigAction->setData(false);
+}
+
+void VBoxSelectorWnd::vmClone(const QString &aUuid /* = QString::null */)
+{
+    UIVMItem *item = aUuid.isNull() ? mVMListView->selectedItem() : mVMModel->itemById(aUuid);
 
     AssertMsgReturnVoid(item, ("Item must be always selected here"));
 
-    // open a direct session to modify VM settings
-    QString id = item->id();
-    CSession session = vboxGlobal().openSession(id);
-    if (session.isNull())
-        return;
+    CMachine machine = item->machine();
 
-    CMachine m = session.GetMachine();
-    AssertMsgReturn(!m.isNull(), ("Machine must not be null"), (void) 0);
-
-    /* Don't show the inaccessible warning if the user open the vm settings. */
-    mDoneInaccessibleWarningOnce = true;
-
-    UISettingsDialog *dlg = new UIVMSettingsDlg(this, m, strCategory, strControl);
-    dlg->getFrom();
-
-    if (dlg->exec() == QDialog::Accepted)
-    {
-        QString oldName = m.GetName();
-        dlg->putBackTo();
-
-        m.SaveSettings();
-        if (!m.isOk())
-            vboxProblem().cannotSaveMachineSettings(m);
-
-        /* To check use the result in future
-         * vboxProblem().cannotApplyMachineSettings(m, res); */
-    }
-
-    delete dlg;
-
-    mVMListView->setFocus();
-
-    session.UnlockMachine();
+    UICloneVMWizard wzd(this, machine);
+    wzd.exec();
 }
 
 void VBoxSelectorWnd::vmDelete(const QString &aUuid /* = QString::null */)
@@ -766,13 +777,9 @@ void VBoxSelectorWnd::vmDelete(const QString &aUuid /* = QString::null */)
     AssertMsgReturnVoid(item, ("Item must be always selected here"));
 
     CMachine machine = item->machine();
-    int rc = vboxProblem().confirmMachineDeletion(machine);
+    int rc = msgCenter().confirmMachineDeletion(machine);
     if (rc != QIMessageBox::Cancel)
     {
-#if defined(RT_OS_WINDOWS) && defined(RT_ARCH_AMD64)
-        /* XXX currently disabled due to a bug in ComSafeArrayIn on 64-bit Windows hosts! */
-        vboxProblem().showWin64Warning();
-#else
         if (rc == QIMessageBox::Yes)
         {
             /* Unregister and cleanup machine's data & hard-disks: */
@@ -783,21 +790,20 @@ void VBoxSelectorWnd::vmDelete(const QString &aUuid /* = QString::null */)
                 CProgress progress = machine.Delete(mediums);
                 if (machine.isOk())
                 {
-                    vboxProblem().showModalProgressDialog(progress, item->name(), ":/progress_delete_90px.png", 0, true);
+                    msgCenter().showModalProgressDialog(progress, item->name(), ":/progress_delete_90px.png", 0, true);
                     if (progress.GetResultCode() != 0)
-                        vboxProblem().cannotDeleteMachine(machine, progress);
+                        msgCenter().cannotDeleteMachine(machine, progress);
                 }
             }
             if (!machine.isOk())
-                vboxProblem().cannotDeleteMachine(machine);
+                msgCenter().cannotDeleteMachine(machine);
         }
         else
-#endif
         {
             /* Just unregister machine: */
             machine.Unregister(KCleanupMode_DetachAllReturnNone);
             if (!machine.isOk())
-                vboxProblem().cannotDeleteMachine(machine);
+                msgCenter().cannotDeleteMachine(machine);
         }
     }
 }
@@ -833,7 +839,7 @@ void VBoxSelectorWnd::vmDiscard(const QString &aUuid /* = QString::null */)
 
     AssertMsgReturnVoid(item, ("Item must be always selected here"));
 
-    if (!vboxProblem().confirmDiscardSavedState(item->machine()))
+    if (!msgCenter().confirmDiscardSavedState(item->machine()))
         return;
 
     /* open a session to modify VM settings */
@@ -843,7 +849,7 @@ void VBoxSelectorWnd::vmDiscard(const QString &aUuid /* = QString::null */)
     session.createInstance(CLSID_Session);
     if (session.isNull())
     {
-        vboxProblem().cannotOpenSession(session);
+        msgCenter().cannotOpenSession(session);
         return;
     }
 
@@ -852,14 +858,14 @@ void VBoxSelectorWnd::vmDiscard(const QString &aUuid /* = QString::null */)
         foundMachine.LockMachine(session, KLockType_Write);
     if (!vbox.isOk())
     {
-        vboxProblem().cannotOpenSession(vbox, item->machine());
+        msgCenter().cannotOpenSession(vbox, item->machine());
         return;
     }
 
     CConsole console = session.GetConsole();
     console.DiscardSavedState(true /* fDeleteFile */);
     if (!console.isOk())
-        vboxProblem().cannotDiscardSavedState(console);
+        msgCenter().cannotDiscardSavedState(console);
 
     session.UnlockMachine();
 }
@@ -888,9 +894,9 @@ void VBoxSelectorWnd::vmPause(bool aPause, const QString &aUuid /* = QString::nu
     if (!ok)
     {
         if (aPause)
-            vboxProblem().cannotPauseMachine(console);
+            msgCenter().cannotPauseMachine(console);
         else
-            vboxProblem().cannotResumeMachine(console);
+            msgCenter().cannotResumeMachine(console);
     }
 
     session.UnlockMachine();
@@ -1238,13 +1244,18 @@ void VBoxSelectorWnd::retranslateUi()
     mVmConfigAction->setToolTip(mVmConfigAction->text().remove('&').remove('.') +
         (mVmConfigAction->shortcut().toString().isEmpty() ? "" : QString(" (%1)").arg(mVmConfigAction->shortcut().toString())));
 
+    mVmCloneAction->setText(tr("Cl&one..."));
+    mVmCloneAction->setShortcut(gSS->keySequence(UISelectorShortcuts::CloneVMShortcut));
+    mVmCloneAction->setStatusTip(tr("Clone the selected virtual machine"));
+
     mVmDeleteAction->setText(tr("&Remove"));
     mVmDeleteAction->setShortcut(gSS->keySequence(UISelectorShortcuts::RemoveVMShortcut));
     mVmDeleteAction->setStatusTip(tr("Remove the selected virtual machine"));
 
     /* Note: mVmStartAction text is set up in vmListViewCurrentChanged() */
 
-    mVmDiscardAction->setText(tr("D&iscard"));
+    mVmDiscardAction->setIconText(tr("Discard"));
+    mVmDiscardAction->setText(tr("D&iscard Saved State"));
     mVmDiscardAction->setShortcut(gSS->keySequence(UISelectorShortcuts::DiscardVMShortcut));
     mVmDiscardAction->setStatusTip(
         tr("Discard the saved state of the selected virtual machine"));
@@ -1285,15 +1296,12 @@ void VBoxSelectorWnd::retranslateUi()
     mVmOpenInFileManagerAction->setShortcut(gSS->keySequence(UISelectorShortcuts::ShowVMInFileManagerShortcut));
     mVmCreateShortcut->setShortcut(gSS->keySequence(UISelectorShortcuts::CreateVMAliasShortcut));
 
-    mHelpActions.retranslateUi();
-
 #ifdef Q_WS_MAC
     mFileMenu->setTitle(tr("&File", "Mac OS X version"));
 #else /* Q_WS_MAC */
     mFileMenu->setTitle(tr("&File", "Non Mac OS X version"));
 #endif /* !Q_WS_MAC */
     mVMMenu->setTitle(tr("&Machine"));
-    mHelpMenu->setTitle(tr("&Help"));
 
 #ifdef VBOX_GUI_WITH_SYSTRAY
     if (vboxGlobal().isTrayMenu())
@@ -1332,7 +1340,7 @@ void VBoxSelectorWnd::vmListViewCurrentChanged(bool aRefreshDetails,
 
         KMachineState state = item->machineState();
         bool running = item->sessionState() != KSessionState_Unlocked;
-        bool modifyEnabled = !running && state != KMachineState_Saved;
+        bool modifyEnabled = state != KMachineState_Stuck && state != KMachineState_Saved /* for now! */;
 
         if (   aRefreshDetails
             || aRefreshDescription)
@@ -1344,6 +1352,7 @@ void VBoxSelectorWnd::vmListViewCurrentChanged(bool aRefreshDetails,
 
         /* enable/disable modify actions */
         mVmConfigAction->setEnabled(modifyEnabled);
+        mVmCloneAction->setEnabled(!running);
         mVmDeleteAction->setEnabled(!running);
         mVmDiscardAction->setEnabled(state == KMachineState_Saved && !running);
         mVmPauseAction->setEnabled(   state == KMachineState_Running
@@ -1443,7 +1452,7 @@ void VBoxSelectorWnd::vmListViewCurrentChanged(bool aRefreshDetails,
         {
             /* the VM is inaccessible */
             m_pVMDesktop->updateDetailsErrorText(
-                VBoxProblemReporter::formatErrorInfo(item->accessError()));
+                UIMessageCenter::formatErrorInfo(item->accessError()));
             mVmRefreshAction->setEnabled(true);
         }
         else
@@ -1474,6 +1483,7 @@ void VBoxSelectorWnd::vmListViewCurrentChanged(bool aRefreshDetails,
 
         /* disable modify actions */
         mVmConfigAction->setEnabled(false);
+        mVmCloneAction->setEnabled(false);
         mVmDeleteAction->setEnabled(item != NULL);
         mVmDiscardAction->setEnabled(false);
         mVmPauseAction->setEnabled(false);
@@ -1532,7 +1542,7 @@ void VBoxSelectorWnd::mediumEnumFinished(const VBoxMediaList &list)
             if ((*it).state() == KMediumState_Inaccessible)
                 break;
 
-        if (it != list.end() && vboxProblem().remindAboutInaccessibleMedia())
+        if (it != list.end() && msgCenter().remindAboutInaccessibleMedia())
         {
             /* Show the VDM dialog but don't refresh once more after a
              * just-finished refresh */
@@ -1656,7 +1666,7 @@ void VBoxSelectorWnd::sltDownloaderUserManualEmbed()
 {
     /* If there is User Manual downloader created => show the process bar: */
     if (UIDownloaderUserManual *pDl = UIDownloaderUserManual::current())
-        statusBar()->addWidget(pDl->processWidget(this), 0);
+        statusBar()->addWidget(pDl->progressWidget(this), 0);
 }
 
 void VBoxSelectorWnd::showViewContextMenu(const QPoint &pos)
@@ -1717,3 +1727,50 @@ void VBoxSelectorWnd::showViewContextMenu(const QPoint &pos)
     }
 }
 
+void VBoxSelectorWnd::prepareMenuHelp(QMenu *pMenu)
+{
+    /* Do not touch if filled already: */
+    if (!pMenu->isEmpty())
+        return;
+
+    /* Help submenu: */
+    pMenu->addAction(gActionPool->action(UIActionIndex_Simple_Help));
+    pMenu->addAction(gActionPool->action(UIActionIndex_Simple_Web));
+
+    pMenu->addSeparator();
+
+    pMenu->addAction(gActionPool->action(UIActionIndex_Simple_ResetWarnings));
+
+    pMenu->addSeparator();
+
+#ifdef VBOX_WITH_REGISTRATION
+    pMenu->addAction(gActionPool->action(UIActionIndex_Simple_Register));
+#endif /* VBOX_WITH_REGISTRATION */
+
+    pMenu->addAction(gActionPool->action(UIActionIndex_Simple_Update));
+
+#ifndef Q_WS_MAC
+    pMenu->addSeparator();
+#endif /* !Q_WS_MAC */
+
+    pMenu->addAction(gActionPool->action(UIActionIndex_Simple_About));
+
+    /* Configure connections: */
+    VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_Help), SIGNAL(triggered()),
+                        &msgCenter(), SLOT(sltShowHelpHelpDialog()));
+    VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_Web), SIGNAL(triggered()),
+                        &msgCenter(), SLOT(sltShowHelpWebDialog()));
+    VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_ResetWarnings), SIGNAL(triggered()),
+                        &msgCenter(), SLOT(sltResetSuppressedMessages()));
+#ifdef VBOX_WITH_REGISTRATION
+    VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_Register), SIGNAL(triggered()),
+                        &vboxGlobal(), SLOT(showRegistrationDialog()));
+    VBoxGlobal::connect(gEDataEvents, SIGNAL(sigCanShowRegistrationDlg(bool)),
+                        gActionPool->action(UIActionIndex_Simple_Register), SLOT(setEnabled(bool)));
+#endif /* VBOX_WITH_REGISTRATION */
+    VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_Update), SIGNAL(triggered()),
+                        &vboxGlobal(), SLOT(showUpdateDialog()));
+    VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_About), SIGNAL(triggered()),
+                        &msgCenter(), SLOT(sltShowHelpAboutDialog()));
+}
+
