@@ -1,4 +1,4 @@
-/* $Id: UIExportApplianceWzd.cpp $ */
+/* $Id: UIExportApplianceWzd.cpp 38343 2011-08-08 10:11:12Z vboxsync $ */
 /** @file
  *
  * VBox frontends: Qt4 GUI ("VirtualBox"):
@@ -23,7 +23,7 @@
 /* Local includes */
 #include "UIExportApplianceWzd.h"
 #include "VBoxGlobal.h"
-#include "VBoxProblemReporter.h"
+#include "UIMessageCenter.h"
 
 class VMListWidgetItems : public QListWidgetItem
 {
@@ -93,6 +93,7 @@ void UIExportApplianceWzd::retranslateUi()
 
     /* Extra buttons */
     setButtonText(QWizard::CustomButton1, tr("Restore Defaults"));
+    setButtonText(QWizard::FinishButton, tr("Export"));
 }
 
 void UIExportApplianceWzd::sltCurrentIdChanged(int iId)
@@ -172,7 +173,7 @@ bool UIExportApplianceWzdPage1::validatePage()
     }
 
     if (!savedMachines.isEmpty())
-        return vboxProblem().confirmExportMachinesInSaveState(savedMachines, this);
+        return msgCenter().confirmExportMachinesInSaveState(savedMachines, this);
 
     return true;
 }
@@ -452,9 +453,6 @@ void UIExportApplianceWzdPage3::initializePage()
             strName = QDir::toNativeSeparators(QString("%1/%2").arg(vboxGlobal().documentsPath()).arg(strName));
         m_pFileSelector->setPath(strName);
     }
-    AssertMsg(!field("applianceWidget").value<ExportAppliancePointer>().isNull(),
-              ("Appliance Widget Pointer is not set!\n"));
-    field("applianceWidget").value<ExportAppliancePointer>()->prepareExport();
 }
 
 bool UIExportApplianceWzdPage3::isComplete() const
@@ -488,7 +486,6 @@ UIExportApplianceWzdPage4::UIExportApplianceWzdPage4()
 
     /* Register 'applianceWidget' field! */
     registerField("applianceWidget", this, "applianceWidget");
-    m_pApplianceWidget = m_pSettingsCnt;
 }
 
 void UIExportApplianceWzdPage4::retranslateUi()
@@ -525,7 +522,7 @@ bool UIExportApplianceWzdPage4::validatePage()
 bool UIExportApplianceWzdPage4::prepareSettingsWidget()
 {
     CVirtualBox vbox = vboxGlobal().virtualBox();
-    CAppliance *appliance = m_pSettingsCnt->init();
+    CAppliance *appliance = m_pApplianceWidget->init();
     bool fResult = appliance->isOk();
     if (fResult)
     {
@@ -543,7 +540,7 @@ bool UIExportApplianceWzdPage4::prepareSettingsWidget()
                 fResult = m.isOk();
                 if (!fResult)
                 {
-                    vboxProblem().cannotExportAppliance(m, appliance, this);
+                    msgCenter().cannotExportAppliance(m, appliance, this);
                     return false;
                 }
                 /* Now add some new fields the user may change */
@@ -558,18 +555,19 @@ bool UIExportApplianceWzdPage4::prepareSettingsWidget()
                 break;
         }
         /* Make sure the settings widget get the new descriptions */
-        m_pSettingsCnt->populate();
+        m_pApplianceWidget->populate();
     }
     if (!fResult)
-        vboxProblem().cannotExportAppliance(appliance, this);
+        msgCenter().cannotExportAppliance(appliance, this);
     return fResult;
 }
 
 bool UIExportApplianceWzdPage4::exportAppliance()
 {
-    AssertMsg(!field("applianceWidget").value<ExportAppliancePointer>().isNull(),
-              ("Appliance Widget Pointer is not set!\n"));
-    CAppliance *appliance = field("applianceWidget").value<ExportAppliancePointer>()->appliance();
+    /* Fetch all settings from the appliance editor. */
+    m_pApplianceWidget->prepareExport();
+    /* Get the appliance. */
+    CAppliance *appliance = m_pApplianceWidget->appliance();
     /* We need to know every filename which will be created, so that we can
      * ask the user for confirmation of overwriting. For that we iterating
      * over all virtual systems & fetch all descriptions of the type
@@ -599,18 +597,18 @@ bool UIExportApplianceWzdPage4::exportAppliance()
     if (fResult)
     {
         /* Show some progress, so the user know whats going on */
-        vboxProblem().showModalProgressDialog(progress, tr("Checking files ..."), "", this);
+        msgCenter().showModalProgressDialog(progress, tr("Checking files ..."), "", this);
         if (progress.GetCanceled())
             return false;
         if (!progress.isOk() || progress.GetResultCode() != 0)
         {
-            vboxProblem().cannotCheckFiles(progress, this);
+            msgCenter().cannotCheckFiles(progress, this);
             return false;
         }
     }
     QVector<QString> exists = explorer.Exists(files);
     /* Check if the file exists already, if yes get confirmation for overwriting from the user. */
-    if (!vboxProblem().askForOverridingFiles(exists, this))
+    if (!msgCenter().askForOverridingFiles(exists, this))
         return false;
     /* Ok all is confirmed so delete all the files which exists */
     if (!exists.isEmpty())
@@ -620,12 +618,12 @@ bool UIExportApplianceWzdPage4::exportAppliance()
         if (fResult)
         {
             /* Show some progress, so the user know whats going on */
-            vboxProblem().showModalProgressDialog(progress1, tr("Removing files ..."), "", this);
+            msgCenter().showModalProgressDialog(progress1, tr("Removing files ..."), "", this);
             if (progress1.GetCanceled())
                 return false;
             if (!progress1.isOk() || progress1.GetResultCode() != 0)
             {
-                vboxProblem().cannotRemoveFiles(progress1, this);
+                msgCenter().cannotRemoveFiles(progress1, this);
                 return false;
             }
         }
@@ -656,19 +654,19 @@ bool UIExportApplianceWzdPage4::exportVMs(CAppliance &appliance)
     if (fResult)
     {
         /* Show some progress, so the user know whats going on */
-        vboxProblem().showModalProgressDialog(progress, tr("Exporting Appliance ..."), ":/progress_export_90px.png", this, true);
+        msgCenter().showModalProgressDialog(progress, tr("Exporting Appliance ..."), ":/progress_export_90px.png", this, true);
         if (progress.GetCanceled())
             return false;
         if (!progress.isOk() || progress.GetResultCode() != 0)
         {
-            vboxProblem().cannotExportAppliance(progress, &appliance, this);
+            msgCenter().cannotExportAppliance(progress, &appliance, this);
             return false;
         }
         else
             return true;
     }
     if (!fResult)
-        vboxProblem().cannotExportAppliance(&appliance, this);
+        msgCenter().cannotExportAppliance(&appliance, this);
     return false;
 }
 
