@@ -1,4 +1,4 @@
-/* $Id: vboxhgcm.c $ */
+/* $Id: vboxhgcm.c 38314 2011-08-04 14:05:58Z vboxsync $ */
 
 /** @file
  * VBox HGCM connection
@@ -336,21 +336,21 @@ DECLINLINE(PCRVBOXHGSMI_CLIENT) _crVBoxHGSMIClientGet(CRConnection *conn)
 {
 #ifdef VBOX_CRHGSMI_WITH_D3DDEV
     PCRVBOXHGSMI_CLIENT pClient = (PCRVBOXHGSMI_CLIENT)VBoxCrHgsmiQueryClient();
-    Assert(pClient);
+    CRASSERT(pClient);
     return pClient;
 #else
     if (conn->HgsmiClient.pHgsmi)
         return &conn->HgsmiClient;
     {
         PVBOXUHGSMI pHgsmi = VBoxCrHgsmiCreate();
-        Assert(pHgsmi);
+        CRASSERT(pHgsmi);
         if (pHgsmi)
         {
             int rc = _crVBoxHGSMIClientInit(&conn->HgsmiClient, pHgsmi);
             AssertRC(rc);
             if (RT_SUCCESS(rc))
             {
-                Assert(conn->HgsmiClient.pHgsmi);
+                CRASSERT(conn->HgsmiClient.pHgsmi);
                 return &conn->HgsmiClient;
             }
             VBoxCrHgsmiDestroy(pHgsmi);
@@ -502,7 +502,7 @@ DECLINLINE(void*) _crVBoxHGSMIRecvBufData(PCRVBOXHGSMI_CLIENT pClient, uint32_t
 {
     VBOXUHGSMI_BUFFER_LOCK_FLAGS fFlags;
     int rc;
-    Assert(!pClient->pvHGBuffer);
+    CRASSERT(!pClient->pvHGBuffer);
     fFlags.Value = 0;
     rc = pClient->pHGBuffer->pfnLock(pClient->pHGBuffer, 0, cbBuffer, fFlags, &pClient->pvHGBuffer);
     AssertRC(rc);
@@ -535,7 +535,7 @@ DECLINLINE(void) _crVBoxHGSMIFillCmd(VBOXUHGSMI_BUFFER_SUBMIT *pSubm, PCRVBOXHGS
 DECLINLINE(void) _crVBoxHGSMIWaitCmd(PCRVBOXHGSMI_CLIENT pClient)
 {
     int rc = CRVBOXHGSMI_BUF_WAIT(pClient->pCmdBuffer);
-    Assert(rc == 0);
+    CRASSERT(rc == 0);
 }
 #endif
 
@@ -600,7 +600,7 @@ static int crVBoxHGCMCall(void *pvData, unsigned cbData)
 #ifdef IN_GUEST
 
 # ifdef RT_OS_WINDOWS
-    DWORD cbReturned;
+    DWORD cbReturned, lerr;
 
     if (DeviceIoControl (g_crvboxhgcm.hGuestDrv,
                          VBOXGUEST_IOCTL_HGCM_CALL(cbData),
@@ -611,8 +611,19 @@ static int crVBoxHGCMCall(void *pvData, unsigned cbData)
     {
         return VINF_SUCCESS;
     }
-    crDebug("vboxCall failed with %x\n", GetLastError());
-    return VERR_NOT_SUPPORTED;
+    lerr=GetLastError();
+    crDebug("vboxCall failed with %x\n", lerr);
+    /*On windows if we exceed max buffer len, we only get ERROR_GEN_FAILURE, and parms.hdr.result isn't changed.
+     *Before every call here we set it to VERR_WRONG_ORDER, so checking it here as well.
+     */
+    if (ERROR_GEN_FAILURE==lerr && VERR_WRONG_ORDER==((VBoxGuestHGCMCallInfo*)pvData)->result)
+    {
+        return VERR_OUT_OF_RANGE;
+    }
+    else
+    {
+        return VERR_NOT_SUPPORTED;
+    }
 # else
     int rc;
 #  if defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
@@ -808,7 +819,13 @@ static void *crVBoxHGCMAlloc(CRConnection *conn)
 {
     void *pvBuff;
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
     pvBuff = _crVBoxHGCMAlloc(conn);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
     return pvBuff;
 }
@@ -865,7 +882,13 @@ static void _crVBoxHGCMWriteExact(CRConnection *conn, const void *buf, unsigned
 static void crVBoxHGCMWriteExact(CRConnection *conn, const void *buf, unsigned int len)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
     _crVBoxHGCMWriteExact(conn, buf, len);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -945,7 +968,7 @@ crVBoxHGCMWriteReadExact(CRConnection *conn, const void *buf, unsigned int len,
 
     rc = crVBoxHGCMCall(&parms, sizeof(parms));
 
-#ifdef RT_OS_LINUX
+#if defined(RT_OS_LINUX) || defined(RT_OS_WINDOWS)
     if (VERR_OUT_OF_RANGE==rc && CR_VBOXHGCM_USERALLOCATED==bufferKind)
     {
         /*Buffer is too big, so send it in split chunks*/
@@ -1057,6 +1080,10 @@ static void crVBoxHGCMSend(CRConnection *conn, void **bufp,
     CRVBOXHGCMBUFFER *hgcm_buffer;
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
 
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
+
     if (!bufp) /* We're sending a user-allocated buffer. */
     {
 #ifndef IN_GUEST
@@ -1067,6 +1094,9 @@ static void crVBoxHGCMSend(CRConnection *conn, void **bufp,
             CRASSERT(!conn->u32InjectClientID);
             crDebug("SHCRGL: sending userbuf with %d bytes\n", len);
             crVBoxHGCMWriteReadExact(conn, start, len, CR_VBOXHGCM_USERALLOCATED);
+#endif
+#ifdef CHROMIUM_THREADSAFE
+            crUnlockMutex(&g_crvboxhgcm.mutex);
 #endif
             VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
         return;
@@ -1107,6 +1137,10 @@ static void crVBoxHGCMSend(CRConnection *conn, void **bufp,
      */
     *bufp = NULL;
 
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
+
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -1147,7 +1181,13 @@ static void crVBoxHGCMPollHost(CRConnection *conn)
 static void crVBoxHGCMSingleRecv(CRConnection *conn, void *buf, unsigned int len)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
     crVBoxHGCMReadExact(conn, buf, len);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -1193,7 +1233,13 @@ static void _crVBoxHGCMFree(CRConnection *conn, void *buf)
 static void crVBoxHGCMFree(CRConnection *conn, void *buf)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
     _crVBoxHGCMFree(conn, buf);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -1278,7 +1324,13 @@ static void _crVBoxHGCMReceiveMessage(CRConnection *conn)
 static void crVBoxHGCMReceiveMessage(CRConnection *conn)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
     _crVBoxHGCMReceiveMessage(conn);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -1315,8 +1367,6 @@ static int crVBoxHGCMSetVersion(CRConnection *conn, unsigned int vMajor, unsigne
 
     if (RT_FAILURE(rc) || RT_FAILURE(parms.hdr.result))
     {
-        Assert(0);
-
         crWarning("Host doesn't accept our version %d.%d. Make sure you have appropriate additions installed!",
                   parms.vMajor.u.value32, parms.vMinor.u.value32);
         return FALSE;
@@ -1345,8 +1395,6 @@ static int crVBoxHGCMSetPID(CRConnection *conn, unsigned long long pid)
 
     if (RT_FAILURE(rc) || RT_FAILURE(parms.hdr.result))
     {
-        Assert(0);
-
         crWarning("SHCRGL_GUEST_FN_SET_PID failed!");
         return FALSE;
     }
@@ -1384,7 +1432,6 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
         /* @todo check if we could rollback to softwareopengl */
         if (g_crvboxhgcm.hGuestDrv == INVALID_HANDLE_VALUE)
         {
-            Assert(0);
             crDebug("could not open VBox Guest Additions driver! rc = %d\n", GetLastError());
             VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
             return FALSE;
@@ -1399,7 +1446,6 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
         {
             crDebug("could not open Guest Additions kernel module! rc = %d\n", errno);
             VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
-            Assert(0);
             return FALSE;
         }
     }
@@ -1451,7 +1497,6 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
         else
         {
             crDebug("HGCM connect failed with rc=0x%x\n", info.result);
-            Assert(0);
 
             VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
             return FALSE;
@@ -1461,7 +1506,6 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
     {
 #ifdef RT_OS_WINDOWS
         DWORD winEr = GetLastError();
-        Assert(0);
         crDebug("IOCTL for HGCM connect failed with rc=0x%x\n", winEr);
 #else
         crDebug("IOCTL for HGCM connect failed with rc=0x%x\n", errno);
@@ -1588,8 +1632,14 @@ static void crVBoxHGCMDoDisconnect( CRConnection *conn )
 static void crVBoxHGCMInstantReclaim(CRConnection *conn, CRMessage *mess)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
     _crVBoxHGCMFree(conn, mess);
     CRASSERT(FALSE);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -1628,7 +1678,7 @@ bool _crVBoxHGSMIInit()
         crDebug("CrHgsmi is %s", bHasHGSMI ? "ENABLED" : "DISABLED");
     }
 
-    Assert(bHasHGSMI >= 0);
+    CRASSERT(bHasHGSMI >= 0);
 
     return bHasHGSMI;
 }
@@ -1646,7 +1696,7 @@ static void *_crVBoxHGSMIDoAlloc(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient
     int rc;
 
     buf = _crVBoxHGSMIBufAlloc(pClient, CRVBOXHGSMI_BUF_SIZE(cbSize));
-    Assert(buf);
+    CRASSERT(buf);
     if (buf)
     {
         VBOXUHGSMI_BUFFER_LOCK_FLAGS fFlags;
@@ -1697,17 +1747,25 @@ static void *crVBoxHGSMIAlloc(CRConnection *conn)
 
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
 
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
+
     pClient = _crVBoxHGSMIClientGet(conn);
     if (pClient)
     {
         pvBuf = _crVBoxHGSMIDoAlloc(conn, pClient);
-        Assert(pvBuf);
+        CRASSERT(pvBuf);
     }
     else
     {
         pvBuf = _crVBoxHGCMAlloc(conn);
     }
 
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
+
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 
     return pvBuf;
@@ -1716,7 +1774,13 @@ static void *crVBoxHGSMIAlloc(CRConnection *conn)
 static void crVBoxHGSMIFree(CRConnection *conn, void *buf)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
     _crVBoxHGSMIFree(conn, buf);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -1728,7 +1792,7 @@ static void _crVBoxHGSMIPollHost(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient
     PVBOXUHGSMI_BUFFER pRecvBuffer;
     uint32_t cbBuffer;
 
-    Assert(parms);
+    CRASSERT(parms);
 
     parms->hdr.result      = VERR_WRONG_ORDER;
     parms->hdr.u32ClientID = conn->u32ClientID;
@@ -1742,7 +1806,7 @@ static void _crVBoxHGSMIPollHost(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient
     _crVBoxHGSMICmdBufferUnlock(pClient);
 
     pRecvBuffer = _crVBoxHGSMIRecvBufGet(pClient);
-    Assert(pRecvBuffer);
+    CRASSERT(pRecvBuffer);
     if (!pRecvBuffer)
         return;
 
@@ -1765,7 +1829,7 @@ static void _crVBoxHGSMIPollHost(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient
     _crVBoxHGSMIWaitCmd(pClient);
 
     parms = (CRVBOXHGSMIREAD *)_crVBoxHGSMICmdBufferLockRo(pClient, sizeof (*parms));
-    Assert(parms);
+    CRASSERT(parms);
     if (!parms)
     {
         crWarning("_crVBoxHGSMICmdBufferLockRo failed\n");
@@ -1782,7 +1846,7 @@ static void _crVBoxHGSMIPollHost(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient
     if (cbBuffer)
     {
         void *pvData = _crVBoxHGSMIRecvBufData(pClient, cbBuffer);
-        Assert(pvData);
+        CRASSERT(pvData);
         if (pvData)
         {
             conn->pBuffer  = pvData;
@@ -1830,7 +1894,7 @@ _crVBoxHGSMIWriteReadExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient, void
     {
         void *pvBuf;
         pBuf = _crVBoxHGSMIBufAlloc(pClient, len);
-        Assert(pBuf);
+
         if (!pBuf)
         {
             /* fallback */
@@ -1838,7 +1902,7 @@ _crVBoxHGSMIWriteReadExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient, void
             return;
         }
 
-        Assert(!offBuffer);
+        CRASSERT(!offBuffer);
 
         offBuffer = 0;
         fFlags.Value = 0;
@@ -1869,7 +1933,7 @@ _crVBoxHGSMIWriteReadExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient, void
     do
     {
         PVBOXUHGSMI_BUFFER pRecvBuffer = _crVBoxHGSMIRecvBufGet(pClient);
-        Assert(pRecvBuffer);
+        CRASSERT(pRecvBuffer);
         if (!pRecvBuffer)
         {
             break;
@@ -1899,7 +1963,7 @@ _crVBoxHGSMIWriteReadExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient, void
         _crVBoxHGSMIWaitCmd(pClient);
 
         parms = (CRVBOXHGSMIWRITEREAD *)_crVBoxHGSMICmdBufferLockRo(pClient, sizeof (*parms));
-        Assert(parms);
+        CRASSERT(parms);
         if (parms)
         {
             uint32_t cbWriteback = parms->cbWriteback;
@@ -1913,7 +1977,7 @@ _crVBoxHGSMIWriteReadExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient, void
                 if (cbWriteback)
                 {
                     void *pvData = _crVBoxHGSMIRecvBufData(pClient, cbWriteback);
-                    Assert(pvData);
+                    CRASSERT(pvData);
                     if (pvData)
                     {
                         conn->pBuffer  = pvData;
@@ -1925,7 +1989,7 @@ _crVBoxHGSMIWriteReadExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClient, void
             else if (VERR_BUFFER_OVERFLOW == rc)
             {
                 PVBOXUHGSMI_BUFFER pOldBuf = pClient->pHGBuffer;
-                Assert(!pClient->pvHGBuffer);
+                CRASSERT(!pClient->pvHGBuffer);
                 CRASSERT(cbWriteback>pClient->pHGBuffer->cbBuffer);
                 crDebug("Reallocating host buffer from %d to %d bytes", conn->cbHostBufferAllocated, cbWriteback);
 
@@ -1976,7 +2040,7 @@ static void _crVBoxHGSMIWriteExact(CRConnection *conn, PCRVBOXHGSMI_CLIENT pClie
     if (conn->u32InjectClientID)
     {
         CRVBOXHGSMIINJECT *parms = (CRVBOXHGSMIINJECT *)_crVBoxHGSMICmdBufferLock(pClient, sizeof (*parms));
-        Assert(parms);
+        CRASSERT(parms);
         if (!parms)
         {
             return;
@@ -2073,6 +2137,10 @@ static void crVBoxHGSMISend(CRConnection *conn, void **bufp,
 
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
 
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
+
     if (!bufp) /* We're sending a user-allocated buffer. */
     {
         pClient = _crVBoxHGSMIClientGet(conn);
@@ -2086,6 +2154,9 @@ static void crVBoxHGSMISend(CRConnection *conn, void **bufp,
             CRASSERT(!conn->u32InjectClientID);
             crDebug("SHCRGL: sending userbuf with %d bytes\n", len);
             _crVBoxHGSMIWriteReadExact(conn, pClient, (void*)start, 0, len, false);
+#endif
+#ifdef CHROMIUM_THREADSAFE
+            crUnlockMutex(&g_crvboxhgcm.mutex);
 #endif
             VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
             return;
@@ -2093,16 +2164,28 @@ static void crVBoxHGSMISend(CRConnection *conn, void **bufp,
 
         /* fallback */
         crVBoxHGCMSend(conn, bufp, start, len);
+#ifdef CHROMIUM_THREADSAFE
+        crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
         VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
         return;
     }
 
     hgcm_buffer = (CRVBOXHGCMBUFFER *) *bufp - 1;
-    Assert(hgcm_buffer->magic == CR_VBOXHGCM_BUFFER_MAGIC);
+    CRASSERT(hgcm_buffer->magic == CR_VBOXHGCM_BUFFER_MAGIC);
+    if (hgcm_buffer->magic != CR_VBOXHGCM_BUFFER_MAGIC)
+    {
+        crError("HGCM buffer magic mismatch");
+    }
+
+
     if (hgcm_buffer->kind != CR_VBOXHGCM_UHGSMI_BUFFER)
     {
         /* fallback */
         crVBoxHGCMSend(conn, bufp, start, len);
+#ifdef CHROMIUM_THREADSAFE
+        crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
         VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
         return;
     }
@@ -2113,15 +2196,22 @@ static void crVBoxHGSMISend(CRConnection *conn, void **bufp,
      */
 
     pBuf = _crVBoxHGSMIBufFromHdr(hgcm_buffer);
-    Assert(pBuf);
+    CRASSERT(pBuf);
     if (!pBuf)
     {
         crVBoxHGCMSend(conn, bufp, start, len);
+#ifdef CHROMIUM_THREADSAFE
+        crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
         VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
         return;
     }
 
     pClient = (PCRVBOXHGSMI_CLIENT)pBuf->pvUserData;
+    if (pClient != &conn->HgsmiClient)
+    {
+        crError("HGSMI client mismatch");
+    }
 
     /* Length would be passed as part of HGCM pointer description
      * No need to prepend it to the buffer
@@ -2145,41 +2235,28 @@ static void crVBoxHGSMISend(CRConnection *conn, void **bufp,
      */
     *bufp = NULL;
 
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
+
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
 static void crVBoxHGSMIWriteExact(CRConnection *conn, const void *buf, unsigned int len)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
-    Assert(0);
 
     CRASSERT(0);
-//    PCRVBOXHGSMI_CLIENT pClient;
-//    PVBOXUHGSMI_BUFFER pBuf = _crVBoxHGSMIBufFromMemPtr(buf);
-//    if (!pBuf)
-//        return;
-//    pClient = (PCRVBOXHGSMI_CLIENT)pBuf->pvUserData;
-//    _crVBoxHGSMIWriteExact(conn, pClient, pBuf, 0, len);
-//    _crVBoxHGSMIBufFree(pClient, pBuf);
+
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
 static void crVBoxHGSMISingleRecv(CRConnection *conn, void *buf, unsigned int len)
 {
-//    PCRVBOXHGSMI_CLIENT pClient;
-//    PVBOXUHGSMI_BUFFER pBuf;
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
-//    pBuf = _crVBoxHGSMIBufFromMemPtr(buf);
-//    Assert(pBuf);
-    Assert(0);
+
     CRASSERT(0);
-//    if (!pBuf)
-//    {
-//        VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
-//        return;
-//    }
-//    pClient = (PCRVBOXHGSMI_CLIENT)pBuf->pvUserData;
-//    _crVBoxHGSMIReadExact(conn, pClient);
+
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -2187,9 +2264,18 @@ static void crVBoxHGSMIReceiveMessage(CRConnection *conn)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
 
-    Assert(0);
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
+
+    CRASSERT(0);
 
     _crVBoxHGCMReceiveMessage(conn);
+
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
+
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
@@ -2199,7 +2285,7 @@ static void crVBoxHGSMIReceiveMessage(CRConnection *conn)
 static void crVBoxHGSMIAccept( CRConnection *conn, const char *hostname, unsigned short port )
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
-    Assert(0);
+    CRASSERT(0);
 
     CRASSERT(conn && conn->pHostBuffer);
 #ifdef IN_GUEST
@@ -2215,34 +2301,44 @@ static int crVBoxHGSMIDoConnect( CRConnection *conn )
 
 static void crVBoxHGSMIDoDisconnect( CRConnection *conn )
 {
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
 #ifndef VBOX_CRHGSMI_WITH_D3DDEV
     if (conn->HgsmiClient.pHgsmi)
     {
         PVBOXUHGSMI pHgsmi;
         _crVBoxHGSMIClientTerm(&conn->HgsmiClient, &pHgsmi);
-        Assert(pHgsmi);
+        CRASSERT(pHgsmi);
         VBoxCrHgsmiDestroy(pHgsmi);
     }
-
 #endif
     crVBoxHGCMDoDisconnect(conn);
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
 }
 
 static void crVBoxHGSMIInstantReclaim(CRConnection *conn, CRMessage *mess)
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
-    Assert(0);
+#ifdef CHROMIUM_THREADSAFE
+    crLockMutex(&g_crvboxhgcm.mutex);
+#endif
+    CRASSERT(0);
 
     _crVBoxHGSMIFree(conn, mess);
-    CRASSERT(FALSE);
 
+#ifdef CHROMIUM_THREADSAFE
+    crUnlockMutex(&g_crvboxhgcm.mutex);
+#endif
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
 }
 
 static void crVBoxHGSMIHandleNewMessage( CRConnection *conn, CRMessage *msg, unsigned int len )
 {
     VBOXCRHGSMIPROFILE_FUNC_PROLOGUE();
-    Assert(0);
+    CRASSERT(0);
 
     CRASSERT(FALSE);
     VBOXCRHGSMIPROFILE_FUNC_EPILOGUE();
