@@ -1,4 +1,4 @@
-/* $Id: SUPR3HardenedVerify.cpp $ */
+/* $Id: SUPR3HardenedVerify.cpp 37344 2011-06-07 12:23:53Z vboxsync $ */
 /** @file
  * VirtualBox Support Library - Verification of Hardened Installation.
  */
@@ -1186,11 +1186,10 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
            full access. So, to work around we relax the hardening a bit and
            permit grand parents and beyond to be group writable by admin. */
         if (pFsObjState->Stat.st_gid != 80 /*admin*/) /** @todo dynamically resolve the admin group? */
-#endif
-#ifdef RT_OS_FREEBSD
-        /* PC-BSD 9 has group-writable application directory, similar to OSX and
-           their Applications directory */
-        if (pFsObjState->Stat.st_gid != 5 /*operators*/) /* Allow operators group-writes */
+#elif defined(RT_OS_FREEBSD)
+        /* HACK ALERT: PC-BSD 9 has group-writable application directory,
+           similar to OS X and their /Applications directory (see above). */
+        if (pFsObjState->Stat.st_gid != 5 /*operators*/)
 #endif
             return supR3HardenedSetError3(VERR_SUPLIB_WRITE_NON_SYS_GROUP, pErrInfo,
                                           "The group is not a system group and it has write access to '", pszPath, "'");
