@@ -1,4 +1,4 @@
-/* $Id: VBoxNetFltInternal.h $ */
+/* $Id: VBoxNetFltInternal.h 38408 2011-08-10 20:07:04Z vboxsync $ */
 /** @file
  * VBoxNetFlt - Network Filter Driver (Host), Internal Header.
  */
@@ -183,8 +183,6 @@ typedef struct VBOXNETFLTINS
             bool fIsVNICTemplate;
             /** Handle to list of created VNICs. */
             list_t hVNICs;
-            /** Instance number while creating VNICs. */
-            uint64_t uInstance;
             /** The MAC address of the host interface. */
             RTMAC MacAddr;
             /** Handle of this interface (lower MAC). */
@@ -242,7 +240,7 @@ typedef struct VBOXNETFLTINS
             /** @name Windows instance data.
              * @{ */
             /** Filter driver device context. */
-            ADAPT IfAdaptor;
+            VBOXNETFLTWIN WinIf;
 
             volatile uint32_t cModeNetFltRefs;
             volatile uint32_t cModePassThruRefs;
@@ -252,8 +250,8 @@ typedef struct VBOXNETFLTINS
 #endif
             /** The MAC address of the interface. Caching MAC for performance reasons. */
             RTMAC MacAddr;
-            /** mutex used to synchronize ADAPT init/deinit */
-            RTSEMMUTEX hAdaptMutex;
+            /** mutex used to synchronize WinIf init/deinit */
+            RTSEMMUTEX hWinIfMutex;
             /** @}  */
 # else
 #  error "PORTME"
