@@ -1,4 +1,4 @@
-/* $Id: initterm.cpp $ */
+/* $Id: initterm.cpp 35640 2011-01-19 20:44:02Z vboxsync $ */
 
 /** @file
  * MS COM / XPCOM Abstraction Layer - Initialization and Termination.
@@ -246,87 +246,30 @@ static uint32_t gCOMMainInitCount = 0;
  *
  * @return S_OK on success and a COM result code in case of failure.
  */
-HRESULT Initialize()
+HRESULT Initialize(bool fGui)
 {
     HRESULT rc = E_FAIL;
 
 #if !defined(VBOX_WITH_XPCOM)
 
-    DWORD flags = COINIT_MULTITHREADED
+    /*
+     * We initialize COM in GUI thread in STA, to be compliant with QT and
+     * OLE requirments (for example to allow D&D), while other threads
+     * initialized in regular MTA. To allow fast proxyless access from
+     * GUI thread to COM objects, we explicitly provide our COM objects
+     * with free threaded marshaller.
+     * !!!!! Please think twice before touching this code !!!!!
+     */
+    DWORD flags = fGui ?
+                  COINIT_APARTMENTTHREADED
+                | COINIT_SPEED_OVER_MEMORY
+                :
+                  COINIT_MULTITHREADED
                 | COINIT_DISABLE_OLE1DDE
                 | COINIT_SPEED_OVER_MEMORY;
 
     rc = CoInitializeEx(NULL, flags);
 
-    /// @todo the below rough method of changing the apartment type doesn't
-    /// work on some systems for unknown reason (CoUninitialize() simply does
-    /// nothing there, or at least all 10 000 of subsequent CoInitializeEx()
-    /// continue to return RPC_E_CHANGED_MODE there). The problem on those
-    /// systems is related to the "Extend support for advanced text services
-    /// to all programs" checkbox in the advanced language settings dialog,
-    /// i.e. the problem appears when this checkbox is checked and disappears
-    /// if you clear it. For this reason, we disable the code below and
-    /// instead initialize COM in MTA as early as possible, before 3rd party
-    /// libraries we use have done so (i.e. Qt).
-# if 0
-    /* If we fail to set the necessary apartment model, it may mean that some
-     * DLL that was indirectly loaded by the process calling this function has
-     * already initialized COM on the given thread in an incompatible way
-     * which we can't leave with. Therefore, we try to fix this by using the
-     * brute force method: */
-
-    if (rc == RPC_E_CHANGED_MODE)
-    {
-        /* Before we use brute force, we need to check if we are in the
-         * neutral threaded apartment -- in this case there is no need to
-         * worry at all. */
-
-        rc = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
-        if (rc == RPC_E_CHANGED_MODE)
-        {
-            /* This is a neutral apartment, reset the error */
-            rc = S_OK;
-
-            LogFlowFunc(("COM is already initialized in neutral threaded "
-                         "apartment mode,\nwill accept it.\n"));
-        }
-        else if (rc == S_FALSE)
-        {
-            /* balance the test CoInitializeEx above */
-            CoUninitialize();
-            rc = RPC_E_CHANGED_MODE;
-
-            LogFlowFunc(("COM is already initialized in single threaded "
-                         "apartment mode,\nwill reinitialize as "
-                         "multi threaded.\n"));
-
-            enum { MaxTries = 10000 };
-            int tries = MaxTries;
-            while (rc == RPC_E_CHANGED_MODE && tries --)
-            {
-                CoUninitialize();
-                rc = CoInitializeEx(NULL, flags);
-                if (rc == S_OK)
-                {
-                    /* We've successfully reinitialized COM; restore the
-                     * initialization reference counter */
-
-                    LogFlowFunc(("Will call CoInitializeEx() %d times.\n",
-                                 MaxTries - tries));
-
-                    while (tries ++ < MaxTries)
-                    {
-                        rc = CoInitializeEx(NULL, flags);
-                        Assert(rc == S_FALSE);
-                    }
-                }
-            }
-        }
-        else
-            AssertMsgFailed(("rc=%08X\n", rc));
-    }
-# endif
-
     /* the overall result must be either S_OK or S_FALSE (S_FALSE means
      * "already initialized using the same apartment model") */
     AssertMsg(rc == S_OK || rc == S_FALSE, ("rc=%08X\n", rc));
@@ -341,6 +284,10 @@ HRESULT Initialize()
         ASMAtomicCmpXchgHandle(&gCOMMainThread, hSelf, NIL_RTTHREAD, fRc);
     else
         fRc = false;
+
+    if (fGui)
+        Assert(RTThreadIsMain(hSelf));
+
     if (!fRc)
     {
         if (   gCOMMainThread == hSelf
@@ -359,6 +306,9 @@ HRESULT Initialize()
 
 #else /* !defined (VBOX_WITH_XPCOM) */
 
+    /* Unused here */
+    NOREF(fGui);
+
     if (ASMAtomicXchgBool(&gIsXPCOMInitialized, true) == true)
     {
         /* XPCOM is already initialized on the main thread, no special
