@@ -1,4 +1,4 @@
-/* $Id: HostNetworkInterfaceImpl.cpp $ */
+/* $Id: HostNetworkInterfaceImpl.cpp 38406 2011-08-10 15:40:47Z vboxsync $ */
 
 /** @file
  *
@@ -42,12 +42,13 @@ HostNetworkInterface::~HostNetworkInterface()
 
 HRESULT HostNetworkInterface::FinalConstruct()
 {
-    return S_OK;
+    return BaseFinalConstruct();
 }
 
 void HostNetworkInterface::FinalRelease()
 {
     uninit ();
+    BaseFinalRelease();
 }
 
 // public initializer/uninitializer for internal purposes only
@@ -550,7 +551,6 @@ STDMETHODIMP HostNetworkInterface::DhcpRediscover ()
 
 HRESULT HostNetworkInterface::setVirtualBox(VirtualBox *pVBox)
 {
-    HRESULT hrc;
     AutoCaller autoCaller(this);
     if (FAILED(autoCaller.rc())) return autoCaller.rc();
     unconst(mVBox) = pVBox;
@@ -560,32 +560,14 @@ HRESULT HostNetworkInterface::setVirtualBox(VirtualBox *pVBox)
     if (m.IPAddress == 0 && mIfType == HostNetworkInterfaceType_HostOnly)
     {
         Bstr tmpAddr, tmpMask;
-        hrc = mVBox->GetExtraData(BstrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw()).raw(), tmpAddr.asOutParam());
+        HRESULT hrc = mVBox->GetExtraData(BstrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw()).raw(), tmpAddr.asOutParam());
         hrc = mVBox->GetExtraData(BstrFmt("HostOnly/%ls/IPNetMask", mInterfaceName.raw()).raw(), tmpMask.asOutParam());
         if (tmpAddr.isEmpty())
-        {
             tmpAddr = getDefaultIPv4Address(mInterfaceName);
-            /*
-             * We need to write the default IP address and mask to extra data now,
-             * so the interface gets re-created after vboxnetadp.ko reload.
-             * Note that we avoid calling EnableStaticIpConfig since it would
-             * change the address on host's interface as well and we want to
-             * postpone the change until VM actually starts.
-             */
-            hrc = mVBox->SetExtraData(BstrFmt("HostOnly/%ls/IPAddress", 
-                                              mInterfaceName.raw()).raw(),
-                                      tmpAddr.raw());
-            ComAssertComRCRet(hrc, hrc);
-        }
 
         if (tmpMask.isEmpty())
-        {
             tmpMask = Bstr(VBOXNET_IPV4MASK_DEFAULT);
-            hrc = mVBox->SetExtraData(BstrFmt("HostOnly/%ls/IPNetMask", 
-                                              mInterfaceName.raw()).raw(),
-                                      Bstr(VBOXNET_IPV4MASK_DEFAULT).raw());
-            ComAssertComRCRet(hrc, hrc);
-        }
+
         m.IPAddress = inet_addr(Utf8Str(tmpAddr).c_str());
         m.networkMask = inet_addr(Utf8Str(tmpMask).c_str());
     }
@@ -593,7 +575,7 @@ HRESULT HostNetworkInterface::setVirtualBox(VirtualBox *pVBox)
     if (m.IPV6Address.isEmpty())
     {
         Bstr tmpPrefixLen;
-        hrc = mVBox->GetExtraData(BstrFmt("HostOnly/%ls/IPV6Address", mInterfaceName.raw()).raw(), m.IPV6Address.asOutParam());
+        HRESULT hrc = mVBox->GetExtraData(BstrFmt("HostOnly/%ls/IPV6Address", mInterfaceName.raw()).raw(), m.IPV6Address.asOutParam());
         if (!m.IPV6Address.isEmpty())
         {
             hrc = mVBox->GetExtraData(BstrFmt("HostOnly/%ls/IPV6PrefixLen", mInterfaceName.raw()).raw(), tmpPrefixLen.asOutParam());
