@@ -1,4 +1,4 @@
-/* $Id: SystemPropertiesImpl.cpp $ */
+/* $Id: SystemPropertiesImpl.cpp 37423 2011-06-12 18:37:56Z vboxsync $ */
 
 /** @file
  *
@@ -61,12 +61,13 @@ SystemProperties::~SystemProperties()
 
 HRESULT SystemProperties::FinalConstruct()
 {
-    return S_OK;
+    return BaseFinalConstruct();
 }
 
 void SystemProperties::FinalRelease()
 {
     uninit();
+    BaseFinalRelease();
 }
 
 // public methods only for internal purposes
@@ -273,19 +274,6 @@ STDMETHODIMP SystemProperties::COMGETTER(InfoVDSize)(LONG64 *infoVDSize)
     return S_OK;
 }
 
-STDMETHODIMP SystemProperties::COMGETTER(NetworkAdapterCount)(ULONG *count)
-{
-    CheckComArgOutPointerValid(count);
-
-    AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
-
-    /* no need to lock, this is const */
-    *count = SchemaDefs::NetworkAdapterCount;
-
-    return S_OK;
-}
-
 STDMETHODIMP SystemProperties::COMGETTER(SerialPortCount)(ULONG *count)
 {
     CheckComArgOutPointerValid(count);
@@ -325,6 +313,69 @@ STDMETHODIMP SystemProperties::COMGETTER(MaxBootPosition)(ULONG *aMaxBootPositio
     return S_OK;
 }
 
+
+STDMETHODIMP SystemProperties::GetMaxNetworkAdapters(ChipsetType_T aChipset, ULONG *count)
+{
+    CheckComArgOutPointerValid(count);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    ULONG uResult = 0;
+
+    /* no need for locking, no state */
+    switch (aChipset)
+    {
+        case ChipsetType_PIIX3:
+            uResult = SchemaDefs::NetworkAdapterCount; /* == 8 */
+            break;
+        case ChipsetType_ICH9:
+            uResult = 36;
+            break;
+        default:
+            AssertMsgFailed(("Invalid chipset type %d\n", aChipset));
+    }
+
+    *count = uResult;
+
+    return S_OK;
+}
+
+STDMETHODIMP SystemProperties::GetMaxNetworkAdaptersOfType(ChipsetType_T aChipset, NetworkAttachmentType_T aType, ULONG *count)
+{
+    CheckComArgOutPointerValid(count);
+
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+
+    ULONG uResult = 0;
+    HRESULT rc = GetMaxNetworkAdapters(aChipset, &uResult);
+    if (FAILED(rc))
+        return rc;
+
+    /* no need for locking, no state */
+    switch (aType)
+    {
+        case NetworkAttachmentType_NAT:
+        case NetworkAttachmentType_Internal:
+            /* chipset default is OK */
+            break;
+        case NetworkAttachmentType_Bridged:
+            /* Maybe use current host interface count here? */
+            break;
+        case NetworkAttachmentType_HostOnly:
+            uResult = 8;
+            break;
+        default:
+            AssertMsgFailed(("Unhandled attachment type %d\n", aType));
+    }
+
+    *count = uResult;
+
+    return S_OK;
+}
+
+
 STDMETHODIMP SystemProperties::GetMaxDevicesPerPortForStorageBus(StorageBus_T aBus,
                                                                  ULONG *aMaxDevicesPerPort)
 {
@@ -1084,4 +1135,3 @@ HRESULT SystemProperties::setDefaultVRDEExtPack(const Utf8Str &aExtPack)
 
     return S_OK;
 }
-
