@@ -1,10 +1,10 @@
-/* $Id: time.h $ */
+/* $Id: time.h 36549 2011-04-05 09:28:24Z vboxsync $ */
 /** @file
  * IPRT - Internal RTTime header
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -33,9 +33,9 @@ RT_C_DECLS_BEGIN
 
 #if defined(IN_RING3) || defined(IN_RC)
 
-extern uint64_t g_u64ProgramStartNanoTS;
-extern uint64_t g_u64ProgramStartMicroTS;
-extern uint64_t g_u64ProgramStartMilliTS;
+extern DECLHIDDEN(uint64_t) g_u64ProgramStartNanoTS;
+extern DECLHIDDEN(uint64_t) g_u64ProgramStartMicroTS;
+extern DECLHIDDEN(uint64_t) g_u64ProgramStartMilliTS;
 
 #endif
 
