@@ -19,7 +19,7 @@
  * CDDL HEADER END
  */
 /*
- * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
+ * Copyright 2010-2011 Sun Microsystems, Inc.  All rights reserved.
  * Use is subject to license terms.
  */
 
@@ -56,6 +56,9 @@
 #include <sys/modctl.h>
 #include <sys/machparam.h>
 #include <sys/utsname.h>
+#include <sys/ctf_api.h>
+
+#include <iprt/assert.h>
 
 #include "vbi.h"
 
@@ -75,7 +78,7 @@ static void (*p_contig_free)(void *, size_t) = contig_free;
  */
 /* Introduced in v9 */
 static int use_kflt = 0;
-page_t *vbi_page_get_fromlist(uint_t freelist, caddr_t virtAddr);
+static page_t *vbi_page_get_fromlist(uint_t freelist, caddr_t virtAddr, size_t pgsize);
 
 
 /*
@@ -141,40 +144,6 @@ static int vbi_verbose = 0;
 /* Introduced in v8 */
 static int vbi_is_initialized = 0;
 
-/* Introduced in v6 */
-static int vbi_is_nevada = 0;
-
-#ifdef _LP64
-/* 64-bit Solaris 10 offsets */
-/* CPU */
-static int off_s10_cpu_runrun   = 232;
-static int off_s10_cpu_kprunrun = 233;
-/* kthread_t */
-static int off_s10_t_preempt    = 42;
-
-/* 64-bit Solaris 11 (Nevada/OpenSolaris) offsets */
-/* CPU */
-static int off_s11_cpu_runrun   = 216;
-static int off_s11_cpu_kprunrun = 217;
-/* kthread_t */
-static int off_s11_t_preempt    = 42;
-#else
-/* 32-bit Solaris 10 offsets */
-/* CPU */
-static int off_s10_cpu_runrun   = 124;
-static int off_s10_cpu_kprunrun = 125;
-/* kthread_t */
-static int off_s10_t_preempt    = 26;
-
-/* 32-bit Solaris 11 (Nevada/OpenSolaris) offsets */
-/* CPU */
-static int off_s11_cpu_runrun   = 112;
-static int off_s11_cpu_kprunrun = 113;
-/* kthread_t */
-static int off_s11_t_preempt    = 26;
-#endif
-
-
 /* Which offsets will be used */
 static int off_cpu_runrun       = -1;
 static int off_cpu_kprunrun     = -1;
@@ -213,6 +182,35 @@ _init(void)
 }
 #endif
 
+static int
+vbi_get_ctf_member_offset(ctf_file_t *ctfp, const char *structname, const char *membername, int *offset)
+{
+	AssertReturn(ctfp, CTF_ERR);
+	AssertReturn(structname, CTF_ERR);
+	AssertReturn(membername, CTF_ERR);
+	AssertReturn(offset, CTF_ERR);
+
+	ctf_id_t typeident = ctf_lookup_by_name(ctfp, structname);
+	if (typeident != CTF_ERR)
+	{
+		ctf_membinfo_t memberinfo;
+		bzero(&memberinfo, sizeof(memberinfo));
+		if (ctf_member_info(ctfp, typeident, membername, &memberinfo) != CTF_ERR)
+		{
+			*offset = (memberinfo.ctm_offset >> 3);
+			cmn_err(CE_NOTE, "%s::%s at %d\n", structname, membername, *offset);
+			return (0);
+		}
+		else
+			cmn_err(CE_NOTE, "ctf_member_info failed for struct %s member %s\n", structname, membername);
+	}
+	else
+		cmn_err(CE_NOTE, "ctf_lookup_by_name failed for struct %s\n", structname);
+
+	return (CTF_ERR);
+}
+
+
 int
 vbi_init(void)
 {
@@ -247,7 +245,7 @@ vbi_init(void)
 		p_contig_free = (void (*)(void *, size_t))
 			kobj_getsymvalue("contig_free", 1);
 		if (p_contig_free == NULL) {
-			cmn_err(CE_NOTE, " contig_free() not found in kernel\n");
+			cmn_err(CE_NOTE, "contig_free() not found in kernel\n");
 			return (EINVAL);
 		}
 	}
@@ -266,44 +264,33 @@ vbi_init(void)
 		}
 	}
 
-
 	/*
-	 * Check if this is S10 or Nevada
+	 * CTF probing for fluid, private members.
 	 */
-	if (!strncmp(utsname.release, "5.11", sizeof("5.11") - 1)) {
-		/* Nevada detected... */
-		vbi_is_nevada = 1;
+	int err = 0;
+	modctl_t *genunix_modctl = mod_hold_by_name("genunix");
+	if (genunix_modctl)
+	{
+		ctf_file_t *ctfp = ctf_modopen(genunix_modctl->mod_mp, &err);
+		if (ctfp)
+		{
+			do {
+				err = vbi_get_ctf_member_offset(ctfp, "kthread_t", "t_preempt", &off_t_preempt); AssertBreak(!err);
+				err = vbi_get_ctf_member_offset(ctfp, "cpu_t", "cpu_runrun", &off_cpu_runrun); AssertBreak(!err);
+				err = vbi_get_ctf_member_offset(ctfp, "cpu_t", "cpu_kprunrun", &off_cpu_kprunrun); AssertBreak(!err);
+			} while (0);
+		}
 
-		off_cpu_runrun = off_s11_cpu_runrun;
-		off_cpu_kprunrun = off_s11_cpu_kprunrun;
-		off_t_preempt = off_s11_t_preempt;
-	} else {
-		/* Solaris 10 detected... */
-		vbi_is_nevada = 0;
-
-		off_cpu_runrun = off_s10_cpu_runrun;
-		off_cpu_kprunrun = off_s10_cpu_kprunrun;
-		off_t_preempt = off_s10_t_preempt;
+		mod_release_mod(genunix_modctl);
+	}
+	else
+	{
+		cmn_err(CE_NOTE, "failed to open module genunix.\n");
+		err = EINVAL;
 	}
 
-	/*
-	 * Sanity checking...
-	 */
-	/* CPU */
-	char crr = VBI_CPU_RUNRUN;
-	char krr = VBI_CPU_KPRUNRUN;
-	if (   (crr < 0 || crr > 1)
-		|| (krr < 0 || krr > 1)) {
-		cmn_err(CE_NOTE, ":CPU structure sanity check failed! OS version mismatch.\n");
-		return EINVAL;
-	}
-
-	/* Thread */
-	char t_preempt = VBI_T_PREEMPT;
-	if (t_preempt < 0 || t_preempt > 32) {
-		cmn_err(CE_NOTE, ":Thread structure sanity check failed! OS version mismatch.\n");
-		return EINVAL;
-	}
+	if (err)
+		return (EINVAL);
 
 	vbi_is_initialized = 1;
 
@@ -366,7 +353,7 @@ vbi_internal_alloc(uint64_t *phys, size_t size, uint64_t alignment, int contig)
 	ptr = contig_alloc(size, &attr, PAGESIZE, 1);
 
 	if (ptr == NULL) {
-		cmn_err(CE_NOTE, "vbi_internal_alloc() failure for %lu bytes", size);
+		cmn_err(CE_NOTE, "vbi_internal_alloc() failure for %lu bytes contig=%d", size, contig);
 		return (NULL);
 	}
 
@@ -561,7 +548,7 @@ vbi_set_priority(void *thread, int priority)
 }
 
 void *
-vbi_thread_create(void *func, void *arg, size_t len, int priority)
+vbi_thread_create(void (*func)(void *), void *arg, size_t len, int priority)
 {
 	kthread_t *t;
 
@@ -1012,9 +999,7 @@ vbi_user_map(caddr_t *va, uint_t prot, uint64_t *palist, size_t len)
 	as_rangelock(as);
 	map_addr(va, len, 0, 0, MAP_SHARED);
 	if (*va != NULL)
-	{
 		error = as_map(as, *va, len, segvbi_create, &args);
-	}
 	else
 		error = ENOMEM;
 	if (error)
@@ -1029,7 +1014,7 @@ vbi_user_map(caddr_t *va, uint_t prot, uint64_t *palist, size_t len)
  */
 
 struct vbi_cpu_watch {
-	void (*vbi_cpu_func)();
+	void (*vbi_cpu_func)(void *, int, int);
 	void *vbi_cpu_arg;
 };
 
@@ -1050,7 +1035,7 @@ vbi_watcher(cpu_setup_t state, int icpu, void *arg)
 }
 
 vbi_cpu_watch_t *
-vbi_watch_cpus(void (*func)(), void *arg, int current_too)
+vbi_watch_cpus(void (*func)(void *, int, int), void *arg, int current_too)
 {
 	int c;
 	vbi_cpu_watch_t *w;
@@ -1210,7 +1195,7 @@ vbi_gtimer_begin(
 	t->g_func = func;
 	t->g_cyclic = CYCLIC_NONE;
 
-	omni.cyo_online = (void (*)())vbi_gtimer_online;
+	omni.cyo_online = (void (*)(void *, cpu_t *, cyc_handler_t *, cyc_time_t *))vbi_gtimer_online;
 	omni.cyo_offline = NULL;
 	omni.cyo_arg = t;
 
@@ -1332,11 +1317,11 @@ vbi_pages_alloc(uint64_t *phys, size_t size)
 				for (int64_t i = 0; i < npages; i++, virtAddr += PAGESIZE)
 				{
 					/* get a page from the freelists */
-					page_t *ppage = vbi_page_get_fromlist(1 /* freelist */, virtAddr);
+					page_t *ppage = vbi_page_get_fromlist(1 /* freelist */, virtAddr, PAGESIZE);
 					if (!ppage)
 					{
 						/* try from the cachelists */
-						ppage = vbi_page_get_fromlist(2 /* cachelist */, virtAddr);
+						ppage = vbi_page_get_fromlist(2 /* cachelist */, virtAddr, PAGESIZE);
 						if (!ppage)
 						{
 							/* damn */
@@ -1433,25 +1418,21 @@ vbi_page_to_pa(page_t **pp_pages, pgcnt_t i)
 }
 
 
-/*
- * This is revision 9 of the interface.
- */
-page_t *vbi_page_get_fromlist(uint_t freelist, caddr_t virtAddr)
+static page_t *
+vbi_page_get_fromlist(uint_t freelist, caddr_t virtAddr, size_t pgsize)
 {
+	/* pgsize only applies when using the freelist */
 	seg_t kernseg;
 	kernseg.s_as = &kas;
 	page_t *ppage = NULL;
 	if (freelist == 1)
 	{
 		ppage = page_get_freelist(&vbipagevp, 0 /* offset */, &kernseg, virtAddr,
-							PAGESIZE, 0 /* flags */, NULL /* local group */);
-		if (!ppage)
+							pgsize, 0 /* flags */, NULL /* local group */);
+		if (!ppage && use_kflt)
 		{
-			if (use_kflt)
-			{
-				ppage = page_get_freelist(&vbipagevp, 0 /* offset */, &kernseg, virtAddr,
-							PAGESIZE, 0x0200 /* PG_KFLT */, NULL /* local group */);
-			}
+			ppage = page_get_freelist(&vbipagevp, 0 /* offset */, &kernseg, virtAddr,
+						pgsize, 0x0200 /* PG_KFLT */, NULL /* local group */);
 		}
 	}
 	else
@@ -1459,23 +1440,151 @@ page_t *vbi_page_get_fromlist(uint_t freelist, caddr_t virtAddr)
 		/* cachelist */
 		ppage = page_get_cachelist(&vbipagevp, 0 /* offset */, &kernseg, virtAddr,
 							0 /* flags */, NULL /* local group */);
-		if (!ppage)
+		if (!ppage && use_kflt)
 		{
-			if (use_kflt)
-			{
-				ppage = page_get_cachelist(&vbipagevp, 0 /* offset */, &kernseg, virtAddr,
-							0x0200 /* PG_KFLT */, NULL /* local group */);
-			}
+			ppage = page_get_cachelist(&vbipagevp, 0 /* offset */, &kernseg, virtAddr,
+						0x0200 /* PG_KFLT */, NULL /* local group */);
 		}
 	}
 	return ppage;
 }
 
 
+/*
+ * Large page code.
+ */
+
+page_t *
+vbi_large_page_alloc(uint64_t *pphys, size_t pgsize)
+{
+	pgcnt_t const npages = pgsize >> PAGESHIFT;
+	page_t *pproot, *pp, *pplist;
+	pgcnt_t ipage;
+	caddr_t vaddr;
+	seg_t kernseg;
+	int rc;
+
+	/*
+	 * Reserve available memory for a large page and create it.
+	 */
+	rc = page_resv(npages, KM_NOSLEEP);
+	if (!rc)
+		return NULL;
+
+	rc = page_create_wait(npages, 0 /* flags */);
+	if (!rc) {
+		page_unresv(npages);
+		return NULL;
+	}
+
+	/*
+	 * Get a page off the free list.  We set vaddr to 0 since we don't know
+	 * where the memory is going to be mapped.
+	 */
+	vaddr = NULL;
+	kernseg.s_as = &kas;
+	pproot = vbi_page_get_fromlist(1 /* freelist */, vaddr, pgsize);
+	if (!pproot)
+	{
+		page_create_putback(npages);
+		page_unresv(npages);
+		return NULL;
+	}
+	AssertMsg(!(page_pptonum(pproot) & (npages - 1)), ("%p:%lx npages=%lx\n", pproot, page_pptonum(pproot), npages));
+
+	/*
+	 * Mark all the sub-pages as non-free and not-hashed-in.
+	 * It is paramount that we destroy the list (before freeing it).
+	 */
+	pplist = pproot;
+	for (ipage = 0; ipage < npages; ipage++) {
+		pp = pplist;
+		AssertPtr(pp);
+		AssertMsg(page_pptonum(pp) == ipage + page_pptonum(pproot),
+			("%p:%lx %lx+%lx\n", pp, page_pptonum(pp), ipage, page_pptonum(pproot)));
+		page_sub(&pplist, pp);
+		AssertMsg(PP_ISFREE(pp), ("%p\n", pp));
+		AssertMsg(pp->p_szc == pproot->p_szc, ("%p - %d expected %d \n", pp, pp->p_szc, pproot->p_szc));
+
+		PP_CLRFREE(pp);
+		PP_CLRAGED(pp);
+	}
+
+	*pphys = (uint64_t)page_pptonum(pproot) << PAGESHIFT;
+	AssertMsg(!(*pphys & (pgsize - 1)), ("%llx %zx\n", *pphys, pgsize));
+	return pproot;
+}
+
+void
+vbi_large_page_free(page_t *pproot, size_t pgsize)
+{
+	pgcnt_t const npages = pgsize >> PAGESHIFT;
+	pgcnt_t ipage;
+
+	Assert(page_get_pagecnt(pproot->p_szc) == npages);
+	AssertMsg(!(page_pptonum(pproot) & (npages - 1)), ("%p:%lx npages=%lx\n", pproot, page_pptonum(pproot), npages));
+
+	/*
+	 * We need to exclusively lock the sub-pages before freeing
+	 * the large one.
+	 */
+	for (ipage = 0; ipage < npages; ipage++) {
+		page_t *pp = page_nextn(pproot, ipage);
+		AssertMsg(page_pptonum(pp) == ipage + page_pptonum(pproot),
+			("%p:%lx %lx+%lx\n", pp, page_pptonum(pp), ipage, page_pptonum(pproot)));
+		AssertMsg(!PP_ISFREE(pp), ("%p\n", pp));
+
+		int rc = page_tryupgrade(pp);
+		if (!rc) {
+			page_unlock(pp);
+			while (!page_lock(pp, SE_EXCL, NULL /* mutex */, P_RECLAIM)) {
+				/*nothing*/;
+			}
+		}
+	}
+
+	/*
+	 * Free the large page and unreserve the memory.
+	 */
+	page_free_pages(pproot);
+	page_unresv(npages);
+}
+
+int
+vbi_large_page_premap(page_t *pproot, size_t pgsize)
+{
+	pgcnt_t const npages = pgsize >> PAGESHIFT;
+	pgcnt_t ipage;
+
+	Assert(page_get_pagecnt(pproot->p_szc) == npages);
+	AssertMsg(!(page_pptonum(pproot) & (npages - 1)), ("%p:%lx npages=%lx\n", pproot, page_pptonum(pproot), npages));
+
+	/*
+	 * We need to downgrade the sub-pages from exclusive to shared locking
+	 * because otherwise we cannot <you go figure>.
+	 */
+	for (ipage = 0; ipage < npages; ipage++) {
+	    page_t *pp = page_nextn(pproot, ipage);
+	    AssertMsg(page_pptonum(pp) == ipage + page_pptonum(pproot),
+		    ("%p:%lx %lx+%lx\n", pp, page_pptonum(pp), ipage, page_pptonum(pproot)));
+	    AssertMsg(!PP_ISFREE(pp), ("%p\n", pp));
+
+	    if (page_tryupgrade(pp) == 1)
+		    page_downgrade(pp);
+	    AssertMsg(!PP_ISFREE(pp), ("%p\n", pp));
+	}
+
+	return 0;
+}
+
+
 /*
  * As more functions are added, they should start with a comment indicating
  * the revision and above this point in the file and the revision level should
  * be increased. Also change vbi_modlmisc at the top of the file.
+ *
+ * NOTE! We'll start care about this if anything in here ever makes it into
+ *       the solaris kernel proper.
  */
 uint_t vbi_revision_level = 9;
 
