@@ -1,4 +1,4 @@
-/* $Id: memobj-r0drv-solaris.c $ */
+/* $Id: memobj-r0drv-solaris.c 37281 2011-05-31 21:32:44Z vboxsync $ */
 /** @file
  * IPRT - Ring-0 Memory Objects, Solaris.
  */
@@ -32,6 +32,7 @@
 #include "internal/iprt.h"
 #include <iprt/memobj.h>
 
+#include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/err.h>
 #include <iprt/log.h>
@@ -56,11 +57,14 @@ typedef struct RTR0MEMOBJSOLARIS
     void               *pvHandle;
     /** Access during locking. */
     int                 fAccess;
+    /** Set if large pages are involved in an RTR0MEMOBJTYPE_PHYS
+     *  allocation. */
+    bool                fLargePage;
 } RTR0MEMOBJSOLARIS, *PRTR0MEMOBJSOLARIS;
 
 
 
-int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
+DECLHIDDEN(int) rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
 {
     PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)pMem;
 
@@ -70,20 +74,17 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
             vbi_lowmem_free(pMemSolaris->Core.pv, pMemSolaris->Core.cb);
             break;
 
-        case RTR0MEMOBJTYPE_CONT:
         case RTR0MEMOBJTYPE_PHYS:
-            vbi_phys_free(pMemSolaris->Core.pv, pMemSolaris->Core.cb);
+            if (!pMemSolaris->Core.u.Phys.fAllocated)
+            {   /* nothing to do here */;   }
+            else if (pMemSolaris->fLargePage)
+                vbi_large_page_free(pMemSolaris->pvHandle, pMemSolaris->Core.cb);
+            else
+                vbi_phys_free(pMemSolaris->Core.pv, pMemSolaris->Core.cb);
             break;
 
         case RTR0MEMOBJTYPE_PHYS_NC:
-#if 0
-            vbi_phys_free(pMemSolaris->Core.pv, pMemSolaris->Core.cb);
-#else
-            if (pMemSolaris->Core.u.Phys.fAllocated == true)
-                ddi_umem_free(pMemSolaris->Cookie);
-            else
-                vbi_pages_free(pMemSolaris->pvHandle, pMemSolaris->Core.cb);
-#endif
+            vbi_pages_free(pMemSolaris->pvHandle, pMemSolaris->Core.cb);
             break;
 
         case RTR0MEMOBJTYPE_PAGE:
@@ -107,6 +108,7 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
             break;
         }
 
+        case RTR0MEMOBJTYPE_CONT: /* we don't use this type here. */
         default:
             AssertMsgFailed(("enmType=%d\n", pMemSolaris->Core.enmType));
             return VERR_INTERNAL_ERROR;
@@ -116,7 +118,7 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
 }
 
 
-int rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
+DECLHIDDEN(int) rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
 {
     /* Create the object. */
     PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_PAGE, NULL, cb);
@@ -130,14 +132,14 @@ int rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecu
         return VERR_NO_PAGE_MEMORY;
     }
 
-    pMemSolaris->Core.pv = virtAddr;
+    pMemSolaris->Core.pv  = virtAddr;
     pMemSolaris->pvHandle = NULL;
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
 }
 
 
-int rtR0MemObjNativeAllocLow(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
+DECLHIDDEN(int) rtR0MemObjNativeAllocLow(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
 {
     NOREF(fExecutable);
 
@@ -161,65 +163,42 @@ int rtR0MemObjNativeAllocLow(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecut
 }
 
 
-int rtR0MemObjNativeAllocCont(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
+DECLHIDDEN(int) rtR0MemObjNativeAllocCont(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
 {
     NOREF(fExecutable);
     return rtR0MemObjNativeAllocPhys(ppMem, cb, _4G - 1, PAGE_SIZE /* alignment */);
 }
 
 
-int rtR0MemObjNativeAllocPhysNC(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest)
+DECLHIDDEN(int) rtR0MemObjNativeAllocPhysNC(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest)
 {
 #if HC_ARCH_BITS == 64
     PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_PHYS_NC, NULL, cb);
     if (!pMemSolaris)
         return VERR_NO_MEMORY;
 
-    /* Allocate physically non-contiguous page-aligned memory. */
-    uint64_t physAddr = PhysHighest;
-
-# if 0
-    /*
-     * The contig_alloc() way of allocating NC pages is broken or does not match our semantics. Refer #4716 for details.
-     */
-#  if 0
-    /* caddr_t virtAddr  = vbi_phys_alloc(&physAddr, cb, PAGE_SIZE, 0 /* non-contiguous */);
-#  endif
-    caddr_t virtAddr = ddi_umem_alloc(cb, DDI_UMEM_SLEEP, &pMemSolaris->Cookie);
-    if (RT_UNLIKELY(virtAddr == NULL))
-    {
-        rtR0MemObjDelete(&pMemSolaris->Core);
-        return VERR_NO_MEMORY;
-    }
-    pMemSolaris->Core.pv = virtAddr;
-    pMemSolaris->Core.u.Phys.PhysBase = physAddr;
-    pMemSolaris->Core.u.Phys.fAllocated = true;
-    pMemSolaris->pvHandle = NULL;
-# else
-    void *pvPages = vbi_pages_alloc(&physAddr, cb);
+    uint64_t PhysAddr = PhysHighest;
+    void *pvPages = vbi_pages_alloc(&PhysAddr, cb);
     if (!pvPages)
     {
         LogRel(("rtR0MemObjNativeAllocPhysNC: vbi_pages_alloc failed.\n"));
         rtR0MemObjDelete(&pMemSolaris->Core);
         return VERR_NO_MEMORY;
     }
-    pMemSolaris->Core.pv = NULL;
-    pMemSolaris->Core.u.Phys.PhysBase = physAddr;
-    pMemSolaris->Core.u.Phys.fAllocated = false;
-    pMemSolaris->pvHandle = pvPages;
-# endif
+    pMemSolaris->Core.pv   = NULL;
+    pMemSolaris->pvHandle  = pvPages;
 
-    Assert(!(physAddr & PAGE_OFFSET_MASK));
+    Assert(!(PhysAddr & PAGE_OFFSET_MASK));
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
-#else
-    /** @todo rtR0MemObjNativeAllocPhysNC / solaris */
+
+#else /* 32 bit: */
     return VERR_NOT_SUPPORTED; /* see the RTR0MemObjAllocPhysNC specs */
 #endif
 }
 
 
-int rtR0MemObjNativeAllocPhys(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest, size_t uAlignment)
+DECLHIDDEN(int) rtR0MemObjNativeAllocPhys(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS PhysHighest, size_t uAlignment)
 {
     AssertMsgReturn(PhysHighest >= 16 *_1M, ("PhysHigest=%RHp\n", PhysHighest), VERR_NOT_SUPPORTED);
 
@@ -227,48 +206,71 @@ int rtR0MemObjNativeAllocPhys(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, RTHCPHYS Ph
     if (!pMemSolaris)
         return VERR_NO_MEMORY;
 
-    AssertCompile(NIL_RTHCPHYS == UINT64_MAX);
-
-    /* Allocate physically contiguous memory aligned as specified. */
-    uint64_t physAddr = PhysHighest;
-    caddr_t  virtAddr = vbi_phys_alloc(&physAddr, cb, uAlignment, 1 /* contiguous */);
-    if (RT_UNLIKELY(virtAddr == NULL))
+    /*
+     * Allocating one large page gets special treatment.
+     */
+    static uint32_t s_cbLargePage = UINT32_MAX;
+    if (s_cbLargePage == UINT32_MAX)
     {
-        rtR0MemObjDelete(&pMemSolaris->Core);
-        return VERR_NO_CONT_MEMORY;
+#if 0 /* currently not entirely stable, so disabled. */
+        if (page_num_pagesizes() > 1)
+            ASMAtomicWriteU32(&s_cbLargePage, page_get_pagesize(1));
+        else
+#endif
+            ASMAtomicWriteU32(&s_cbLargePage, 0);
     }
-    Assert(!(physAddr & PAGE_OFFSET_MASK));
-    Assert(physAddr < PhysHighest);
-    Assert(physAddr + cb <= PhysHighest);
-#if 0
-    if (uAlignment != PAGE_SIZE)
+    uint64_t PhysAddr;
+    if (   cb == s_cbLargePage
+        && cb == uAlignment
+        && PhysHighest == NIL_RTHCPHYS)
     {
-        /* uAlignment is always a multiple of PAGE_SIZE */
-        pgcnt_t cPages = (cb + uAlignment - 1) >> PAGE_SHIFT;
-        void *pvPage = virtAddr;
-        while (cPages-- > 0)
+        /*
+         * Allocate one large page.
+         */
+        void *pvPages = vbi_large_page_alloc(&PhysAddr, cb);
+        if (pvPages)
         {
-            uint64_t u64Page = vbi_va_to_pa(pvPage);
-            if (u64Page & (uAlignment - 1))
-            {
-                LogRel(("rtR0MemObjNativeAllocPhys: alignment mismatch! cb=%u uAlignment=%u physAddr=%#x\n", cb, uAlignment, u64Page));
-                vbi_phys_free(virtAddr, cb);
-                rtR0MemObjDelete(&pMemSolaris->Core);
-                return VERR_NO_MEMORY;
-            }
-            pvPage = (void *)((uintptr_t)pvPage + PAGE_SIZE);
+            AssertMsg(!(PhysAddr & (cb - 1)), ("%RHp\n", PhysAddr));
+            pMemSolaris->Core.pv                = NULL;
+            pMemSolaris->Core.u.Phys.PhysBase   = PhysAddr;
+            pMemSolaris->Core.u.Phys.fAllocated = true;
+            pMemSolaris->pvHandle               = pvPages;
+            pMemSolaris->fLargePage             = true;
+
+            *ppMem = &pMemSolaris->Core;
+            return VINF_SUCCESS;
         }
     }
-#endif
-    pMemSolaris->Core.pv = virtAddr;
-    pMemSolaris->Core.u.Cont.Phys = physAddr;
-    pMemSolaris->pvHandle = NULL;
-    *ppMem = &pMemSolaris->Core;
-    return VINF_SUCCESS;
+    else
+    {
+        /*
+         * Allocate physically contiguous memory aligned as specified.
+         */
+        AssertCompile(NIL_RTHCPHYS == UINT64_MAX);
+        PhysAddr = PhysHighest;
+        caddr_t pvMem = vbi_phys_alloc(&PhysAddr, cb, uAlignment, 1 /* contiguous */);
+        if (RT_LIKELY(pvMem))
+        {
+            Assert(!(PhysAddr & PAGE_OFFSET_MASK));
+            Assert(PhysAddr < PhysHighest);
+            Assert(PhysAddr + cb <= PhysHighest);
+
+            pMemSolaris->Core.pv                = pvMem;
+            pMemSolaris->Core.u.Phys.PhysBase   = PhysAddr;
+            pMemSolaris->Core.u.Phys.fAllocated = true;
+            pMemSolaris->pvHandle               = NULL;
+            pMemSolaris->fLargePage             = false;
+
+            *ppMem = &pMemSolaris->Core;
+            return VINF_SUCCESS;
+        }
+    }
+    rtR0MemObjDelete(&pMemSolaris->Core);
+    return VERR_NO_CONT_MEMORY;
 }
 
 
-int rtR0MemObjNativeEnterPhys(PPRTR0MEMOBJINTERNAL ppMem, RTHCPHYS Phys, size_t cb, uint32_t uCachePolicy)
+DECLHIDDEN(int) rtR0MemObjNativeEnterPhys(PPRTR0MEMOBJINTERNAL ppMem, RTHCPHYS Phys, size_t cb, uint32_t uCachePolicy)
 {
     AssertReturn(uCachePolicy == RTMEM_CACHE_POLICY_DONT_CARE, VERR_NOT_SUPPORTED);
 
@@ -278,15 +280,16 @@ int rtR0MemObjNativeEnterPhys(PPRTR0MEMOBJINTERNAL ppMem, RTHCPHYS Phys, size_t
         return VERR_NO_MEMORY;
 
     /* There is no allocation here, it needs to be mapped somewhere first. */
-    pMemSolaris->Core.u.Phys.fAllocated = false;
-    pMemSolaris->Core.u.Phys.PhysBase = Phys;
+    pMemSolaris->Core.u.Phys.fAllocated   = false;
+    pMemSolaris->Core.u.Phys.PhysBase     = Phys;
     pMemSolaris->Core.u.Phys.uCachePolicy = uCachePolicy;
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
 }
 
 
-int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, uint32_t fAccess, RTR0PROCESS R0Process)
+DECLHIDDEN(int) rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, uint32_t fAccess,
+                                         RTR0PROCESS R0Process)
 {
     AssertReturn(R0Process == RTR0ProcHandleSelf(), VERR_INVALID_PARAMETER);
     NOREF(fAccess);
@@ -296,14 +299,13 @@ int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t c
     if (!pMemSolaris)
         return VERR_NO_MEMORY;
 
+    /* Lock down user pages. */
     int fPageAccess = S_READ;
     if (fAccess & RTMEM_PROT_WRITE)
         fPageAccess = S_WRITE;
     if (fAccess & RTMEM_PROT_EXEC)
         fPageAccess = S_EXEC;
     void *pvPageList = NULL;
-
-    /* Lock down user pages */
     int rc = vbi_lock_va((caddr_t)R3Ptr, cb, fPageAccess, &pvPageList);
     if (rc != 0)
     {
@@ -312,15 +314,16 @@ int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t c
         return VERR_LOCK_FAILED;
     }
 
-    pMemSolaris->Core.u.Lock.R0Process = (RTR0PROCESS)vbi_proc();
-    pMemSolaris->pvHandle = pvPageList;
-    pMemSolaris->fAccess = fPageAccess;
+    /* Fill in the object attributes and return successfully. */
+    pMemSolaris->Core.u.Lock.R0Process  = R0Process;
+    pMemSolaris->pvHandle               = pvPageList;
+    pMemSolaris->fAccess                = fPageAccess;
     *ppMem = &pMemSolaris->Core;
     return VINF_SUCCESS;
 }
 
 
-int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb, uint32_t fAccess)
+DECLHIDDEN(int) rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb, uint32_t fAccess)
 {
     NOREF(fAccess);
 
@@ -328,6 +331,7 @@ int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb,
     if (!pMemSolaris)
         return VERR_NO_MEMORY;
 
+    /* Lock down kernel pages. */
     int fPageAccess = S_READ;
     if (fAccess & RTMEM_PROT_WRITE)
         fPageAccess = S_WRITE;
@@ -342,6 +346,7 @@ int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb,
         return VERR_LOCK_FAILED;
     }
 
+    /* Fill in the object attributes and return successfully. */
     pMemSolaris->Core.u.Lock.R0Process = NIL_RTR0PROCESS;
     pMemSolaris->pvHandle = pvPageList;
     pMemSolaris->fAccess = fPageAccess;
@@ -350,7 +355,7 @@ int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb,
 }
 
 
-int rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixed, size_t cb, size_t uAlignment)
+DECLHIDDEN(int) rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixed, size_t cb, size_t uAlignment)
 {
     PRTR0MEMOBJSOLARIS  pMemSolaris;
 
@@ -358,7 +363,7 @@ int rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixed, siz
      * Use xalloc.
      */
     void *pv = vmem_xalloc(heap_arena, cb, uAlignment, 0 /*phase*/, 0 /*nocross*/,
-                     NULL /*minaddr*/, NULL /*maxaddr*/, VM_SLEEP);
+                           NULL /*minaddr*/, NULL /*maxaddr*/, VM_SLEEP);
     if (RT_UNLIKELY(!pv))
         return VERR_NO_MEMORY;
 
@@ -377,94 +382,111 @@ int rtR0MemObjNativeReserveKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixed, siz
 }
 
 
-int rtR0MemObjNativeReserveUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3PtrFixed, size_t cb, size_t uAlignment, RTR0PROCESS R0Process)
+DECLHIDDEN(int) rtR0MemObjNativeReserveUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3PtrFixed, size_t cb, size_t uAlignment, RTR0PROCESS R0Process)
 {
     return VERR_NOT_SUPPORTED;
 }
 
-int rtR0MemObjNativeMapKernel(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ pMemToMap, void *pvFixed, size_t uAlignment,
-                              unsigned fProt, size_t offSub, size_t cbSub)
+
+DECLHIDDEN(int) rtR0MemObjNativeMapKernel(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ pMemToMap, void *pvFixed, size_t uAlignment,
+                                          unsigned fProt, size_t offSub, size_t cbSub)
 {
     /** @todo rtR0MemObjNativeMapKernel / Solaris - Should be fairly simple alloc kernel memory and memload it. */
     return VERR_NOT_SUPPORTED;
 }
 
 
-int rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, PRTR0MEMOBJINTERNAL pMemToMap, RTR3PTR R3PtrFixed, size_t uAlignment, unsigned fProt, RTR0PROCESS R0Process)
+DECLHIDDEN(int) rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, PRTR0MEMOBJINTERNAL pMemToMap, RTR3PTR R3PtrFixed,
+                                        size_t uAlignment, unsigned fProt, RTR0PROCESS R0Process)
 {
+    /*
+     * Fend off things we cannot do.
+     */
     AssertMsgReturn(R3PtrFixed == (RTR3PTR)-1, ("%p\n", R3PtrFixed), VERR_NOT_SUPPORTED);
     AssertMsgReturn(R0Process == RTR0ProcHandleSelf(), ("%p != %p\n", R0Process, RTR0ProcHandleSelf()), VERR_NOT_SUPPORTED);
     if (uAlignment != PAGE_SIZE)
         return VERR_NOT_SUPPORTED;
 
-    PRTR0MEMOBJSOLARIS pMemToMapSolaris = (PRTR0MEMOBJSOLARIS)pMemToMap;
-    size_t cb        = pMemToMapSolaris->Core.cb;
-    void *pv         = pMemToMapSolaris->Core.pv;
-    pgcnt_t cPages   = (cb + PAGE_SIZE - 1) >> PAGE_SHIFT;
+    /*
+     * Get parameters from the source object.
+     */
+    PRTR0MEMOBJSOLARIS  pMemToMapSolaris = (PRTR0MEMOBJSOLARIS)pMemToMap;
+    void               *pv               = pMemToMapSolaris->Core.pv;
+    size_t              cb               = pMemToMapSolaris->Core.cb;
+    pgcnt_t             cPages           = cb >> PAGE_SHIFT;
 
-    /* Create the mapping object */
-    PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_MAPPING, pv, cb);
+    /*
+     * Create the mapping object
+     */
+    PRTR0MEMOBJSOLARIS pMemSolaris;
+    pMemSolaris = (PRTR0MEMOBJSOLARIS)rtR0MemObjNew(sizeof(*pMemSolaris), RTR0MEMOBJTYPE_MAPPING, pv, cb);
     if (RT_UNLIKELY(!pMemSolaris))
         return VERR_NO_MEMORY;
 
+    int rc = VINF_SUCCESS;
     uint64_t *paPhysAddrs = kmem_zalloc(sizeof(uint64_t) * cPages, KM_SLEEP);
-    if (RT_UNLIKELY(!paPhysAddrs))
-        return VERR_NO_MEMORY;
-
-    if (   pMemToMapSolaris->Core.enmType == RTR0MEMOBJTYPE_PHYS_NC
-        && pMemSolaris->Core.u.Phys.fAllocated == false)
+    if (RT_LIKELY(paPhysAddrs))
     {
         /*
-         * The PhysNC object has no kernel mapping backing it. The call to vbi_pages_premap()
-         * prepares the physical pages to be mapped into user or kernel space.
+         * Prepare the pages according to type.
          */
-        int rc = vbi_pages_premap(pMemToMapSolaris->pvHandle, cb, paPhysAddrs);
-        if (rc)
+        if (pMemToMapSolaris->Core.enmType == RTR0MEMOBJTYPE_PHYS_NC)
+            rc = vbi_pages_premap(pMemToMapSolaris->pvHandle, cb, paPhysAddrs);
+        else if (   pMemToMapSolaris->Core.enmType == RTR0MEMOBJTYPE_PHYS
+                 && pMemToMapSolaris->fLargePage)
         {
-            LogRel(("rtR0MemObjNativeMapUser: vbi_pages_premap failed. rc=%d\n", rc));
-            kmem_free(paPhysAddrs, sizeof(uint64_t) * cPages);
-            rtR0MemObjDelete(&pMemSolaris->Core);
-            return VERR_MAP_FAILED;
+            RTHCPHYS Phys = pMemToMapSolaris->Core.u.Phys.PhysBase;
+            for (pgcnt_t iPage = 0; iPage < cPages; iPage++, Phys += PAGE_SIZE)
+                paPhysAddrs[iPage] = Phys;
+            rc = vbi_large_page_premap(pMemToMapSolaris->pvHandle, cb);
         }
+        else
+        {
+            /* Have kernel mapping, just translate virtual to physical. */
+            AssertPtr(pv);
+            rc = 0;
+            for (pgcnt_t iPage = 0; iPage < cPages; iPage++)
+            {
+                paPhysAddrs[iPage] = vbi_va_to_pa(pv);
+                if (RT_UNLIKELY(paPhysAddrs[iPage] == -(uint64_t)1))
+                {
+                    LogRel(("rtR0MemObjNativeMapUser: no page to map.\n"));
+                    rc = -1;
+                    break;
+                }
+                pv = (void *)((uintptr_t)pv + PAGE_SIZE);
+            }
+        }
+        if (!rc)
+        {
+            /*
+             * Perform the actual mapping.
+             */
+            caddr_t UserAddr = NULL;
+            rc = vbi_user_map(&UserAddr, fProt, paPhysAddrs, cb);
+            if (!rc)
+            {
+                pMemSolaris->Core.u.Mapping.R0Process = R0Process;
+                pMemSolaris->Core.pv                  = UserAddr;
+
+                *ppMem = &pMemSolaris->Core;
+                kmem_free(paPhysAddrs, sizeof(uint64_t) * cPages);
+                return VINF_SUCCESS;
+            }
+
+            LogRel(("rtR0MemObjNativeMapUser: vbi_user_map failed.\n"));
+        }
+        rc = VERR_MAP_FAILED;
+        kmem_free(paPhysAddrs, sizeof(uint64_t) * cPages);
     }
     else
-    {
-        /*
-         * All other memory object types have allocated memory with kernel mappings.
-         */
-        for (pgcnt_t iPage = 0; iPage < cPages; iPage++)
-        {
-            paPhysAddrs[iPage] = vbi_va_to_pa(pv);
-            if (RT_UNLIKELY(paPhysAddrs[iPage] == -(uint64_t)1))
-            {
-                LogRel(("rtR0MemObjNativeMapUser: no page to map.\n"));
-                kmem_free(paPhysAddrs, sizeof(uint64_t) * cPages);
-                rtR0MemObjDelete(&pMemSolaris->Core);
-                return VERR_MAP_FAILED;
-            }
-            pv = (void *)((uintptr_t)pv + PAGE_SIZE);
-        }
-    }
-
-    caddr_t virtAddr = NULL;
-    int rc = vbi_user_map(&virtAddr, fProt, paPhysAddrs, cb);
-    if (rc != 0)
-    {
-        LogRel(("rtR0MemObjNativeMapUser: vbi mapping failure.\n"));
-        kmem_free(paPhysAddrs, sizeof(uint64_t) * cPages);
-        rtR0MemObjDelete(&pMemSolaris->Core);
-        return VERR_MAP_FAILED;
-    }
-
-    pMemSolaris->Core.u.Mapping.R0Process = (RTR0PROCESS)vbi_proc();
-    pMemSolaris->Core.pv = virtAddr;
-    *ppMem = &pMemSolaris->Core;
-    kmem_free(paPhysAddrs, sizeof(uint64_t) * cPages);
-    return VINF_SUCCESS;
+        rc = VERR_NO_MEMORY;
+    rtR0MemObjDelete(&pMemSolaris->Core);
+    return rc;
 }
 
 
-int rtR0MemObjNativeProtect(PRTR0MEMOBJINTERNAL pMem, size_t offSub, size_t cbSub, uint32_t fProt)
+DECLHIDDEN(int) rtR0MemObjNativeProtect(PRTR0MEMOBJINTERNAL pMem, size_t offSub, size_t cbSub, uint32_t fProt)
 {
     NOREF(pMem);
     NOREF(offSub);
@@ -474,12 +496,20 @@ int rtR0MemObjNativeProtect(PRTR0MEMOBJINTERNAL pMem, size_t offSub, size_t cbSu
 }
 
 
-RTHCPHYS rtR0MemObjNativeGetPagePhysAddr(PRTR0MEMOBJINTERNAL pMem, size_t iPage)
+DECLHIDDEN(RTHCPHYS) rtR0MemObjNativeGetPagePhysAddr(PRTR0MEMOBJINTERNAL pMem, size_t iPage)
 {
     PRTR0MEMOBJSOLARIS pMemSolaris = (PRTR0MEMOBJSOLARIS)pMem;
 
     switch (pMemSolaris->Core.enmType)
     {
+        case RTR0MEMOBJTYPE_PHYS_NC:
+            if (pMemSolaris->Core.u.Phys.fAllocated)
+            {
+                uint8_t *pb = (uint8_t *)pMemSolaris->Core.pv + ((size_t)iPage << PAGE_SHIFT);
+                return vbi_va_to_pa(pb);
+            }
+            return vbi_page_to_pa(pMemSolaris->pvHandle, iPage);
+
         case RTR0MEMOBJTYPE_PAGE:
         case RTR0MEMOBJTYPE_LOW:
         case RTR0MEMOBJTYPE_LOCK:
@@ -497,16 +527,7 @@ RTHCPHYS rtR0MemObjNativeGetPagePhysAddr(PRTR0MEMOBJINTERNAL pMem, size_t iPage)
 
         case RTR0MEMOBJTYPE_CONT:
         case RTR0MEMOBJTYPE_PHYS:
-            return pMemSolaris->Core.u.Cont.Phys + (iPage << PAGE_SHIFT);
-
-        case RTR0MEMOBJTYPE_PHYS_NC:
-            if (pMemSolaris->Core.u.Phys.fAllocated == true)
-            {
-                uint8_t *pb = (uint8_t *)pMemSolaris->Core.pv + ((size_t)iPage << PAGE_SHIFT);
-                return vbi_va_to_pa(pb);
-            }
-            return vbi_page_to_pa(pMemSolaris->pvHandle, iPage);
-
+            AssertFailed(); /* handled by the caller */
         case RTR0MEMOBJTYPE_RES_VIRT:
         default:
             return NIL_RTHCPHYS;
