@@ -1,4 +1,4 @@
-/* $Id: sched-posix.cpp $ */
+/* $Id: sched-posix.cpp 36555 2011-04-05 12:34:09Z vboxsync $ */
 /** @file
  * IPRT - Scheduling, POSIX.
  */
@@ -508,7 +508,7 @@ static void *rtSchedNativeProberThread(void *pvUser)
  * @returns iprt status code.
  * @param   enmType     The thread type to be assumed for the current thread.
  */
-int rtSchedNativeCalcDefaultPriority(RTTHREADTYPE enmType)
+DECLHIDDEN(int) rtSchedNativeCalcDefaultPriority(RTTHREADTYPE enmType)
 {
     Assert(enmType > RTTHREADTYPE_INVALID && enmType < RTTHREADTYPE_END);
 
@@ -643,7 +643,7 @@ static void *rtSchedNativeValidatorThread(void *pvUser)
  * @returns iprt status code.
  * @param   enmPriority     The priority to validate and set.
  */
-int rtProcNativeSetPriority(RTPROCPRIORITY enmPriority)
+DECLHIDDEN(int) rtProcNativeSetPriority(RTPROCPRIORITY enmPriority)
 {
     Assert(enmPriority > RTPROCPRIORITY_INVALID && enmPriority < RTPROCPRIORITY_LAST);
 
@@ -739,7 +739,7 @@ int rtProcNativeSetPriority(RTPROCPRIORITY enmPriority)
  * @param   Thread      The thread in question.
  * @param   enmType     The thread type.
  */
-int rtThreadNativeSetPriority(PRTTHREADINT pThread, RTTHREADTYPE enmType)
+DECLHIDDEN(int) rtThreadNativeSetPriority(PRTTHREADINT pThread, RTTHREADTYPE enmType)
 {
     Assert(enmType > RTTHREADTYPE_INVALID && enmType < RTTHREADTYPE_END);
     Assert(enmType == g_pProcessPriority->paTypes[enmType].enmType);
