@@ -1,4 +1,4 @@
-/* $Id: tstRTStrFormat.cpp $ */
+/* $Id: tstRTStrFormat.cpp 37966 2011-07-14 13:39:15Z vboxsync $ */
 /** @file
  * IPRT Testcase - String formatting.
  */
@@ -588,6 +588,17 @@ int main()
                 &s_abHex1[0], &s_abHex1[0x10]);
     CHECKSTR(pszBuf2);
 
+    /*
+     * x86 register formatting.
+     */
+    RTTestSub(hTest, "x86 register format types (%RAx86[*])");
+    CHECK42("%RAx86[cr0]", UINT64_C(0x80000011),    "80000011{PE,ET,PG}");
+    CHECK42("%RAx86[cr0]", UINT64_C(0x80000001),    "80000001{PE,PG}");
+    CHECK42("%RAx86[cr0]", UINT64_C(0x00000001),    "00000001{PE}");
+    CHECK42("%RAx86[cr0]", UINT64_C(0x80000000),    "80000000{PG}");
+    CHECK42("%RAx86[cr4]", UINT64_C(0x80000001),    "80000001{VME,unkn=80000000}");
+    CHECK42("%#RAx86[cr4]", UINT64_C(0x80000001),    "0x80000001{VME,unkn=0x80000000}");
+
     /*
      * Custom types.
      */
