@@ -1,10 +1,10 @@
-/* $Id: IOMAll.cpp $ */
+/* $Id: IOMAll.cpp 37467 2011-06-15 13:08:45Z vboxsync $ */
 /** @file
  * IOM - Input / Output Monitor - Any Context.
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,6 +21,9 @@
 #define LOG_GROUP LOG_GROUP_IOM
 #include <VBox/vmm/iom.h>
 #include <VBox/vmm/mm.h>
+#if defined(IEM_VERIFICATION_MODE) && defined(IN_RING3)
+# include <VBox/vmm/iem.h>
+#endif
 #include <VBox/param.h>
 #include "IOMInternal.h"
 #include <VBox/vmm/vm.h>
@@ -33,49 +36,7 @@
 #include <VBox/err.h>
 #include <VBox/log.h>
 #include <iprt/assert.h>
-
-
-
-/**
- * Try take the EMT/IOM lock, wait in ring-3 return VERR_SEM_BUSY in R0/RC.
- *
- * @retval  VINF_SUCCESS on success (always in ring-3).
- * @retval  VERR_SEM_BUSY in RC and R0 if the semaphore is busy.
- *
- * @param   pVM         VM handle.
- */
-int iomLock(PVM pVM)
-{
-    Assert(pVM->cCpus == 1 || !PGMIsLockOwner(pVM));
-    int rc = PDMCritSectEnter(&pVM->iom.s.EmtLock, VERR_SEM_BUSY);
-    return rc;
-}
-
-
-/**
- * Try take the EMT/IOM lock, no waiting.
- *
- * @retval  VINF_SUCCESS on success.
- * @retval  VERR_SEM_BUSY if busy.
- *
- * @param   pVM         VM handle.
- */
-int iomTryLock(PVM pVM)
-{
-    int rc = PDMCritSectTryEnter(&pVM->iom.s.EmtLock);
-    return rc;
-}
-
-
-/**
- * Release EMT/IOM lock.
- *
- * @param   pVM         VM handle.
- */
-void iomUnlock(PVM pVM)
-{
-    PDMCritSectLeave(&pVM->iom.s.EmtLock);
-}
+#include "IOMInline.h"
 
 
 /**
@@ -86,9 +47,10 @@ void iomUnlock(PVM pVM)
  */
 VMMDECL(bool) IOMIsLockOwner(PVM pVM)
 {
-    return PDMCritSectIsOwner(&pVM->iom.s.EmtLock);
+    return PDMCritSectIsOwner(&pVM->iom.s.CritSect);
 }
 
+
 /**
  * Returns the contents of register or immediate data of instruction's parameter.
  *
@@ -257,13 +219,18 @@ bool iomSaveDataToReg(PDISCPUSTATE pCpu, PCOP_PARAMETER pParam, PCPUMCTXCORE pRe
  */
 VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t cbValue)
 {
+/** @todo should initialize *pu32Value here because it can happen that some
+ *        handle is buggy and doesn't handle all cases. */
     /* Take the IOM lock before performing any device I/O. */
-    int rc2 = iomLock(pVM);
+    int rc2 = IOM_LOCK(pVM);
 #ifndef IN_RING3
     if (rc2 == VERR_SEM_BUSY)
         return VINF_IOM_HC_IOPORT_READ;
 #endif
     AssertRC(rc2);
+#if defined(IEM_VERIFICATION_MODE) && defined(IN_RING3)
+    IEMNotifyIOPortRead(pVM, Port, cbValue);
+#endif
 
 #ifdef VBOX_WITH_STATISTICS
     /*
@@ -285,7 +252,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value,
     if (    !pRange
         ||   (unsigned)Port - (unsigned)pRange->Port >= (unsigned)pRange->cPorts)
     {
-        pRange = iomIOPortGetRange(&pVM->iom.s, Port);
+        pRange = iomIOPortGetRange(pVM, Port);
         if (pRange)
             pVM->iom.s.CTX_SUFF(pRangeLastRead) = pRange;
     }
@@ -300,52 +267,35 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value,
         if (!pfnInCallback)
         {
             STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
-            iomUnlock(pVM);
+            IOM_UNLOCK(pVM);
             return VINF_IOM_HC_IOPORT_READ;
         }
 #endif
         void           *pvUser    = pRange->pvUser;
         PPDMDEVINS      pDevIns   = pRange->pDevIns;
-        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
+        IOM_UNLOCK(pVM);
 
         /*
-         * Call the device - 4 variations.
+         * Call the device.
          */
-        VBOXSTRICTRC    rcStrict;
-        if (pCritSect)
+        VBOXSTRICTRC rcStrict = PDMCritSectEnter(pDevIns->CTX_SUFF(pCritSectRo), VINF_IOM_HC_IOPORT_READ);
+        if (rcStrict != VINF_SUCCESS)
         {
-            iomUnlock(pVM);
-            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_READ);
-            if (rcStrict != VINF_SUCCESS)
-            {
-                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
-                return rcStrict;
-            }
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
+            return rcStrict;
+        }
 #ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
-                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
-            }
-            else
-#endif
-                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
-            PDMCritSectLeave(pCritSect);
+        if (pStats)
+        {
+            STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
+            rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
+            STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
         }
         else
-        {
-#ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
-                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
-            }
-            else
 #endif
-                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
-        }
+            rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
+        PDMCritSectLeave(pDevIns->CTX_SUFF(pCritSectRo));
+
 #ifdef VBOX_WITH_STATISTICS
         if (rcStrict == VINF_SUCCESS && pStats)
             STAM_COUNTER_INC(&pStats->CTX_SUFF_Z(In));
@@ -365,14 +315,10 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value,
                 case 4: *(uint32_t *)pu32Value = UINT32_C(0xffffffff); break;
                 default:
                     AssertMsgFailed(("Invalid I/O port size %d. Port=%d\n", cbValue, Port));
-                    if (!pCritSect)
-                        iomUnlock(pVM);
                     return VERR_IOM_INVALID_IOPORT_SIZE;
             }
         }
         Log3(("IOMIOPortRead: Port=%RTiop *pu32=%08RX32 cb=%d rc=%Rrc\n", Port, *pu32Value, cbValue, VBOXSTRICTRC_VAL(rcStrict)));
-        if (!pCritSect)
-            iomUnlock(pVM);
         return rcStrict;
     }
 
@@ -380,14 +326,14 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value,
     /*
      * Handler in ring-3?
      */
-    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(&pVM->iom.s, Port);
+    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(pVM, Port);
     if (pRangeR3)
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
             STAM_COUNTER_INC(&pStats->InRZToR3);
 # endif
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_READ;
     }
 #endif
@@ -402,7 +348,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value,
     {
 # ifndef IN_RING3
         /* Ring-3 will have to create the statistics record. */
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_READ;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -420,11 +366,11 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value,
         case 4: *(uint32_t *)pu32Value = UINT32_C(0xffffffff); break;
         default:
             AssertMsgFailed(("Invalid I/O port size %d. Port=%d\n", cbValue, Port));
-            iomUnlock(pVM);
+            IOM_UNLOCK(pVM);
             return VERR_IOM_INVALID_IOPORT_SIZE;
     }
     Log3(("IOMIOPortRead: Port=%RTiop *pu32=%08RX32 cb=%d rc=VINF_SUCCESS\n", Port, *pu32Value, cbValue));
-    iomUnlock(pVM);
+    IOM_UNLOCK(pVM);
     return VINF_SUCCESS;
 }
 
@@ -448,12 +394,15 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value,
 VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRTGCUINTREG pcTransfers, unsigned cb)
 {
     /* Take the IOM lock before performing any device I/O. */
-    int rc2 = iomLock(pVM);
+    int rc2 = IOM_LOCK(pVM);
 #ifndef IN_RING3
     if (rc2 == VERR_SEM_BUSY)
         return VINF_IOM_HC_IOPORT_READ;
 #endif
     AssertRC(rc2);
+#if defined(IEM_VERIFICATION_MODE) && defined(IN_RING3)
+    IEMNotifyIOPortReadString(pVM, Port, *pGCPtrDst, *pcTransfers, cb);
+#endif
 
 #ifdef LOG_ENABLED
     const RTGCUINTREG cTransfers = *pcTransfers;
@@ -478,7 +427,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPt
     if (    !pRange
         ||   (unsigned)Port - (unsigned)pRange->Port >= (unsigned)pRange->cPorts)
     {
-        pRange = iomIOPortGetRange(&pVM->iom.s, Port);
+        pRange = iomIOPortGetRange(pVM, Port);
         if (pRange)
             pVM->iom.s.CTX_SUFF(pRangeLastRead) = pRange;
     }
@@ -493,52 +442,34 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPt
         if (!pfnInStrCallback)
         {
             STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
-            iomUnlock(pVM);
+            IOM_UNLOCK(pVM);
             return VINF_IOM_HC_IOPORT_READ;
         }
 #endif
         void           *pvUser    = pRange->pvUser;
         PPDMDEVINS      pDevIns   = pRange->pDevIns;
-        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
+        IOM_UNLOCK(pVM);
 
         /*
-         * Call the device - 4 variations.
+         * Call the device.
          */
-        VBOXSTRICTRC    rcStrict;
-        if (pCritSect)
+        VBOXSTRICTRC rcStrict = PDMCritSectEnter(pDevIns->CTX_SUFF(pCritSectRo), VINF_IOM_HC_IOPORT_READ);
+        if (rcStrict != VINF_SUCCESS)
         {
-            iomUnlock(pVM);
-            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_READ);
-            if (rcStrict != VINF_SUCCESS)
-            {
-                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
-                return rcStrict;
-            }
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
+            return rcStrict;
+        }
 #ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
-                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
-            }
-            else
-#endif
-                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
-            PDMCritSectLeave(pCritSect);
+        if (pStats)
+        {
+            STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
+            rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
+            STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
         }
         else
-        {
-#ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
-                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
-            }
-            else
 #endif
-                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
-        }
+            rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
+        PDMCritSectLeave(pDevIns->CTX_SUFF(pCritSectRo));
 
 #ifdef VBOX_WITH_STATISTICS
         if (rcStrict == VINF_SUCCESS && pStats)
@@ -550,8 +481,6 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPt
 #endif
         Log3(("IOMIOPortReadStr: Port=%RTiop pGCPtrDst=%p pcTransfer=%p:{%#x->%#x} cb=%d rc=%Rrc\n",
               Port, pGCPtrDst, pcTransfers, cTransfers, *pcTransfers, cb, VBOXSTRICTRC_VAL(rcStrict)));
-        if (!pCritSect)
-            iomUnlock(pVM);
         return rcStrict;
     }
 
@@ -559,14 +488,14 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPt
     /*
      * Handler in ring-3?
      */
-    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(&pVM->iom.s, Port);
+    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(pVM, Port);
     if (pRangeR3)
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
             STAM_COUNTER_INC(&pStats->InRZToR3);
 # endif
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_READ;
     }
 #endif
@@ -581,7 +510,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPt
     {
 # ifndef IN_RING3
         /* Ring-3 will have to create the statistics record. */
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_READ;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -593,7 +522,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPt
 
     Log3(("IOMIOPortReadStr: Port=%RTiop pGCPtrDst=%p pcTransfer=%p:{%#x->%#x} cb=%d rc=VINF_SUCCESS\n",
           Port, pGCPtrDst, pcTransfers, cTransfers, *pcTransfers, cb));
-    iomUnlock(pVM);
+    IOM_UNLOCK(pVM);
     return VINF_SUCCESS;
 }
 
@@ -616,12 +545,15 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPt
 VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cbValue)
 {
     /* Take the IOM lock before performing any device I/O. */
-    int rc2 = iomLock(pVM);
+    int rc2 = IOM_LOCK(pVM);
 #ifndef IN_RING3
     if (rc2 == VERR_SEM_BUSY)
         return VINF_IOM_HC_IOPORT_WRITE;
 #endif
     AssertRC(rc2);
+#if defined(IEM_VERIFICATION_MODE) && defined(IN_RING3)
+    IEMNotifyIOPortWrite(pVM, Port, u32Value, cbValue);
+#endif
 
 /** @todo bird: When I get time, I'll remove the RC/R0 trees and link the RC/R0
  *        entries to the ring-3 node. */
@@ -645,7 +577,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value,
     if (    !pRange
         ||   (unsigned)Port - (unsigned)pRange->Port >= (unsigned)pRange->cPorts)
     {
-        pRange = iomIOPortGetRange(&pVM->iom.s, Port);
+        pRange = iomIOPortGetRange(pVM, Port);
         if (pRange)
             pVM->iom.s.CTX_SUFF(pRangeLastWrite) = pRange;
     }
@@ -660,52 +592,34 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value,
         if (!pfnOutCallback)
         {
             STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
-            iomUnlock(pVM);
+            IOM_UNLOCK(pVM);
             return VINF_IOM_HC_IOPORT_WRITE;
         }
 #endif
-        void           *pvUser    =  pRange->pvUser;
+        void           *pvUser    = pRange->pvUser;
         PPDMDEVINS      pDevIns   = pRange->pDevIns;
-        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
+        IOM_UNLOCK(pVM);
 
         /*
-         * Call the device - 4 variations.
+         * Call the device.
          */
-        VBOXSTRICTRC    rcStrict;
-        if (pCritSect)
+        VBOXSTRICTRC rcStrict = PDMCritSectEnter(pDevIns->CTX_SUFF(pCritSectRo), VINF_IOM_HC_IOPORT_WRITE);
+        if (rcStrict != VINF_SUCCESS)
         {
-            iomUnlock(pVM);
-            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_WRITE);
-            if (rcStrict != VINF_SUCCESS)
-            {
-                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
-                return rcStrict;
-            }
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
+            return rcStrict;
+        }
 #ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
-                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
-            }
-            else
-#endif
-                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
-            PDMCritSectLeave(pCritSect);
+        if (pStats)
+        {
+            STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+            rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
+            STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
         }
         else
-        {
-#ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
-                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
-            }
-            else
 #endif
-                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
-        }
+            rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
+        PDMCritSectLeave(pDevIns->CTX_SUFF(pCritSectRo));
 
 #ifdef VBOX_WITH_STATISTICS
         if (rcStrict == VINF_SUCCESS && pStats)
@@ -716,8 +630,6 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value,
 # endif
 #endif
         Log3(("IOMIOPortWrite: Port=%RTiop u32=%08RX32 cb=%d rc=%Rrc\n", Port, u32Value, cbValue, VBOXSTRICTRC_VAL(rcStrict)));
-        if (!pCritSect)
-            iomUnlock(pVM);
         return rcStrict;
     }
 
@@ -725,14 +637,14 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value,
     /*
      * Handler in ring-3?
      */
-    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(&pVM->iom.s, Port);
+    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(pVM, Port);
     if (pRangeR3)
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
             STAM_COUNTER_INC(&pStats->OutRZToR3);
 # endif
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
     }
 #endif
@@ -748,7 +660,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value,
     {
 # ifndef IN_RING3
         /* R3 will have to create the statistics record. */
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -758,7 +670,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value,
     }
 #endif
     Log3(("IOMIOPortWrite: Port=%RTiop u32=%08RX32 cb=%d nop\n", Port, u32Value, cbValue));
-    iomUnlock(pVM);
+    IOM_UNLOCK(pVM);
     return VINF_SUCCESS;
 }
 
@@ -782,12 +694,15 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value,
 VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PRTGCUINTREG pcTransfers, unsigned cb)
 {
     /* Take the IOM lock before performing any device I/O. */
-    int rc2 = iomLock(pVM);
+    int rc2 = IOM_LOCK(pVM);
 #ifndef IN_RING3
     if (rc2 == VERR_SEM_BUSY)
         return VINF_IOM_HC_IOPORT_WRITE;
 #endif
     AssertRC(rc2);
+#if defined(IEM_VERIFICATION_MODE) && defined(IN_RING3)
+    IEMNotifyIOPortWriteString(pVM, Port, *pGCPtrSrc, *pcTransfers, cb);
+#endif
 
 #ifdef LOG_ENABLED
     const RTGCUINTREG cTransfers = *pcTransfers;
@@ -812,7 +727,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCP
     if (    !pRange
         ||   (unsigned)Port - (unsigned)pRange->Port >= (unsigned)pRange->cPorts)
     {
-        pRange = iomIOPortGetRange(&pVM->iom.s, Port);
+        pRange = iomIOPortGetRange(pVM, Port);
         if (pRange)
             pVM->iom.s.CTX_SUFF(pRangeLastWrite) = pRange;
     }
@@ -827,52 +742,34 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCP
         if (!pfnOutStrCallback)
         {
             STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
-            iomUnlock(pVM);
+            IOM_UNLOCK(pVM);
             return VINF_IOM_HC_IOPORT_WRITE;
         }
 #endif
         void           *pvUser    = pRange->pvUser;
         PPDMDEVINS      pDevIns   = pRange->pDevIns;
-        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
+        IOM_UNLOCK(pVM);
 
         /*
-         * Call the device - 4 variations.
+         * Call the device.
          */
-        VBOXSTRICTRC    rcStrict;
-        if (pCritSect)
+        VBOXSTRICTRC rcStrict = PDMCritSectEnter(pDevIns->CTX_SUFF(pCritSectRo), VINF_IOM_HC_IOPORT_WRITE);
+        if (rcStrict != VINF_SUCCESS)
         {
-            iomUnlock(pVM);
-            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_WRITE);
-            if (rcStrict != VINF_SUCCESS)
-            {
-                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
-                return rcStrict;
-            }
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
+            return rcStrict;
+        }
 #ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
-                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
-            }
-            else
-#endif
-                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
-            PDMCritSectLeave(pCritSect);
+        if (pStats)
+        {
+            STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+            rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
+            STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
         }
         else
-        {
-#ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-            {
-                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
-                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
-                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
-            }
-            else
 #endif
-                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
-        }
+            rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
+        PDMCritSectLeave(pDevIns->CTX_SUFF(pCritSectRo));
 
 #ifdef VBOX_WITH_STATISTICS
         if (rcStrict == VINF_SUCCESS && pStats)
@@ -884,8 +781,6 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCP
 #endif
         Log3(("IOMIOPortWriteStr: Port=%RTiop pGCPtrSrc=%p pcTransfer=%p:{%#x->%#x} cb=%d rcStrict=%Rrc\n",
               Port, pGCPtrSrc, pcTransfers, cTransfers, *pcTransfers, cb, VBOXSTRICTRC_VAL(rcStrict)));
-        if (!pCritSect)
-            iomUnlock(pVM);
         return rcStrict;
     }
 
@@ -893,14 +788,14 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCP
     /*
      * Handler in ring-3?
      */
-    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(&pVM->iom.s, Port);
+    PIOMIOPORTRANGER3 pRangeR3 = iomIOPortGetRangeR3(pVM, Port);
     if (pRangeR3)
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
             STAM_COUNTER_INC(&pStats->OutRZToR3);
 # endif
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
     }
 #endif
@@ -915,7 +810,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCP
     {
 # ifndef IN_RING3
         /* Ring-3 will have to create the statistics record. */
-        iomUnlock(pVM);
+        IOM_UNLOCK(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -927,7 +822,7 @@ VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCP
 
     Log3(("IOMIOPortWriteStr: Port=%RTiop pGCPtrSrc=%p pcTransfer=%p:{%#x->%#x} cb=%d rc=VINF_SUCCESS\n",
           Port, pGCPtrSrc, pcTransfers, cTransfers, *pcTransfers, cb));
-    iomUnlock(pVM);
+    IOM_UNLOCK(pVM);
     return VINF_SUCCESS;
 }
 
@@ -1140,3 +1035,16 @@ VMMDECL(VBOXSTRICTRC) IOMInterpretOUT(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUST
         AssertMsg(rcStrict == VINF_EM_RAW_GUEST_TRAP || rcStrict == VINF_TRPM_XCPT_DISPATCHED || rcStrict == VINF_TRPM_XCPT_DISPATCHED || RT_FAILURE(rcStrict), ("%Rrc\n", VBOXSTRICTRC_VAL(rcStrict)));
     return rcStrict;
 }
+
+
+/**
+ * Fress an MMIO range after the reference counter has become zero.
+ *
+ * @param   pVM                 The VM handle.
+ * @param   pRange              The range to free.
+ */
+void iomMmioFreeRange(PVM pVM, PIOMMMIORANGE pRange)
+{
+    MMHyperFree(pVM, pRange);
+}
+
