@@ -1,4 +1,4 @@
-/* $Id: PGMAllMap.cpp $ */
+/* $Id: PGMAllMap.cpp 38325 2011-08-05 14:11:58Z vboxsync $ */
 /** @file
  * PGM - Page Manager and Monitor - All context code.
  */
@@ -20,6 +20,7 @@
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_PGM
 #include <VBox/vmm/pgm.h>
+#include <VBox/vmm/em.h>
 #include "PGMInternal.h"
 #include <VBox/vmm/vm.h>
 #include "PGMInline.h"
@@ -269,9 +270,9 @@ VMMDECL(int) PGMMapGetPage(PVM pVM, RTGCPTR GCPtr, uint64_t *pfFlags, PRTHCPHYS
  */
 void pgmMapSetShadowPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iNewPDE)
 {
-    Log4(("pgmMapSetShadowPDEs new pde %x (mappings enabled %d)\n", iNewPDE, pgmMapAreMappingsEnabled(&pVM->pgm.s)));
+    Log4(("pgmMapSetShadowPDEs new pde %x (mappings enabled %d)\n", iNewPDE, pgmMapAreMappingsEnabled(pVM)));
 
-    if (    !pgmMapAreMappingsEnabled(&pVM->pgm.s)
+    if (    !pgmMapAreMappingsEnabled(pVM)
         ||  pVM->cCpus > 1)
         return;
 
@@ -353,7 +354,7 @@ void pgmMapSetShadowPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iNewPDE)
                  */
                 PPGMPOOLPAGE    pPoolPagePd = pgmPoolGetPage(pPool, pShwPdpt->a[iPdPt].u & X86_PDPE_PG_MASK);
                 AssertFatal(pPoolPagePd);
-                if (!pgmPoolIsPageLocked(&pVM->pgm.s, pPoolPagePd))
+                if (!pgmPoolIsPageLocked(pPoolPagePd))
                     pgmPoolLockPage(pPool, pPoolPagePd);
 #ifdef VBOX_STRICT
                 else if (pShwPaePd->a[iPaePde].u & PGM_PDFLAGS_MAPPING)
@@ -423,12 +424,12 @@ void pgmMapSetShadowPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iNewPDE)
  */
 void pgmMapClearShadowPDEs(PVM pVM, PPGMPOOLPAGE pShwPageCR3, PPGMMAPPING pMap, unsigned iOldPDE, bool fDeactivateCR3)
 {
-    Log(("pgmMapClearShadowPDEs: old pde %x (cPTs=%x) (mappings enabled %d) fDeactivateCR3=%RTbool\n", iOldPDE, pMap->cPTs, pgmMapAreMappingsEnabled(&pVM->pgm.s), fDeactivateCR3));
+    Log(("pgmMapClearShadowPDEs: old pde %x (cPTs=%x) (mappings enabled %d) fDeactivateCR3=%RTbool\n", iOldPDE, pMap->cPTs, pgmMapAreMappingsEnabled(pVM), fDeactivateCR3));
 
     /*
      * Skip this if disabled or if it doesn't apply.
      */
-    if (    !pgmMapAreMappingsEnabled(&pVM->pgm.s)
+    if (    !pgmMapAreMappingsEnabled(pVM)
         ||  pVM->cCpus > 1)
         return;
 
@@ -525,7 +526,7 @@ void pgmMapClearShadowPDEs(PVM pVM, PPGMPOOLPAGE pShwPageCR3, PPGMMAPPING pMap,
                 {
                     PPGMPOOLPAGE pPoolPagePd = pgmPoolGetPage(pPool, pShwPdpt->a[iPdpt].u & X86_PDPE_PG_MASK);
                     AssertFatal(pPoolPagePd);
-                    if (pgmPoolIsPageLocked(&pVM->pgm.s, pPoolPagePd))
+                    if (pgmPoolIsPageLocked(pPoolPagePd))
                         pgmPoolUnlockPage(pPool, pPoolPagePd);
                 }
                 break;
@@ -629,7 +630,7 @@ VMMDECL(void) PGMMapCheck(PVM pVM)
     /*
      * Can skip this if mappings are disabled.
      */
-    if (!pgmMapAreMappingsEnabled(&pVM->pgm.s))
+    if (!pgmMapAreMappingsEnabled(pVM))
         return;
 
     /* This only applies to raw mode where we only support 1 VCPU. */
@@ -664,7 +665,7 @@ int pgmMapActivateCR3(PVM pVM, PPGMPOOLPAGE pShwPageCR3)
     /*
      * Skip this if disabled or if it doesn't apply.
      */
-    if (    !pgmMapAreMappingsEnabled(&pVM->pgm.s)
+    if (    !pgmMapAreMappingsEnabled(pVM)
         ||  pVM->cCpus > 1)
         return VINF_SUCCESS;
 
@@ -701,7 +702,7 @@ int pgmMapDeactivateCR3(PVM pVM, PPGMPOOLPAGE pShwPageCR3)
     /*
      * Skip this if disabled or if it doesn't apply.
      */
-    if (    !pgmMapAreMappingsEnabled(&pVM->pgm.s)
+    if (    !pgmMapAreMappingsEnabled(pVM)
         ||  pVM->cCpus > 1)
         return VINF_SUCCESS;
 
@@ -732,7 +733,7 @@ VMMDECL(bool) PGMMapHasConflicts(PVM pVM)
     /*
      * Can skip this if mappings are safely fixed.
      */
-    if (!pgmMapAreMappingsFloating(&pVM->pgm.s))
+    if (!pgmMapAreMappingsFloating(pVM))
         return false;
 
     Assert(pVM->cCpus == 1);
@@ -760,7 +761,7 @@ VMMDECL(bool) PGMMapHasConflicts(PVM pVM)
             unsigned iPT = pCur->cPTs;
             while (iPT-- > 0)
                 if (    pPD->a[iPDE + iPT].n.u1Present /** @todo PGMGstGetPDE. */
-                    &&  (pVM->fRawR0Enabled || pPD->a[iPDE + iPT].n.u1User))
+                    &&  (EMIsRawRing0Enabled(pVM) || pPD->a[iPDE + iPT].n.u1User))
                 {
                     STAM_COUNTER_INC(&pVM->pgm.s.CTX_SUFF(pStats)->StatR3DetectedConflicts);
 
@@ -792,7 +793,7 @@ VMMDECL(bool) PGMMapHasConflicts(PVM pVM)
                 X86PDEPAE Pde = pgmGstGetPaePDE(pVCpu, GCPtr);
 
                 if (   Pde.n.u1Present
-                    && (pVM->fRawR0Enabled || Pde.n.u1User))
+                    && (EMIsRawRing0Enabled(pVM) || Pde.n.u1User))
                 {
                     STAM_COUNTER_INC(&pVM->pgm.s.CTX_SUFF(pStats)->StatR3DetectedConflicts);
 #ifdef IN_RING3
@@ -854,7 +855,7 @@ int pgmMapResolveConflicts(PVM pVM)
             while (iPT-- > 0)
             {
                 if (    pPD->a[iPDE + iPT].n.u1Present /** @todo PGMGstGetPDE. */
-                    &&  (   pVM->fRawR0Enabled
+                    &&  (   EMIsRawRing0Enabled(pVM)
                          || pPD->a[iPDE + iPT].n.u1User))
                 {
                     STAM_COUNTER_INC(&pVM->pgm.s.CTX_SUFF(pStats)->StatR3DetectedConflicts);
@@ -895,7 +896,7 @@ int pgmMapResolveConflicts(PVM pVM)
                 X86PDEPAE Pde = pgmGstGetPaePDE(pVCpu, GCPtr);
 
                 if (   Pde.n.u1Present
-                    && (pVM->fRawR0Enabled || Pde.n.u1User))
+                    && (EMIsRawRing0Enabled(pVM) || Pde.n.u1User))
                 {
                     STAM_COUNTER_INC(&pVM->pgm.s.CTX_SUFF(pStats)->StatR3DetectedConflicts);
 #ifdef IN_RING3
