@@ -1,4 +1,4 @@
-/* $Id: PDMDevice.cpp $ */
+/* $Id: PDMDevice.cpp 37812 2011-07-07 09:56:42Z vboxsync $ */
 /** @file
  * PDM - Pluggable Device and Driver Manager, Device parts.
  */
@@ -191,7 +191,7 @@ int pdmR3DevInit(PVM pVM)
 
         /* Find the device. */
         PPDMDEV pDev = pdmR3DevLookup(pVM, szName);
-        AssertMsgReturn(pDev, ("Configuration error: device '%s' not found!\n", szName), VERR_PDM_DEVICE_NOT_FOUND);
+        AssertLogRelMsgReturn(pDev, ("Configuration error: device '%s' not found!\n", szName), VERR_PDM_DEVICE_NOT_FOUND);
 
         /* Configured priority or use default based on device class? */
         uint32_t u32Order;
@@ -287,7 +287,7 @@ int pdmR3DevInit(PVM pVM)
         CFGMR3SetRestrictedRoot(pConfigNode);
 
         /*
-         * Allocate the device instance.
+         * Allocate the device instance and critical section.
          */
         AssertReturn(paDevs[i].pDev->cInstances < paDevs[i].pDev->pReg->cMaxInstances, VERR_PDM_TOO_MANY_DEVICE_INSTANCES);
         size_t cb = RT_OFFSETOF(PDMDEVINS, achInstanceData[paDevs[i].pDev->pReg->cbInstance]);
@@ -297,12 +297,16 @@ int pdmR3DevInit(PVM pVM)
             rc = MMR3HyperAllocOnceNoRel(pVM, cb, 0, MM_TAG_PDM_DEVICE, (void **)&pDevIns);
         else
             rc = MMR3HeapAllocZEx(pVM, MM_TAG_PDM_DEVICE, cb, (void **)&pDevIns);
-        if (RT_FAILURE(rc))
-        {
-            AssertMsgFailed(("Failed to allocate %d bytes of instance data for device '%s'. rc=%Rrc\n",
-                             cb, paDevs[i].pDev->pReg->szName, rc));
-            return rc;
-        }
+        AssertLogRelMsgRCReturn(rc,
+                                ("Failed to allocate %d bytes of instance data for device '%s'. rc=%Rrc\n",
+                                cb, paDevs[i].pDev->pReg->szName, rc),
+                                rc);
+        PPDMCRITSECT pCritSect;
+        if (paDevs[i].pDev->pReg->fFlags & (PDM_DEVREG_FLAGS_RC | PDM_DEVREG_FLAGS_R0))
+            rc = MMHyperAlloc(pVM, sizeof(*pCritSect), 0, MM_TAG_PDM_DEVICE, (void **)&pCritSect);
+        else
+            rc = MMR3HeapAllocZEx(pVM, MM_TAG_PDM_DEVICE, sizeof(*pCritSect), (void **)&pCritSect);
+        AssertLogRelMsgRCReturn(rc, ("Failed to allocate a critical section for the device\n",  rc), rc);
 
         /*
          * Initialize it.
@@ -334,9 +338,16 @@ int pdmR3DevInit(PVM pVM)
                                                 ? MMHyperR3ToRC(pVM, pDevIns->pvInstanceDataR3) : NIL_RTRCPTR;
         pDevIns->pvInstanceDataR0               = pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_R0
                                                 ? MMHyperR3ToR0(pVM, pDevIns->pvInstanceDataR3) : NIL_RTR0PTR;
-        //pDevIns->pCritSectR3                    = NULL;
-        //pDevIns->pCritSectR0                    = NIL_RTR0PTR;
-        //pDevIns->pCritSectRC                    = NIL_RTRCPTR;
+
+        pDevIns->pCritSectRoR3                  = pCritSect;
+        pDevIns->pCritSectRoRC                  = pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_RC
+                                                ? MMHyperR3ToRC(pVM, pCritSect) : NIL_RTRCPTR;
+        pDevIns->pCritSectRoR0                  = pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_R0
+                                                ? MMHyperR3ToR0(pVM, pCritSect) : NIL_RTR0PTR;
+
+        rc = pdmR3CritSectInitDeviceAuto(pVM, pDevIns, pCritSect, RT_SRC_POS,
+                                         "%s#%u Auto", pDevIns->pReg->szName, pDevIns->iInstance);
+        AssertLogRelRCReturn(rc, rc);
 
         /*
          * Link it into all the lists.
@@ -369,41 +380,13 @@ int pdmR3DevInit(PVM pVM)
         paDevs[i].pDev->cInstances++;
         Log(("PDM: Constructing device '%s' instance %d...\n", pDevIns->pReg->szName, pDevIns->iInstance));
         rc = pDevIns->pReg->pfnConstruct(pDevIns, pDevIns->iInstance, pDevIns->pCfg);
-        if (RT_SUCCESS(rc))
-        {
-            /*
-             * Per-device critsect fun.
-             */
-            if (pDevIns->pCritSectR3)
-            {
-                AssertStmt(PDMCritSectIsInitialized(pDevIns->pCritSectR3), rc = VERR_INTERNAL_ERROR_4);
-                if (pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_R0)
-                {
-                    pDevIns->pCritSectR0 = MMHyperCCToR0(pVM, pDevIns->pCritSectR3);
-                    AssertStmt(pDevIns->pCritSectR0 != NIL_RTR0PTR, rc = VERR_INTERNAL_ERROR_3);
-                }
-                else
-                    AssertStmt(pDevIns->pCritSectR0 == NIL_RTRCPTR, rc = VERR_INTERNAL_ERROR_2);
-
-                if (pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_RC)
-                {
-                    pDevIns->pCritSectRC = MMHyperCCToRC(pVM, pDevIns->pCritSectR3);
-                    AssertStmt(pDevIns->pCritSectRC != NIL_RTRCPTR, rc = VERR_INTERNAL_ERROR_3);
-                }
-                else
-                    AssertStmt(pDevIns->pCritSectRC == NIL_RTRCPTR, rc = VERR_INTERNAL_ERROR_2);
-            }
-            else
-                AssertStmt(   pDevIns->pCritSectRC == NIL_RTRCPTR
-                           && pDevIns->pCritSectR0 == NIL_RTR0PTR,
-                           rc = VERR_INTERNAL_ERROR_5);
-        }
         if (RT_FAILURE(rc))
         {
             LogRel(("PDM: Failed to construct '%s'/%d! %Rra\n", pDevIns->pReg->szName, pDevIns->iInstance, rc));
             paDevs[i].pDev->cInstances--;
-            /* because we're damn lazy right now, we'll say that the destructor will be called even if the constructor fails. */
-            return rc;
+            /* Because we're damn lazy, the destructor will be called even if
+               the constructor fails.  So, no unlinking. */
+            return rc == VERR_VERSION_MISMATCH ? VERR_PDM_DEVICE_VERSION_MISMATCH : rc;
         }
     } /* for device instances */
 
@@ -853,6 +836,30 @@ VMMR3DECL(int) PDMR3DeviceDetach(PVM pVM, const char *pszDevice, unsigned iInsta
 }
 
 
+/**
+ * References the critical section associated with a device for the use by a
+ * timer or similar created by the device.
+ *
+ * @returns Pointer to the critical section.
+ * @param   pVM             The VM handle.
+ * @param   pDevIns         The device instance in question.
+ *
+ * @internal
+ */
+VMMR3_INT_DECL(PPDMCRITSECT) PDMR3DevGetCritSect(PVM pVM, PPDMDEVINS pDevIns)
+{
+    VM_ASSERT_EMT(pVM);
+    VM_ASSERT_STATE(pVM, VMSTATE_CREATING);
+    AssertPtr(pDevIns);
+
+    PPDMCRITSECT pCritSect = pDevIns->pCritSectRoR3;
+    AssertPtr(pCritSect);
+    pCritSect->s.fUsedByTimerOrSimilar = true;
+
+    return pCritSect;
+}
+
+
 /**
  * Attaches a preconfigured driver to an existing device or driver instance.
  *
